/*******************************************************************************
+
+  AGD 1.2
+
+
+  AGDLayout.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         AGDLayout.h
//
//    author: Carsten Gutwenger
//    date:   july 1999
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_AGD_LAYOUT_H
#define _AGD_AGD_LAYOUT_H


#include <AGD/LayoutBuffer.h>


/*{\Manpage {AGDLayout} {} {Layouts} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{LayoutInterface}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5}{LayoutBuffer}
\arhook{6.5}{2}{7.5}
\hdclass{3}{0}{5}{AGDLayout}
\end{hierarchy}
}*/

class __exportAGD AGDLayout : public LayoutBuffer {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a layout interface that stores the
graphical attributes of a particular graph $G$. It can be used, e.g., if
the layout of $G$ shall not be visualized immediately.
}*/

public:

	/*{\Mcreation A 5.5}*/

	AGDLayout(const leda::graph& G) : LayoutBuffer(G), _width(G,0),
		_height(G,0) { }
	/*{\Mcreate creates an instances |\Mvar| of type |\Mname| initialized
	to a layout for graph $G$.}*/

	AGDLayout(const AGDLayout &l) : LayoutBuffer(l),
		_width(l._width), _height(l._height) { }
	/*{\Mcreate copy constructor.}*/

	~AGDLayout() { }


	/*{\Moperations}*/
	/*{\Mtext
	|\Mname| inherits the operations from its base class |LayoutInterface|
	(see Sect.~\ref{Layout Interfaces})
	}*/

	double get_width  (leda::node v) const {
		return _width[v];
	}

	void set_width  (leda::node v, double new_w) {
		_width[v] = new_w;
	}

	double get_height (leda::node v) const {
		return _height[v];
	}

	void set_height (leda::node v, double new_h) {
		_height[v] = new_h;
	}

private:
	leda::node_map<double> _width, _height;
};


#endif
