/*******************************************************************************
+
+  AGD 1.2
+
+
+  AgdKey.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                           AgdKey.h
//
//
//    author: Carsten Gutwenger
//    date:   octobre 1998
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_A_KEY_H
#define _AGD_A_KEY_H



#include <AGD/key.h>
#include <LEDA/core/string.h>
#include <LEDA/internal/std/iostream.h>



class Keymapper;

class __exportAGD AgdKeyStruct {

	friend class Keymapper;

private:
	leda::string str;
	int _id;

	//leda::list_item pos_in_list;

public:

	AgdKeyStruct(const AgdKeyStruct& K) : str(K.str), _id(K._id) { }
	AgdKeyStruct(const leda::string& s, int id) : str (s), _id (id) { }

	const leda::string& info() const {
		return str;
	}

	int index() const {
		return _id;
	}

	LEDA_MEMORY(AgdKeyStruct)
};


class KeySet;


/*{\Manpage {AgdKey} {} {Keys} {k} }*/

class __exportAGD AgdKey {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a key, which
stands for a string stored in a keymapper. If a key
is stored in a keymapper |mapper|, it is assigned a unique index,
which, e.g., can be used for realizing arrays indexed by keys in
|mapper|. We call a key a |nil| key
if it has no corresponding string (similar like a |nil| pointer).
}*/

	friend class Keymapper;
	friend class KeySet;

	AgdKeyStruct *keyp;

public:

	/*{\Mcreation}*/

	AgdKey() { keyp = nil; }
	/*{\Mcreate creates a |nil| key.}*/

	AgdKey(const AgdKey &k) { keyp = k.keyp; }
	AgdKey(AgdKeyStruct *k) { keyp = k; }

	AgdKey(key::Predefined k);
	/*{\Mcreate converts |k| into a key using the default keymapper
	of |AGD|.}*/

	AgdKey(const leda::string &str);
	/*{\Mcreate converts |str| into a key using the default keymapper
	of |AGD|.}*/


	/*{\Moperations}*/

	const leda::string& info() const {
		return keyp->info();
	}
	/*{\Mop returns the string corresponding to |\Mvar|.}*/

	int index() const {
		return keyp->index();
	}
	/*{\Mop returns the index corresponding to |\Mvar|.}*/

	bool valid() const {
		return keyp != nil;
	}
	/*{\Mop returns |true| iff |\Mvar| is not a |nil| key.}*/

	bool operator==(const AgdKey &k) const {
		return (keyp == k.keyp);
	}

	AgdKey &operator=(const AgdKey& k) {
		keyp = k.keyp;
		return *this;
	}

	AgdKey &operator=(AgdKeyStruct *k) {
		keyp = k;
		return *this;
	}

	LEDA_MEMORY(AgdKey)
};


#if defined(__GNUC__) && __GNUC__ < 3
LEDA_BEGIN_NAMESPACE
#endif


__exportAGD ostream& operator<< (ostream& os, const AgdKey& k);
/*{\Mbinopfunc writes |s| to the output stream |os| if $k$ points
to $(s,i)$.}*/

__exportAGD istream& operator>> (istream& is, AgdKey &);

/*
inline ostream& operator<< (ostream& os, const AgdKey& k) {
	os << (k.info());
	return os;
}


inline istream& operator>> (istream& is, AgdKey &) {
	return is;
}
*/


// overloaded compare function for keys

#if !defined(__GNUC__) || __GNUC__ > 2
LEDA_BEGIN_NAMESPACE
#endif

inline int compare (const AgdKey& k1, const AgdKey& k2)
{
	return k2.index() - k1.index();
	//return leda::compare (k1.index(), k2.index());
}
LEDA_END_NAMESPACE


#include <AGD/AGD.h>

#endif
