/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGGridLayoutModule.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                       CGGridLayoutModule.h
//
//    author: Dirk Luetke-Huettmann
//    date:   july 1999
//-------------------------------------------------------------------

#ifndef _AGD_CGGRID_LAYOUT_H
#define _AGD_CGGRID_LAYOUT_H

#include <AGD/AGDModuleOption.h>
#include <AGD/GridCoordinateMapper.h>
#include <AGD/CGGridLayout.h>
#include <AGD/CGLayoutModule.h>

/*{\Manpage {CGGridLayoutModule} {} {Cluster Graph Grid Layout Algorithms} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{AGDModule}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5.5}{CGLayoutModule}
\arhook{7}{2}{8}
\hdclass{2}{0}{7}{CGGridLayoutModule}
\end{hierarchy}
}*/

class __exportAGD CGGridLayoutModule : public CGLayoutModule {

/*{\Mdefinition
The class |\Mname| extends the standard interface of cluster graph layout
algorithms by specialized call functions for cluster graph grid layout algorithms.

\headerline{Exchangeable Modules}
Each cluster graph grid layout algorithm has a module option for its grid coordinate
mapper. Deriving classes must also copy the |mapper| option when
implementing the clone method.

\begin{options}
\item {\bf mapper}\\
a |GridCoordinateMapper| option used for translating grid coordinates
into real coordinates.

\begin{tabular}{ll}
guaranteed precondition: & $\emptyset$\\
required postcondition: & $\emptyset$\\
initial module: & |MaxAllMapper|
\end{tabular}
\end{options}
}*/

public:

  /*{\Minitialization L 5}*/
  
  CGGridLayoutModule ();
  /*{\Mcreate initializes a cluster graph grid layout module. Sets the
    cluster graph grid coordinate mapper to |MaxAllMapper|.}*/
  
  CGGridLayoutModule (const PreCond &P, const PostCond &Q,
		      const GridCoordinateMapper &M);
  /*{\Mcreate initializes a cluster graph grid layout module. Sets the
    cluster graph grid coordinate mapper to $M$.}*/
  
  ~CGGridLayoutModule() { }

  /*{\Moperations 1.5 3.5}*/

  /*{\Mtext       
    \headerline{Extended Standard Interface} }*/

  virtual void call_grid (const ClusterGraph& CG, 
			  CGGridLayout &cgl) = 0;
  /*{\Moppure calls |\Mvar| for cluster graph |CG|. Returns the cluster graph 
    grid layout in |cgl|.}*/
  
  virtual void call_grid (const ClusterGraph& CG, 
			  leda::face f, 
			  CGGridLayout &cgl) = 0;
  /*{\Moppure calls |\Mvar| for cluster graph |CG| and external face $f$. Returns
    the cluster graph grid layout in |cgl|. \precond $G$ represents a planar map, and
    $f$ is |nil| or a face in |CG|.}*/

  IRect bounding_box () const {
    return _grid_bbox;
  }

  AGD_DECLARE_MODULE(GridCoordinateMapper,mapper)

protected:

  IRect _grid_bbox;
};

#endif
