/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGGridLayoutModuleImpl.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      CGGridLayoutModuleImpl.h
//
//    author: Dirk Luetke-Huettmann
//    date:   may 1999
//-------------------------------------------------------------------


#ifndef _AGD_CGGLM_IMPL_H
#define _AGD_CGGLM_IMPL_H

#include <AGD/CGGridLayoutModule.h>
#include <AGD/CGLayoutInterface.h>
#include <AGD/CGGridLayout.h>
/*{\Manpage {CGGridLayoutModuleImpl} {GRID_CONCEPT} {Implementation of Clustergraph Grid Layout Algorithms} {L} }*/ 

/*{\Mbaseclasses

\begin{hierarchy}{11}{5.5}
\hdclass{4}{4.5}{5}{AGDModule}
\arhook{9}{5}{10}
\hdclass{4.5}{3}{6}{CGLayoutModule}
\arhook{10.5}{3.5}{11.5}
\hdclass{5}{1.5}{7}{CGGridLayoutModule}
\arhook{12}{2}{13}
\hdclass{0}{0}{14.5}{CGGridLayoutModuleImpl{\tt<}GRID\_CONCEPT{\tt>}}
\end{hierarchy}
}*/

template <class GRID_CONCEPT>
class __exportAGD CGGridLayoutModuleImpl : public CGGridLayoutModule {

  /*{\Mdefinition
    The class |\Mname| represents a generic method for implementing grid
    layout algorithms for clustergraphs. For a detailed description take 
    a look at the section for the |GridLayoutModuleImpl| class.
    }*/

public:	

  CGGridLayoutModuleImpl() { }
  ~CGGridLayoutModuleImpl() { }

  /*{\Moperations 1.5 4}*/
  /*{\Mtext |\Mname| implements the call() and call\_grid() methods
    of its base classes. Furthermore, there are the following protected
    overridables:}*/

  void call (const ClusterGraph& G, CGLayoutInterface& A) {
    GRID_CONCEPT C(G, A);
    get_node_grid_size(C,A);
    do_call(C);
    finish_call(C,A);
  }

  void call (const ClusterGraph& G, leda::face f, CGLayoutInterface& A) {
    GRID_CONCEPT C(G,f,A);
    get_node_grid_size(C,A);
    do_call(C);
    finish_call(C,A);
  }

  void call_grid (const ClusterGraph& G, CGGridLayout &gl) {
    GRID_CONCEPT C(G,gl);
    get_node_grid_size(C,gl);
    do_call(C);
    finish_call_grid(C,gl);
  }
  
  void call_grid (const ClusterGraph& G, leda::face f, CGGridLayout &gl) {
    GRID_CONCEPT C(G,f,gl);
    get_node_grid_size(C,gl);
    do_call(C);
    finish_call_grid(C,gl);
  }

protected:

	virtual void do_call (GRID_CONCEPT &C) = 0;
	/*{\Moppure implements the algorithm.}*/

	virtual void get_node_grid_size (GRID_CONCEPT &C, const LayoutInterface &A) {
		C.get_node_grid_size(mapper.get(),A);
	}
	/*{\Mop determines initial grid sizes of nodes. The default implementation
	calls C.get\_node\_grid\_size() with the current grid coordinate mapper.}*/

	virtual void get_node_grid_size (GRID_CONCEPT &C, const GridLayout &gl) {
		C.get_node_grid_size(gl);
	}
	/*{\Mop determines initial grid sizes of nodes. The grid sizes in |gl|
	represent the user's input. The default implementation
	calls C.get\_node\_grid\_size() with |gl|.}*/


	void finish_call(GRID_CONCEPT &C, LayoutInterface &A) {
		_grid_bbox = C.get_bbox();
		C.update(mapper.get(),A);
	}

	void finish_call_grid(GRID_CONCEPT &C, CGGridLayout &gl) {
		_grid_bbox = C.get_bbox();
		C.update(gl);
	}

};

#endif
