/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGLayout.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         CGLayout.h
//
//    author: Dirk Luetke-Huettmann
//    date:   september 1999
//-------------------------------------------------------------------


#ifndef _AGD_CG_LAYOUT
#define _AGD_CG_LAYOUT


#include <AGD/CGLayoutInterface.h>


/*{\Manpage {CGLayout} {} {Cluster Graph Layouts} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{LayoutInterface}
\arhook{5}{3.5}{6}
\hdclass{0.5}{1.5}{6}{CGLayoutInterface}
\arhook{6.5}{2}{7.5}
\hdclass{3}{0}{5}{CGLayout}
\end{hierarchy}
}*/

class __exportAGD CGLayout : public CGLayoutInterface {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a layout interface that stores the
graphical attributes of a particular clustergraph $CG$. It can be used, e.g., if
the layout of $CG$ shall not be visualized immediately.
}*/

public:

	/*{\Mcreation A 5.5}*/

  CGLayout(const ClusterGraph& CG) :
    CGLayoutInterface(CG),  _pos(CG), _bends(CG), 
    _s_anchor(CG), _t_anchor(CG), _width(CG,0), _height(CG,0) { }
  /*{\Mcreate creates an instances |\Mvar| of type |\Mname| initialized
    to a layout for graph $G$.}*/

  CGLayout(const CGLayout &l) : 
    CGLayoutInterface(l), _pos(l._pos), _bends(l._bends), 
    _s_anchor(l._s_anchor), _t_anchor(l._t_anchor),
    _width(l._width), _height(l._height) { }
  /*{\Mcreate copy constructor.}*/
  
  ~CGLayout() { }
  
  /*{\Moperations}*/
  /*{\Mtext
    |\Mname| inherits the operations from its base class
    |CGLayoutInterface | (see Sect.~\ref{Layout Interfaces})
    }*/
  
  double get_width  (leda::node v) const {
    return _width[v];
  }
  
  void set_width  (leda::node v, double new_w) {
    _width[v] = new_w;
  }
  
  double get_height (leda::node v) const {
    return _height[v];
  }
  
  void set_height (leda::node v, double new_h) {
    _height[v] = new_h;
  }
  
  
  double get_x (leda::node v) const {
    return _pos[v].xcoord();
  }
  
  void set_x (leda::node v, double new_x) {
    _pos[v].xcoord(new_x);
  }
  
  double get_y (leda::node v) const {
    return _pos[v].ycoord();
  }
  
  void set_y (leda::node v, double new_y) {
    _pos[v].ycoord(new_y);
  }
  
  DPoint get_position (leda::node v) const {
    return _pos[v];
  }
  
  void set_position (leda::node v, const DPoint& pos) {
    _pos[v] = pos;
  }
  
  DPolyline get_bend_points (leda::edge e) const {
    return _bends[e];
  }
  
  void set_bend_points (leda::edge e, const DPolyline& l) {
    _bends[e] = l;
  }
  
  DPoint get_source_anchor (leda::edge e) const {
    return _s_anchor[e];
  }
  
  void set_source_anchor (leda::edge e, const DPoint& pos) {
    _s_anchor[e] = pos;
  }
  
  DPoint get_target_anchor (leda::edge e) const {
    return _t_anchor[e];
  }
  
  void set_target_anchor (leda::edge e, const DPoint& pos) {
    _t_anchor[e] = pos;
  }
  
  void set_cluster_region(Cluster* cl, 
			  double x,
			  double y,
			  double width,
			  double height) {
    _cl_x[cl] = x;
    _cl_y[cl] = y;
    _cl_width[cl] = width;
    _cl_height[cl] = height;
  }

private:
	leda::node_map<DPoint> _pos;
	leda::edge_map<DPolyline> _bends;
	leda::edge_map<DPoint> _s_anchor, _t_anchor;
	leda::node_map<double> _width, _height;
	leda::map<Cluster*, double> _cl_x;
	leda::map<Cluster*, double> _cl_y;
	leda::map<Cluster*, double> _cl_width;
	leda::map<Cluster*, double> _cl_height;
};


#endif

