/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGLayoutInterface.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      CGLayoutInterface.h
//
//    author: Dirk Luetke-Huettmann
//    date:   June 1999
//-------------------------------------------------------------------

#ifndef _AGD_CLUSTER_GRAPH_LAYOUT_INTERFACE_H
#define _AGD_CLUSTER_GRAPH_LAYOUT_INTERFACE_H

#include <AGD/LayoutInterface.h>
#include <AGD/ClusterGraph.h>

/*{\Manpage {CGLayoutInterface} {} {Layout Interfaces for Cluster Graphs} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{2.5}
\hdclass{0}{1.5}{6}{LayoutInterface}
\arhook{6}{2}{7}
\hdclass{1}{0}{6.5}{CGLayoutInterface}
\end{hierarchy}
}*/

class __exportAGD CGLayoutInterface : public LayoutInterface {

/*{\Mdefinition
The class |\Mname| extends its base class |LayoutInterface| by
functionality for cluster graphs.
Our graphical model for cluster graphs contains the following
additional attribute:
\begin{itemize}
\item a rectangular region of type |DPoint| for each cluster |cl|,
which bounds the nodes contained in |cl|.
\end{itemize}
}*/

public:

    /*{\Minitialization A 6.5}*/
  
    CGLayoutInterface(const ClusterGraph& CG);
	/*{\Mcreate initializes a layout interface for cluster graph |CG|.}*/
	
	~CGLayoutInterface();

	/*{\Moperations}*/

	const ClusterGraph &get_graph() const {
		return *_cg_p;
	}
	/*{\Mop returns the associated cluster graph.}*/

	virtual void set_cluster_region(Cluster* cl, 
					double x,
					double y,
					double width,
					double height) = 0;
	/*{\Moppure sets the region of cluster |cl| to the rectangular 
	  polygon with lower left corner $(x,y)$ and upper right
	  corner $(x + width, y + height)$.}*/

protected:

	void set_graph(const ClusterGraph& CG) {
		_cg_p = &CG;
		LayoutInterface::set_graph(CG);
	}
	/*{\Mop protected method for setting the associated cluster
	graph to |CG|.}*/

	  
private:
	const ClusterGraph* _cg_p;

};

#endif
