/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGLayoutModule.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         CGLayoutModule.h
//
//
//    author: Dirk Luetke-Huettmann
//    date:   nov 1998
//-------------------------------------------------------------------


#ifndef _AGD_CG_LAYOUT_MODULE_H
#define _AGD_CG_LAYOUT_MODULE_H

#include <AGD/CGLayoutInterface.h>
#include <AGD/AGDModule.h>
#include <AGD/ClusterGraph.h>

class LayoutInterface;
class PlaneGraphCopy;

/*{\Manpage {CGLayoutModule} {} {Cluster Graph Layout Algorithms} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{AGDModule}
\arhook{5}{2}{6}
\hdclass{1.0}{0}{6}{CGLayoutModule}
\end{hierarchy}
}*/

class __exportAGD CGLayoutModule : public AGDModule {
/*{\Mdefinition
The class |\Mname| defines the standard interface for layout
algorithms that draw cluster graphs. A cluster graph is a
graph with an additional cluster structure.

Such a layout algorithm |\Mvar| gets as input a cluster graph |CG|
and for each node $v$
of |CG| the size of the rectangle surrounding the graphics of $v$. |\Mvar|
computes for each node $v$ a point, which will be the center of the
graphics of $v$, for each edge a list of bend points with a
source and target anchor point, and for each cluster a rectangular bounding
region. Layout algorithms are also allowed to change
the size of the graphics of nodes.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & |in_graph=1| \\
output parameter: & layout & |out_layout=1|
\end{tabular}

The integer constants |in_graph| and |out_layout| denote the
positions in the input and output parameter lists. The values can be used to
reference the desired parameter when setting the pre- and postcondition.  
}*/

public:

  static const int in_graph; 
  static const int out_layout; 

  /*{\Minitialization}*/

  CGLayoutModule ()  : AGDModule(1,1) { };
  /*{\Mcreate initializes a layout algorithm.}*/

  virtual ~CGLayoutModule () { };

  /*{\Moperations 1.2 3.5}*/

  virtual bool check (const ClusterGraph& CG, AgdKey &p) const {
    return check_graph (in_graph, CG, p);
  }
  /*{\Mop returns |true| if $G$ satisfies the preconditions of |\Mvar|.
    Otherwise, |false| is returned and |p| contains a property that
    is not satisfied.}*/

  virtual void call (const ClusterGraph& CG, 
		     CGLayoutInterface& A) = 0;
  /*{\Moppure calls |\Mvar| for cluster graph |CG| with cluster graph layout
    interface $A$. \precond $A$ is a layout interface for cluster graph 
    |CG|.}*/

  virtual void call (const ClusterGraph& CG, 
		     leda::face f_0, 
		     CGLayoutInterface& A) = 0;
  /*{\Moppure calls |\Mvar| for cluster graph |CG| with cluster graph
    layout interface |A|. \precond |CG| represents a planar map, $f$ is |nil| 
    or a face in |CG|, and $A$ is a layout interface for cluster graph |CG|.}*/

};

#endif
