/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGPlaneGraphCopyGridConcept.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      CGPlaneGraphCopyGridConcept.h
//
//    author: Dirk Luetke-Huettmann
//    date:   juli 1999
//-------------------------------------------------------------------


#ifndef _AGD_CGPlaneGraphCopyGridConcept_h
#define _AGD_CGPlaneGraphCopyGridConcept_h


#include <AGD/impl/CGGraphCopyGridConceptBase.h>
#include <AGD/CGLayoutInterface.h>
#include <AGD/CGGridLayout.h>
#include <AGD/PlaneClusterGraphCopy.h>

class PlaneGraphCopy;
class GridCoordinateMapper;


/*{\Manpage {CGPlaneGraphCopyGridConcept} {} {Grid Concept for Plane Cluster Graph Copies} {C} }*/

class __exportAGD CGPlaneGraphCopyGridConcept : public CGGraphCopyGridConceptBase {
/*{\Mdefinition
 The class |\Mname| represents an implementation concept for clustergraph grid layout
algorithms which work on a planar map and access the clustergraph grid
layout via an object of type |CGGridLayout|. The graph is stored as a plane clustergraph
copy |PCG|, which can be modified by the algorithm. The following transformations
are performed by the update functions:
\begin{itemize}
\item If the polyline of an edge $e$ is empty and the polyline of its
reversal is not empty, the reversed polyline of the reversal edge
is chosen as polyline for $e$. Thus, it is sufficient to assign a
polyline to an edge or its reversal, no matter which one corresponds
to an edge in the original graph.
\item The polylines of splitted edges are concatenated.
\item Unnecessary bend points are removed.
\end{itemize}
A computed bounding box can be passed to the grid concept using
set\_bbox(). Otherwise, the bounding box is computed on demand.
}*/ 
	leda::face             _ext_face;

	bool new_graph, new_layout;

	// forbid copying
	CGPlaneGraphCopyGridConcept (const CGPlaneGraphCopyGridConcept &) { }
	CGPlaneGraphCopyGridConcept &operator=(const CGPlaneGraphCopyGridConcept &) {
		return *this;
	}

public:
	// constructors !!!
	
	CGPlaneGraphCopyGridConcept (const ClusterGraph &G, 
				     CGLayoutInterface &A);
	
	CGPlaneGraphCopyGridConcept (const ClusterGraph &G, 
				     leda::face f, 
				     CGLayoutInterface &A);

	CGPlaneGraphCopyGridConcept (const ClusterGraph &G,
				     CGGridLayout &gl);

	CGPlaneGraphCopyGridConcept (const ClusterGraph &G, 
				     leda::face f, 
				     CGGridLayout &gl);

	~CGPlaneGraphCopyGridConcept();


	/*{\Moperations 5 2}*/

	/*{\Mtext |\Mname| implements the constructors and operations which are
	required for template argument |GRID_CONCEPT| of |CGGridLayoutModuleImpl|.
	Furthermore, the following operations are provided:}*/

	void get_node_grid_size (GridCoordinateMapper &gcm,
		const LayoutInterface &A);
	void get_node_grid_size(const GridLayout &gl) {
		CGGraphCopyGridConceptBase::get_node_grid_size(gl);
	}

	void update (GridCoordinateMapper &gcm, LayoutInterface &A);
	void update (CGGridLayout &gl);

	PlaneGraphCopy &get_graph() {
	  return *(PlaneGraphCopy *)(_gcp);
	}
	/*{\Mop returns the plane graph copy |PG|.}*/

	PlaneClusterGraphCopy& get_clustergraph() {
	  return *(PlaneClusterGraphCopy *) (_pgcp);
	}

	/*{\Mop returns the plane clustergraph copy |PCG|.}*/

	leda::face get_exterior_face() {
	  return _ext_face;
	}
	/*{\Mop returns the exterior face of |PG| or |nil|.}*/
        /*{\Moptions warnings=no}*/

	/*
	GridLayout &get_layout() const;
	*/
	/*{\Mop returns the grid layout of |G|.}*/

	/*
	void set_zero_size();
	*/
	/*{\Mop sets the grid width and height of all nodes to $0$.}*/

	/*
	void set_bbox (const IRect &bbox);
	*/
	/*{\Mop sets the bounding box of the grid layout to |bbox|.}*/
};


#endif
