/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGraphWinInterface.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      CGraphWinInterface.h
//
//    author: Dirk Luetke-Huettmann
//    date:   nov 1998
//-------------------------------------------------------------------

#ifndef _AGD_CLUSTER_GRAPH_WIN_INTERFACE_H
#define _AGD_CLUSTER_GRAPH_WIN_INTERFACE_H

#include <AGD/CGLayoutInterface.h>
#include <AGD/CGGridLayout.h>
#include <AGD/CGraphWin.h>

/*{\Manpage {CGraphWinInterface} {} {Interface to CGraphWin} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{2.5}
\hdclass{0}{1.5}{6}{CGLayoutInterface}
\arhook{6}{2}{7}
\hdclass{1}{0}{6.5}{CGraphWinInterface}
\end{hierarchy}
}*/

class __exportAGD CGraphWinInterface : public CGLayoutInterface {

/*{\Mdefinition
	An instance |\Mvar| of type |\Mname| is the interface to the
	graphical attributes of a |CGraphWin| object.
}*/

public:

        /*{\Mcreation}*/

	CGraphWinInterface (CGraphWin& CGW);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to an interface to |CGW|.}*/

	~CGraphWinInterface();

	/*{\Moperations}*/
	/*{\Mtext
	|\Mname| inherits the operations from its base class |CGLayoutInterface|
	(see Sect.~\ref{Layout Interfaces})
	}*/

	double get_x (leda::node v) const {
		return cgw_p->get_position(v).xcoord();
	}

	void set_x (leda::node v, double new_x) {
		cgw_p->set_position(v,leda::point(new_x,get_y(v)));
	}

	double get_y (leda::node v) const {
		return cgw_p->get_position(v).ycoord();
	}

	void set_y (leda::node v, double new_y) {
		cgw_p->set_position(v,leda::point(get_x(v),new_y));
	}

	DPoint get_position (leda::node v) const {
		leda::point p = cgw_p->get_position(v);
		return DPoint(p.xcoord(),p.ycoord());
	}

	void set_position (leda::node v, const DPoint& p) {
		cgw_p->set_position(v,leda::point(p.xcoord(),p.ycoord()));
	}

	DPolyline get_bend_points (leda::edge e) const {
		leda::list<leda::point> points = cgw_p->get_bends(e);
		DPolyline l;
		leda::point p;
		forall(p,points) l.append(DPoint(p.xcoord(),p.ycoord()));
		return l;
	}

	void set_bend_points (leda::edge e, const DPolyline& l) {
		leda::list<leda::point> points;
		DPoint dp;
		forall(dp,l) points.append(leda::point(dp.xcoord(),dp.ycoord()));
		cgw_p->set_bends(e,points);
	}

	double get_width (leda::node v) const {
		return cgw_p->get_radius(v)*2;
	}

	void set_width (leda::node v, double new_w) {
		cgw_p->set_radius (v, new_w/2);
	}

	double get_height (leda::node v) const {
		return cgw_p->get_radius2(v)*2;
	}

	void set_height (leda::node v, double new_h) {
		cgw_p->set_radius2 (v, new_h/2);
	}


	DPoint get_source_anchor (leda::edge e) const {
		leda::point p = cgw_p->get_source_anchor(e);
		return DPoint(p.xcoord(),p.ycoord());
	}

	void set_source_anchor (leda::edge e, const DPoint& pos) {
		cgw_p->set_source_anchor (e, leda::point(pos.xcoord(),pos.ycoord()));
	}

	DPoint get_target_anchor (leda::edge e) const {
		leda::point p = cgw_p->get_target_anchor(e);
		return DPoint(p.xcoord(),p.ycoord());
	}

	void set_target_anchor (leda::edge e, const DPoint& pos) {
		cgw_p->set_target_anchor (e, leda::point(pos.xcoord(),pos.ycoord()));
	}

	void remove_bends(void) {
	  cgw_p->remove_bends();
	}

        void fill_window(void);
	void init (CGraphWin &CGW);
	void update();
	void refresh() {
		cgw_p->update_graph();
	}

	double get_xmax(void) {
	  return(cgw_p->get_xmax());
	}

	void set_cluster_region(Cluster* cl, 
				double x,
				double y,
				double width,
				double height);

private:

	CGraphWin *cgw_p;
	Cluster* rc_p;
	bool old_flush;
};

#endif
