/*******************************************************************************
+
+  AGD 1.2
+
+
+  CPlanarEmbed.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         CPlanarEmbed.h
//
//
//    author: Christoph Buchheim
//    date:   August 1999
//-------------------------------------------------------------------

#ifndef _AGD_CPLANAR_EMBED_H
#define _AGD_CPLANAR_EMBED_H

#include <AGD/AGDModule.h>
#include <AGD/impl/cplanarity/C_planarConnectedC_Graph.h>

/*{\Manpage {CPlanarEmbed} {} {C-Planarity Embedding} {PE} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5.5}{AGDModule}
\arhook{5.5}{2}{6.5}
\hdclass{1.5}{0}{5.5}{CPlanarEmbed}
\end{hierarchy}
}*/

class __exportAGD CPlanarEmbed : public AGDModule {

/*{\Mdefinition
The class |\Mname| represents the c-planarity testing and embedding
algorithm by Q.-W.\ Feng. It tests if a cluster graph |CG| is
c-connected and c-planar and embeds |CG| c-planar.

\horizline
\commoninfo{C-Planar Embedding}{C-Planar Embedding}{Q.-W. Feng}
{C. Liebel, C. Buchheim}{August 1999}{}

\begin{preposttab}
precondition & $\emptyset$
\end{preposttab}

\horizline
}*/

public:

	/*{\Mcreation PE 4.5}*/

  CPlanarEmbed()  : AGDModule(1,1) {
    add_post_rule (key::c_planar);
  }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

  ~CPlanarEmbed() { }


	/*{\Moperations 1.5 2.5}*/

  leda::string name () const {
    return "C-Planar Embedding";
  }

  leda::string author () const {
    return "Q.-W. Feng";
  }
  
  leda::string impl_author () const {
    return "C. Liebel, C. Buchheim";
  }
  
  leda::string impl_date () const {
    return "August 1999";
  }

  AGDModule *clone() const {
    CPlanarEmbed* M = new CPlanarEmbed();
    return M;
  }

  bool check(const ClusterGraph& CG, AgdKey &p) const {
    return check_graph(1, CG, p);
  }

  bool is_cplanar(const ClusterGraph &CG) {
    C_planarConnectedC_Graph algo;
    algo.constructC_Graph(CG);
    return algo.C_planarTest();
  }
	/*{\Mop returns |true| iff |CG| is c-connected and c-planar.}*/

  bool call(ClusterGraph &CG) {
	  leda::list<leda::edge> L;
	  return call(CG,L);
  }
	/*{\Mop calls |\Mvar| for cluster graph |CG|. If |CG| is c-connected
	and c-planar, |CG| is turned into a map, embedded c-planar, 
	and |true| is returned. Otherwise, |false| is returned.}*/

  bool call(ClusterGraph &CG, leda::list<leda::edge>& L) {
    C_planarConnectedC_Graph algo;
    algo.constructC_Graph(CG);
    if(!algo.C_planarEmbed()) return false;
    algo.writeEmbedding(CG,L);
    return true;
  }
	/*{\Mop calls |\Mvar| for cluster graph |CG|. If |CG| is c-connected
	and c-planar, |CG| is turned into a map by adding the edges returned
	in |L|, embedded c-planar, and |true| is returned. Otherwise, |false|
	is returned.}*/
};

/*{\Mimplementation
The running time of is\_cplanar() and call() is $\BigO(n^2)$, where $n$
is the number of vertices in |CG|.
}*/

#endif
