/*******************************************************************************
+
+  AGD 1.2
+
+
+  ClusterGraphGLI.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         ClusterGraphGLI.h
//
//    author: Dirk Luetke-Huettmann
//    date:   august 1999
//-------------------------------------------------------------------


#ifndef _AGD_CG_GLI_H
#define _AGD_CG_GLI_H

#include <AGD/CGGridLayoutInfo.h>
#include <AGD/CGGridLayout.h>

/*{\Manpage {ClusterGraphGLI} {} {GLI's for Cluster Graphs} {gli} }*/ 

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{6}{GridLayoutInfo}
\arhook{6}{3.5}{7}
\hdclass{1.5}{1.5}{6}{CGGridLayoutInfo}
\arhook{7.5}{2}{8.5}
\hdclass{3}{0}{6}{ClusterGraphGLI}
\end{hierarchy}
}*/

class __exportAGD ClusterGraphGLI : public CGGridLayoutInfo {

/*{\Mdefinition The class |\Mname| extends the functionality of its
  base class |GridLayoutInfo| to access the attributes of a
  cluster graph grid layout for a cluster graph |CG|.}*/

  CGGridLayout *_cgglp;
  IRect        _bbox;

public:

  /*{\Minitialization}*/

  ClusterGraphGLI(CGGridLayout &cggl) : _cgglp(&cggl) {
    _bbox = cggl.bounding_box();
  }
  /*{\Mcreate initializes a cluster graph grid layout info with |cggl|.}*/

  ClusterGraphGLI(CGGridLayout &cggl, const IRect &bbox) : _cgglp(&cggl), _bbox(bbox) { }

  ~ClusterGraphGLI() { }

  /*{\Moperations}*/

  int x (leda::node v) const {
    return _cgglp->x(v);
  }
  
  int y (leda::node v) const {
    return _cgglp->y(v);
  }
  
  IPolyline bends (leda::edge e) const {
    return _cgglp->get_compact_bends(e);
  }
  
  int grid_width (leda::node v) const {
    return _cgglp->width(v);
  }
  
  int grid_height (leda::node v) const {
    return _cgglp->height(v);
  }
  
  IRect get_gbbox() {
    return _bbox;
  }
  
  int x(Cluster* cl) const {
    return _cgglp->x(cl);
  }
  /*{\Mop returns the column of |cl| in the cluster graph grid layout.}*/

  int y(Cluster* cl) const {
    return _cgglp->y(cl);
  }
  /*{\Mop returns the row of |cl| in the cluster graph grid layout.}*/

  int width(Cluster* cl) const {
    return _cgglp->width(cl); 
  }
  /*{\Mop returns the width of |cl| in the cluster graph grid layout.}*/

  int height(Cluster* cl) const {
    return _cgglp->height(cl);
  }
  /*{\Mop returns the height of |cl| in the cluster graph grid layout.}*/

  void grid_width (leda::node v, int w) const {
    _cgglp->width(v,w);
  }
  
  void grid_height (leda::node v, int h) const {
    _cgglp->height(v,h);
  }

  /*const leda::list<Cluster*> get_clusters() const {
    return _cgglp->get_clusters();
  }*/
  /* {\Mop returns the list of all clusters in the cluster graph grid layout.}*/


};


#endif
