/*******************************************************************************
+
+  AGD 1.2
+
+
+  CompactionModule.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 46 "Lweb/compaction.lw"
#ifndef _AGD_COMPACTION_MODULE_H 
#define _AGD_COMPACTION_MODULE_H

#include <AGD/AGDModule.h>
#include <AGD/LayoutInterface.h>
#include <AGD/GridLayout.h>

#include <AGD/OrthogonalRepresentation.h>
//#include <AGD/Label.h>

//#include <LEDA/graphwin.h> // for debugging, obacht: should be removed

/*{\Moptions
outfile=CompactionModule.man
}*/

/*{\Manpage {CompactionModule} {} {Compaction Modules} {C} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
  \hdclass{0}{1.5}{5.5}{AGDModule} \arhook{5.5}{2}{6.5}
  \hdclass{0.5}{0}{6.5}{CompactionModule}
\end{hierarchy} }*/

class __exportAGD CompactionModule : public AGDModule {
  
  /*{\Mdefinition |\Mname| is the base class for all compaction modules. A compaction module works on a grid layout or an orthogonal representation of a graph and produces again a grid layout.
    
    \headerline{Input and Output Parameters}
    
    \begin{tabular}{lcc}
    input parameter: & graph & $|in_graph|=1$ \\
    & \ldots & \\
    output parameter: & GridLayout & $|out_layout|=1$
    \end{tabular}
    
    \headerline{Implementation of Compaction Algorithms}
    
    An implementation of a compaction module must override
    the protected methods |call($G$, $L$)| and |call($H$, $L$)|. The first
    gets as input a constant reference to a graph $G$ and a reference to a 
    |GridLayout| $L$ which will be modified. The second method gets a constant
    reference to an |OrthogonalRepresentation| and returns the computed |GridLayout| in $L$.
    }*/
  
  public:

  static const int in_graph;
  static const int in_layout;
  static const int out_layout;
  
  /*{\Minitialization C 5}*/
  
  CompactionModule();
  /*{\Mcreate initializes a compaction module.}*/
  
  virtual ~CompactionModule() {}
  
  /*{\Moperations 1.5 2.5}*/

  bool debug() const { return debug_; }
  void debug(bool d) { debug_ = d; }

  virtual AGDModule *clone() const = 0;
  
  virtual leda::string impl_version() const { return leda::string("1.2"); }
  
  virtual leda::string impl_date() const { return leda::string("October 1999"); }
  
  /*{\Mtext \headerline{Standard Interface} }*/
  
  virtual bool check(const leda::graph& G, AgdKey &p);
  /*{\Mop returns |true| if $G$ satisfies the precondition of
    |\Mvar|.  Otherwise, |false| is returned, and |message|
    contains the error message.}*/

  virtual double call(GridLayout& L) = 0;
  /*{\Moppure implements the compaction algorithm based on the initial
    |GridLayout| $L$ and its underlying graph. 
    Returns the new |GridLayout| in $L$.}*/

  virtual int call(const OrthogonalRepresentation& H, GridLayout& L) = 0;
  /*{\Moppure implements the compaction algorithm for 
    |OrthogonalRepresentation| $H$. The
    computed |GridLayout| will be returned in $L$.}*/

  //  virtual int call(const OrthogonalRepresentation& H, GridLayout &L,
  //       list<Label> &Lambda) = 0;

  void output_level(int o) { output_level_ = o; } 
  /*{\Mop sets the output level to |o|. 
    Reasonable values are $o \in [0 \ldots 10]$. 
    $o = 0$ causes the algorithm to run in silent mode, 
    a value $o \geq 10$ will create the greatest
    possible amount of output. }*/
  
  int output_level() const { return output_level_; } 
  /*{\Mop returns the output level}*/
  
protected:

  int output_level_;

  bool debug_;
  //GraphWin W_debug; // for debugging purposes. obacht: should be removed

};

#endif

