/*******************************************************************************
+
+  AGD 1.2
+
+
+  ConstLedaGraphConcept.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                       ConstLedaGraphConcept.h
//
//    author: Carsten Gutwenger
//    date:   january 2000
//-------------------------------------------------------------------


#ifndef _AGD_CONST_LEDA_GRAPH_CONCEPT_H
#define _AGD_CONST_LEDA_GRAPH_CONCEPT_H


#include <LEDA/graph/graph.h>
class LayoutInterface;
class PlaneGraphCopy;


/*{\Manpage {ConstLedaGraphConcept} {} {Concept for LEDA Graphs} {C} }*/

class __exportAGD ConstLedaGraphConcept {

/*{\Mdefinition
The class |\Mname| represents an implementation concept for layout
algorithms which work on a const LEDA graph |G| and access the layout
via a layout interface $A$. The graph cannot be modified by the algorithm.
}*/

	const leda::graph *_cgp;
	LayoutInterface  *_lip;

	// forbid copying
	ConstLedaGraphConcept (const ConstLedaGraphConcept &) { }
	ConstLedaGraphConcept &operator=(const ConstLedaGraphConcept &) {
		return *this;
	}

public:

	ConstLedaGraphConcept (const leda::graph &G, LayoutInterface &A) :
		_cgp(&G), _lip(&A) { }
		

	// never called
	ConstLedaGraphConcept (const leda::graph &G, leda::face , LayoutInterface &A) { }
	ConstLedaGraphConcept (PlaneGraphCopy &PG, leda::face , LayoutInterface &A) { }


	~ConstLedaGraphConcept() { }


	/*{\Moperations}*/
	/*{\Mtext |\Mname| implements the constructors and operations which are
	required for template argument |CONCEPT| of |LayoutModuleImpl|.
	Furthermore, the following operations are provided:}*/

	void update();


	const leda::graph &get_graph() const {
		return *_cgp;
	}
	/*{\Mop returns the const LEDA graph |G|.}*/

	LayoutInterface &get_layout() {
		return *_lip;
	}
	/*{\Mop returns the layout interface $A$ for |G|.}*/

	static void set_postcondition(leda::list<AgdKey> &post) { }
};


#endif
