/*******************************************************************************
+
+  AGD 1.2
+
+
+  FlowCompaction.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 619 "Lweb/compaction.lw"
#ifndef _AGD_FLOW_COMPACTION_H
#define _AGD_FLOW_COMPACTION_H

#include <AGD/CompactionModule.h>

/*{\Moptions
outfile=FlowCompaction.man
}*/

/*{\Manpage {FlowCompaction} {} {Flow--Based One--dimensional Compaction} {C}
  }*/

/*{\Mbaseclasses

  \begin{hierarchy}{8}{4}
  \hdclass{0}{3}{5.5}{AGDModule} \arhook{5.5}{3.5}{6.5}
  \hdclass{0.5}{1.5}{6.5}{CompactionModule} \arhook{7}{2}{8}
  \hdclass{2.5}{0}{6}{FlowCompaction}
  \end{hierarchy}
  }*/

class __exportAGD FlowCompaction : public CompactionModule {
  
/*{\Mdefinition
The class |\Mname| represents the one--dimensional compaction module based on minimu--cost flow computations in the so called constraint graphs \cite{KM98b}.
}*/
  
public:

  /*{\Mcreation C 5}*/

  FlowCompaction() {}
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/
  
  ~FlowCompaction() {}
  
  /*{\Moperations 1.5 2.5}*/
  
  leda::string name() const { 
    return leda::string("1_dim Compaction Module (optimal)"); 
  }
  
  leda::string long_name() const { 
    return leda::string("Optimal One-Dimensional Compaction Module"); 
  }
  
  leda::string author() const { return leda::string("?"); }
  
  virtual leda::string impl_author() const { 
    return leda::string("G. Klau"); 
  }
  
  virtual leda::string impl_version() const { return
    leda::string("1.0"); }
  
  virtual leda::string impl_date() const { return
                                            leda::string("XI 1998"); }
  
  /*{\Mtext \headerline{Standard Interface} }*/
 
  double call(GridLayout& L); 
  /*{\Mop Performs a visibility--based compaction on the layout. The compaction
    is realized by an alternating sequence of minimum--cost flow computations in the
    constraint graphs for $x$ and $y$--direction}*/
  
  int call(const leda::graph& , LayoutInterface& ) {
    cerr  << "not yet" << endl; return -1;
  }

  int call(const leda::graph& G, leda::node_map<int>& x, leda::node_map<int>& y);
  
  int call(const OrthogonalRepresentation& H, GridLayout& L); 
  /*{\Mop Performs a visibility--based compaction on the constraint graphs
    defined by the shape of the drawing. To do this, $H$ must be equivalent
    to a complete shape description. The result is an drawing with minimum
    total edge length for the given $H$}*/

  //  int call(const OrthogonalRepresentation& H, GridLayout& L, 
  //   leda::list<Label> &Lambda); 
  
  AGDModule *clone() const;
  
};

#endif

