/*******************************************************************************
+
+  AGD 1.2
+
+
+  GraphCopyGridConcept.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      GraphCopyGridConcept.h
//
//    author: Carsten Gutwenger
//    date:   octobre 1998
//-------------------------------------------------------------------


#ifndef _AGD_GCG_CONCEPT_H
#define _AGD_GCG_CONCEPT_H


#include <AGD/impl/GCGridConceptBase.h>


/*{\Manpage {GraphCopyGridConcept} {} {Grid Concept for Graph Copies} {C} }*/

class __exportAGD GraphCopyGridConcept : public GCGridConceptBase {

/*{\Mdefinition
The class |\Mname| represents an implementation concept for grid layout
algorithms which work on a graph copy |GC| and access the grid
layout via an object of type |GridLayout|. The graph copy can be modified by
the algorithm. The following transformations are performed by the update
functions:
\begin{itemize}
\item The polylines of reversed edges are reversed.
\item The polylines of splitted edges are concatenated.
\item Unnecessary bend points are removed.
\end{itemize}
A computed bounding box can be passed to the grid concept using
set\_bbox(). Otherwise, the bounding box is computed on demand.
}*/

	// forbid copying
	GraphCopyGridConcept (const GraphCopyGridConcept &) { }
	GraphCopyGridConcept &operator=(const GraphCopyGridConcept &) {
		return *this;
	}

public:

	GraphCopyGridConcept (const leda::graph &G, LayoutInterface &A);		
	GraphCopyGridConcept (const leda::graph &G, GridLayout &gl);

	// never called
	GraphCopyGridConcept (const leda::graph &G, leda::face f, LayoutInterface &A) { }
	GraphCopyGridConcept (PlaneGraphCopy &PG, leda::face f, LayoutInterface &A) { }
	GraphCopyGridConcept (const leda::graph &G, leda::face f, GridLayout &gl) { }
	GraphCopyGridConcept (PlaneGraphCopy &PG, leda::face f, GridLayout &gl) { }

	~GraphCopyGridConcept();


	/*{\Moperations}*/
	/*{\Mtext |\Mname| implements the constructors and operations which are
	required for template argument |GRID_CONCEPT| of |GridLayoutModuleImpl|.
	Furthermore, the following operations are provided:}*/

	static void set_postcondition(leda::list<AgdKey> &post) { }

	void get_node_grid_size (GridCoordinateMapper &gcm,
		const LayoutInterface &A);
	void get_node_grid_size(const GridLayout &gl) {
		GCGridConceptBase::get_node_grid_size(gl);
	}

	void update (GridCoordinateMapper &gcm, LayoutInterface &A);
	void update (GridLayout &gl);

	/*{\Moptions warnings=no}*/

	/*
	GraphCopy &get_graph() const;
	*/
	/*{\Mop returns the graph copy |GC|.}*/

	/*
	GridLayout &get_layout() const;
	*/
	/*{\Mop returns the grid layout of |G|.}*/

	/*
	void set_zero_size();
	*/
	/*{\Mop sets the grid width and height of all nodes to $0$.}*/

	/*
	void set_bbox (const IRect &bbox);
	*/
	/*{\Mop sets the bounding box of the grid layout to |bbox|.}*/
};


#endif
