/*******************************************************************************
+
+  AGD 1.2
+
+
+  GraphWinAnimInterface.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                     GraphWinAnimInterface.h
//
//    author: Carsten Gutwenger
//    date:   july 1999
//-------------------------------------------------------------------


#include <AGD/AGDLayout.h>
#include <LEDA/graphics/graphwin.h>


#ifndef _AGD_GW_ANIM_INTERFACE_H
#define _AGD_GW_ANIM_INTERFACE_H


/*{\Manpage {GraphWinAnimInterface} {} {Interface with Animation to GraphWin} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{LayoutInterface}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5}{LayoutBuffer}
\arhook{6.5}{2}{7.5}
\hdclass{0}{0}{8}{GraphWinAnimInterface}
\end{hierarchy}
}*/

//class GraphWinAnimInterface : public LayoutBuffer {
class __exportAGD GraphWinAnimInterface : public AGDLayout {

/*{\Mdefinition
	An instance |\Mvar| of type |\Mname| is an interface to the
	graphical attributes of a |GraphWin| object. It buffers the
	graphical attributes until update() is called. The update()
	methods uses the set\_layout() function of |GraphWin|, which
	performs an animation sequence in order to transform the
	old layout into the new one.
}*/

public:

	/*{\Mcreation}*/

	GraphWinAnimInterface (leda::GraphWin& GW);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to an interface to |GW|.}*/

	//GraphWinAnimInterface (const GraphWinAnimInterface &gwi) : LayoutBuffer(gwi),
	GraphWinAnimInterface (const GraphWinAnimInterface &gwi) : AGDLayout(gwi),
		old_flush(gwi.old_flush), gw_p(gwi.gw_p) { }
	/*{\Mcreate copy constructor.}*/

	~GraphWinAnimInterface() {
		gw_p->set_flush(old_flush);
	}


	/*{\Moperations}*/

	void init(leda::GraphWin &GW);
	/*{\Mop initializes |\Mvar| for |GW|.}*/


	/*{\Mtext\medskip
	|\Mname| inherits the operations from its base class |LayoutInterface|
	(see Sect.~\ref{Layout Interfaces}).
	}*/
/*
	double get_width (leda::node v) const {
		return gw_p->get_radius(v)*2;
	}

	void set_width (leda::node v, double new_w) {
		gw_p->set_radius (v, new_w/2);
	}

	double get_height (leda::node v) const {
		return gw_p->get_radius2(v)*2;
	}

	void set_height (leda::node v, double new_h) {
		gw_p->set_radius2 (v, new_h/2);
	}
*/
	void update();
	void refresh() {
		gw_p->update_graph();
	}

private:
	void init_buffer();

	bool old_flush;
	leda::GraphWin *gw_p;
};



#endif
