/*******************************************************************************
+
+  AGD 1.2
+
+
+  GridLayoutInfo.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//                      GridLayoutInfo.h
//
//    author: Carsten Gutwenger
//    date:   july 1999
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_GL_INFO_H
#define _AGD_GL_INFO_H


#include <AGD/IPolyline.h>
#include <LEDA/graph/graph.h>

class IRect;


/*{\Manpage {GridLayoutInfo} {} {Grid Layout Infos} {gli} }*/

class __exportAGD GridLayoutInfo {

/*{\Mdefinition
The class |\Mname| defines a generic interface for accessing attributes
of a grid layout for a graph |G|. It decouples the access of attributes
from the representation of the grid layout, e.g., the grid layout might
be given implicitly by a grid layout of a modified copy of |G|.
|\Mname| is used by |GridCoordinateMapper|.
}*/

public:

	/*{\Minitialization}*/

	GridLayoutInfo() { }
	/*{\Mcreate initializes a grid layout info.}*/

	virtual ~GridLayoutInfo() { }


	/*{\Moperations}*/

	virtual int x (leda::node v) const
	/*{\Moppure returns the column of $v$ in the grid layout.}*/
	= 0;

	virtual int y (leda::node v) const
	/*{\Moppure returns the row of $v$ in the grid layout.}*/
	= 0;

	virtual IPolyline bends (leda::edge e) const
	/*{\Moppure returns the list of bend points of $e$ in the grid
	layout.}*/
	= 0;

	virtual int grid_width (leda::node v) const
	/*{\Moppure returns the width of $v$ in the grid layout.}*/
	= 0;

	virtual int grid_height (leda::node v) const
	/*{\Moppure returns the height of $v$ in the grid layout.}*/
	= 0;

	virtual IRect get_gbbox()
	/*{\Moppure returns a tight bounding box of the grid layout.}*/
	= 0;

	virtual void grid_width (leda::node v, int w) const
	/*{\Moppure sets the width of $v$ in the grid layout to $w$.}*/
	= 0;

	virtual void grid_height (leda::node v, int h) const
	/*{\Moppure sets the height of $v$ in the grid layout to $h$.}*/
	= 0;
};


#endif
