/*******************************************************************************
+
+  AGD 1.2
+
+
+  HierarchyLayoutModule.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                        HierarchyLayout.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_HIERARCHY_LAYOUT_H
#define _AGD_HIERARCHY_LAYOUT_H

#include <AGD/AGDModule.h>


class Hierarchy;
class GraphCopy;
class LayoutInterface;

/*{\Manpage {HierarchyLayoutModule} {} {Hierarchy Layout Modules} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7.5}{2.5}
\hdclass{0.5}{1.5}{5.5}{AGDModule}
\arhook{6}{2}{7}
\hdclass{0}{0}{7.5}{HierarchyLayoutModule}
\end{hierarchy}
}*/

class __exportAGD HierarchyLayoutModule : public AGDModule {

/*{\Mdefinition
Modules of type |\Mname| compute a layout of a node hierarchy |H|.
|H| is a hierarchy of a graph |GC|, which is a copy of a graph |G|.
|GC| may contain dummy nodes that are not copies of nodes in |G|.
These dummy nodes have degree two and represent edge bends in
the final drawing.

\headerline{Implementing Hierarchy Layout Modules}

An implementation of a hierarchy layout module
must override the method call(|GC|,$H$,|A|), which gets as
input const references to a graph copy |GC| and a node hierarchy
|H| of |GC|. It must return the layout in |A|.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & hierarchy & |in_hierarchy=1| \\
output parameter: & layout & |out_layout=1|
\end{tabular}

The integer constants |in_hierarchy| and |out_layout| denote the
positions in the input and output parameter lists. The values can be used to
reference the desired parameter when setting the pre- and postcondition.  
}*/

public:

	// the input and output parameters of a hierarchy layout module
	static const int in_hierarchy;
	static const int out_layout;

	/*{\Minitialization L 5}*/

	HierarchyLayoutModule () : AGDModule(1,1) { }
	/*{\Mcreate initializes a hierarchy layout module.}*/

	virtual ~HierarchyLayoutModule () { }


	/*{\Moperations 1.2 3}*/

	/*{\Mtext
	\headerline{Standard Interface} }*/

	bool check (const GraphCopy& GC, const Hierarchy& H, AgdKey &p) const;
	/*{\Mop returns |true| if |GC| satisfies the precondition of
	|\Mvar|. Otherwise, |false| is returned, and |p| contains a
	property that is not satisfied.}*/

	virtual void call (const GraphCopy& GC, const Hierarchy& H,
		LayoutInterface& A) = 0;
	/*{\Moppure implements the module call for graph $GC$ and hierarchy
	$H$. Assigns the layout to $A$.}*/
};

inline bool HierarchyLayoutModule::check (const GraphCopy& GC, const Hierarchy& ,
	AgdKey &p) const

{       void* ptr = (GraphCopy*)&GC;
	return check_graph (in_hierarchy, *(const leda::graph*)ptr, p);
}


#endif
