/*******************************************************************************
+
+  AGD 1.2
+
+
+  LEDAMakeBiconnected.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                       LEDAMakeBiconnected.h
//
//    author: AGD-Anpassung: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_LEDA_MK_BICONNECTED_H
#define _AGD_LEDA_MK_BICONNECTED_H


#include <AGD/AugmentationModule.h>


/*{\Manpage {LEDAMakeBiconnected} {} {LEDA's Make Biconnected} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{0.3}{1.5}{6.7}{AugmentationModule}
\arhook{7}{2}{8}
\hdclass{1}{0}{7.5}{LEDAMakeBiconnected}
\end{hierarchy}
}*/

class __exportAGD LEDAMakeBiconnected : public AugmentationModule {

/*{\Mdefinition
The |\Mname| module is a front end for LEDA's Make\_Biconnected() function.
It augments a graph $G$ to a biconnected graph $G'$. If $G$ was planar,
the output graph $G'$ is also planar.

The algorithm simply uses DFS and, whenever a cut vertex is discovered,
a new edge is added. Note that the number of added edges may be much
higher than the number of added edges using the modules |PlanAug| or 
|OptPlanAug|.

\horizline
\commoninfo{Make Biconnected}{LEDA's Make Biconnected}{}{S. N\"aher}
{}{}

\begin{preposttab}
precondition & $\emptyset$ \\
postcondition(|PRE|) & $\{\, \emph{biconnected} \,\}
\cup \Big( \{\, \emph{planar, simple, no self loops} \,\} \cap |PRE| \Big)$
\end{preposttab}

\horizline
}*/

public:

	/*{\Mcreation A 5}*/

	LEDAMakeBiconnected();
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/

	virtual ~LEDAMakeBiconnected() { }


	/*{\Moperations}*/

	leda::string name () const {
		return leda::string ("Make Biconnected");
	}

	leda::string long_name () const {
		return leda::string ("LEDA's Make Biconnected");
	}

	leda::string impl_author () const {
		return leda::string ("Stefan N\204her");
	}


	AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext\stdstart{1.5cm}
	\headerline{Standard Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|AugmentationModule|).
	}*/

	/* inherited
	bool check (const graph& G, AgdKey &p) const;
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G, list<edge>& L);
	*/
	/*{\Mop}*/
	
	/* inherited
	int num_added_edges() const;
	*/
	/*{\Mop}*/

	/*{\Mtext \stdend}*/
	

protected:
	void do_call (leda::graph& G, leda::list<leda::edge>& L);

};


/*{\Mimplementation
The algorithm has running time $\BigO(n+m)$, where $n$ is the
number of nodes and $m$ is the number of edges of the input graph $G$.
}*/


#endif
