/*******************************************************************************
+
+  AGD 1.2
+
+
+  OptAcyclicSubgraph.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         OptAcyclicSubgraph.h
//
//
// author: Matthias Elf
// date:   march 1998
//-------------------------------------------------------------------


#ifndef _AGD_MAX_ACYCLIC_SUBGRAPH_H
#define _AGD_MAX_ACYCLIC_SUBGRAPH_H


#include <AGD/SubgraphModule.h>


/*{\Manpage {OptAcyclicSubgraph} {} {Maximum Acyclic Subgraph} {S} }*/

/*{\Mbaseclasses
\abacus
\begin{hierarchy}{10}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{SubgraphModule}
\arhook{7}{2}{8}
\hdclass{2}{0}{6.5}{OptAcyclicSubgraph}
\end{hierarchy}
}*/


class __exportAGDopt OptAcyclicSubgraph : public SubgraphModule {

/*{\Mdefinition
The module |\Mname| computes a maximum acyclic subgraph of an
input graph G \cite{GJR85}.

\horizline
\commoninfo{Maximum Acyclic Subgraph}{Maximum Acyclic Subgraph}{M. Gr\"otschel, M. J\"unger, G. Reinelt}{M. Elf}
{september 1998}{}

\begin{preposttab}
precondition & $\{ \emph{directed} \} $ \\
postcondition(|PRE|) & $\{ \emph{maximum acyclic, directed} \} $
\end{preposttab}

\horizline
}*/

  public:

    /*{\Mcreation S 5}*/
    
    OptAcyclicSubgraph();
    /*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/
    
    
    ~OptAcyclicSubgraph() { }
    
    /*{\Moperations 1.2 3.5}*/

    leda::string name () const {
	return "Maximum Acyclic Subgraph";
    }

    leda::string author () const {
	return "M. Gr\224tschel, M. J\201nger, G. Reinelt";
    }

    leda::string impl_author () const {
	return "M. Elf";
    }

    leda::string impl_date () const {
	return "September 1998";
    }

    AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext\stdstart{1.5cm}
	\headerline{Standard Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|SubgraphModule|).
	}*/

	/* inherited
	bool check (const graph& G, AgdKey &p) const;
	*/
	/*{\Mop}*/

	/* inherited
	bool call (const graph& G, list<edge>& L);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (GraphCopy& G, list<edge>& L);
	*/
	/*{\Mop}*/

	/* inherited
	int num_del_edges() const;
	*/
	/*{\Mop}*/

	/*{\Mtext \stdend}*/
	
  protected:
    void do_call (const leda::graph& G, leda::list<leda::edge>& feedback);
};


#endif
