/*******************************************************************************
+
+  AGD 1.2
+
+
+  RankAssignment.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                        RankAssignment.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_RANK_ASSIGNEMENT_H
#define _AGD_RANK_ASSIGNEMENT_H

#include <AGD/AGDModule.h>


/*{\Manpage {RankAssignment} {} {RankAssignment Modules} {R} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5.5}{AGDModule}
\arhook{5.5}{2}{6.5}
\hdclass{1.5}{0}{5.5}{RankAssignment}
\end{hierarchy}
}*/

class __exportAGD RankAssignment : public AGDModule {

/*{\Mdefinition
Modules of type |\Mname| partition the nodes of a graph $G$ into
levels. Each node $v$ is assigned a non-negative rank $i$, what
means that $v$ belongs to the $i$th level. 

Such a ranking can
be used to build a node hierarchy of $G$. The hierarchical
drawing method by Sugiyama builds a node hierarchy in a first
step and computes a layout that represents the hierarchy, i.e.,
all nodes on the same level have the same y-coordinate.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & |in_graph=1| \\
output parameter: & ordered node partition & |out_node_partition=1|
\end{tabular}

The integer constants |in_hierarchy| and |out_node_partition| denote the
positions in the input and output parameter lists. The values can be used to
reference the desired parameter when setting the pre- and postcondition.  
}*/

public:

	// the input and output parameters of a rank assignment module
	static const int in_graph;
	static const int out_node_partition;

	/*{\Minitialization}*/

	RankAssignment () : AGDModule(1,1) { }

	/*{\Mcreate initializes a rank assignment module.}*/

	virtual ~RankAssignment () { }


	/*{\Moperations 1.2 3}*/

	/*{\Mtext
	\headerline{Standard Interface} }*/

	bool check (const leda::graph& G, AgdKey &p) const;
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |p| contains a property that
	is not satisfied.}*/

	virtual void call (const leda::graph& G, leda::node_array<int>& rank) = 0;
	/*{\Moppure implements the module call. Returns the rank for each
	node $v\in G$ in |rank[v]|.}*/
};


#endif
