/*******************************************************************************
+
+  AGD 1.2
+
+
+  ShapeModule.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 410 "Lweb/shape.lw"
#ifndef _AGD_SHAPE_MODULE_H
#define _AGD_SHAPE_MODULE_H

#include <AGD/OrthogonalRepresentation.h>
#include <AGD/TopologyModule.h>

/*{\Manpage {ShapeModule} {} {Shape Module} {S} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1.5}{1.5}{5.5}{TopologyModule}
\arhook{7}{2}{8}
\hdclass{3}{0}{5.5}{ShapeModule}
\end{hierarchy}
}*/

class __exportAGD ShapeModule : public TopologyModule {

  /*{\Mdefinition
    The class |\Mname| extends the functionality of a [[TopologyModule]]. For orthogonal drawings, one might be interested in not only the topology but also the shape of the drawing. An instance of class |\Mname| takes as input an orthogonal drawing (or just a sketch of it) and produces as output an [[OrthogonalRepresenation]]. This might be used as input for a [[CompactionModule]].

    \headerline{Input and Output Parameters}

    \begin{tabular}{lrl}
    input parameter: & graph                  & $|in_graph|=1$\\
    & layout                 & $|in_layout|=2$ \\
    output parameter: & orthogonal representation & $|out_orth_rep|=1$
    \end{tabular}

    }*/

public:

  static const int in_graph;
  static const int in_layout;
  static const int out_ortho_rep;

/*{\Minitialization S 5}*/
  
  ShapeModule();
  
  /*{\Mcreate initializes a shape module.}*/
  
  virtual ~ShapeModule() { }
  
  /*{\Moperations 1.5 2.5}*/

  leda::string name() const { return leda::string("Shape Module"); }
  
  leda::string long_name() const { 
    return leda::string("Shape Module"); 
  }
  
  leda::string author() const { return leda::string("Gunnar W. Klau"); }
  
  virtual leda::string impl_author() const { return leda::string("Gunnar W. Klau"); }
  
  virtual leda::string impl_version() const { return leda::string("1.0"); }
  
  virtual leda::string impl_date() const { return leda::string("May 1998"); }
  
  /*{\Mtext
  \headerline{Standard Interface} }*/

  bool check(const leda::graph& G, AgdKey &p) const;
  /*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
  Otherwise, |false| is returned and |p| contains a property
  that is not satisfied.}*/

  OrthogonalRepresentation &call(const leda::graph &G,
				 const LayoutInterface &L);
  /*{\Mop calls the ShapeModule for LayoutInterface $L$ and its underlying graph. Eventually, edges must be added in order to make the graph a map. These edges will be contained in the list $M$. }*/

  AGDModule *clone() const;

  /*{\Mtext
  \headerline{Options} }*/

  bool get_round() const { return round_; }
  /*{\Mop returns the value of the rounding option. If set, each scanned angle is set to the closest multiple of $90^\circ$.}*/

  void set_round(bool r) { round_ = r; }
  /*{\Mop sets the parameter round to |r|}*/

private:

  bool round_;

};

#endif

