/*******************************************************************************
+
+  AGD 1.2
+
+
+  TRLiteral.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         TRLiteral.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_TRLiteral_H
#define _AGD_TRLiteral_H


#include <AGD/AgdKey.h>



/*{\Manpage {TRLiteral} {} {Tagged Rule Literals} {l} }*/

class __exportAGD TRLiteral {

/*{\Mdefinition
An instance |\Mvar| of the data type |\Mname| is a pair $(k,i)$
of a key $k$ and an index $i$. Tagged rule literals are used
by tagged rules.
}*/

public:

	/*{\Mcreation}*/

	TRLiteral () {
		key = nil, index = 1;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(|nil|,1)$.}*/

	TRLiteral (AgdKey k) {
		key = k; index = 1;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(k,1)$.}*/

	TRLiteral (AgdKey k, int i) {
		key = k; index = i;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(k,i)$.}*/

	AgdKey key;
	int index;


	// Input & Output (currently dummy only)

	friend istream& operator>> (istream& is, TRLiteral& ) {
		return is;
	}

	friend ostream& operator<< (ostream& os, const TRLiteral& l) {
		os << "(" << l.key << "," << l.index << ")";
		return os;
	}

};


#endif
