/*********************************************************************

  FILENAME  : C_planarConnectedC_Graph.h
  VERSION   : 1.0.0
  AUTHOR    : Christoph Liebel
  LANGUAGE  : C++

*********************************************************************/


#ifndef C_PLANARCONNECTEDC_GRAPH_H
#define C_PLANARCONNECTEDC_GRAPH_H

#include "AGD/impl/cplanarity/clusterGraph.h"
#include "AGD/impl/cplanarity/extendedPlanGraph.h"



class clusterData {

    friend class C_planarConnectedC_Graph;

    clusterData();
    ~clusterData();

  // basic data

    int number;

    int nodes;
    int edges;

  // indicator whether the cluster contains only one or more than one node
  // incident to virtual edges of the cluster

     int oneVirtEdge;

  // edge numbers at the beginning of the C-planarity test

    int* originalToActual;
    int* actualToOriginal;

  // numbering of the extended graph induced by this cluster

    int* oldNode;
    int* oldEdge;

    int* saveNodeNumbers;
    int* saveEdgeNumbers;

    int* graphEnd;

  // numbering of the updated graph G

    int* storeNumbers;
    int* storeEdges;

    int* nodeNumbers;
    int* edgeNumbers;

    int* clusterEnd;

  // data of the incomplete embedded extended graph induced by this cluster

    extendedPlanGraph* cluster_graph;
    leafKey<outEdge,int,int>** leafBKeys;
};


class C_planarConnectedC_Graph : public clusterGraph {

  public:

    
                    C_planarConnectedC_Graph();
                    C_planarConnectedC_Graph(const clusterGraph&);
                    C_planarConnectedC_Graph(const C_planarConnectedC_Graph&);
    virtual         ~C_planarConnectedC_Graph();

    virtual  int    constructC_Graph(const ClusterGraph& C);
             int    C_planarEmbed();
             int    C_planarTest(int onlyPlanar = 1);
    virtual void    deleteGraph();

             int    getC_embedded() const;
             int    getC_planar() const;
    pqQueue<int>**    getEmbedding() const;

    void writeEmbedding(ClusterGraph& C,leda::list<leda::edge>& newEdges);


    const C_planarConnectedC_Graph& operator=(const C_planarConnectedC_Graph&);



  protected:

    int C_planar;
    int C_embedded;

    pqQueue<int>** embedding;



  private:

    int actual_N;
    int actual_M;

    int additionalEdgesCounter;

    linkedList<outEdge>* saveLoops;
    int* rememberEdgeNumbers;

    linkedList<int>* clusterNodes;
    linkedList<outEdge>* updateAdjList;

    pqStack<int>** generatedHubs;
    pqStack<int>** hubs;

    pqStack<clusterData*>* S;

     

    
    void    addVirtualEdge(int storeVirtNode, int storeNode, int cluster,
                           int* inCluster);
    void    change_and_correctNumbers(clusterData* info);
    void    changeWheel(int hubNumber, int* & rim);
     int    clusterTest(treeElement* clusterPtr);
    void    constructWheelGraph(pqNode<outEdge,int,int>* root,
                                linkedList<outEdge>* & hAL, int & count_N,
                                int & count_M, int & _node, int parentNum,
                                int cluster);
    void    correctClusterNodes_and_generatedHubs(int cluster, int storeNode,
                                                  int newN, int* newNode);
    void    countPQTree(pqNode<outEdge,int,int>* root, int & countNode,
                        int & countEdge);
    void    C_planarCluster(treeElement* helpPtr);
     int    findEdges(int* inBNodes, int & v_Edges, int cluster, int& onlyOne,
                      int& storeVirtNode, int& storeNode);
    void    findExtendedCluster(int nodes, linkedList<outEdge>* & cAL,
                                linkedList<outEdge>* vEL);
     int    findNodes(int cluster);
    void    findSubGraph(int nodes, int edges, int* inBNodes,
                         linkedList<outEdge>* & cAL, linkedList<outEdge>* & vEL,
                         int cluster);
    void    findVirtualOrder(pqQueue<int>** & embedVirtual);
    void    getLoops();
    void    saveOriginalEdgeNumbers(clusterData* & info);
    void    updateGraph(linkedList<outEdge>* & hAL, int countNode, int countEdge,
                        int* inCluster);


};
  


#endif

