/*******************************************************************************
+
+  AGD 1.2
+
+
+  l_matcher.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//  ToDo;
// 
//
// l_matcher.cpp: implementation of the l_matcher class.
//
//////////////////////////////////////////////////////////////////////

#include "l_matcher.h"
#include <LEDA/graph/graph_misc.h>
#include <LEDA/graph/graph_alg.h>

using namespace leda;


// To Do: update_label
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

l_matcher::l_matcher() : use_blabels_ (false),
strategy_ (SIZE),  planar_(N_CHECKED)
{}
//*************************************************************
l_matcher::~l_matcher()
{}
//*************************************************************
l_matcher::l_matcher(graph &Gr, MATCH_STRATEGY ms) : planar_(N_CHECKED)
{
 G_ = &Gr;
 strategy_ = ms;
 nTests_ = 0;
}
//*************************************************************
list<edge> l_matcher::augment (bool use_blabels, MATCH_STRATEGY strat)
{
 use_blabels_ = use_blabels;
 strategy_ = strat;
 nTests_ = 0;
 result_.clear ();

 BC_.init (G_, true);
 if (MAKE_CONNECTED (*G_, BC_, result_))
      BC_.update (true);

 BC_.DFS_num (BC_.bcnode_maximal_degree ());
 pendants_ = BC_.all_blocks_with_degree (1);
 block blok;

 forall (blok, pendants_)
   reduce_chain (blok, (cutpoint)father(blok));
 
 MASSIVE_ = (bcn_degree (BC_.root ()) - 1 > ceil ((float)pendants_.size ()/2)) ? true : false;
 
 while ( !is_ready ()) {
	 label first, sec;
	 block new_block;
	 bool found_match = false;

// To have larger labels I try  to "extend" in every iteration
// simple labels (lables of size 1).
	 list<label> size_one;
	 forall ( first, labels_ )
		 if ( first->size () == 1)
			 size_one.push (first);

	 if ( !use_blabels_ )
		 forall ( first, size_one )
			try_extend (first);
// Mit den b-labels kann folgendes passieren: Waehrend der Erweiterung
// eines labels kann es mit einem anderen c-label c2 zu einem b-label verknuepft
// werden. Beide werden geloescht, der Fehler tritt auf, wenn c2
// bearbeitet wird, weil es nicht mehr da ist.
	 else
		 forall ( first, size_one )
			 if ( labels_.search (first))//Beim search kann man Sortierung ausnutzen
				 try_extend (first);

	 if (strategy_ == SIZE)
		found_match = size_strategy (first, sec);
	 if (strategy_ == CONNECT_CONDITION )
		found_match = concond_strategy (first, sec);

	 if ( !found_match ) {//there are no labels planar to first
		 if (number_of_labels () == 1) {
			 join_pendants (first);		 
			 return terminate ();
		 }
		 block ancestor;
		 cutpoint new_head;
		 if ( first->size () == 1)  {
//Connect the single pendant of first to a block, which is 
//ancestor of first.head ()
			new_head = first->head ();
			if ( new_head == BC_.root ())  {
// new_head is root and degree (new_head) > 2
// Search for a block which is adj to new_head but do not belongs
// to path {new_head, pendant_of_first}
					ancestor = (block)first_adj (new_head);
					block wrong_ancestor = (block)find_last_before (first->first_pendant (), new_head);
					if ( wrong_ancestor == ancestor )
						ancestor = (block)last_adj (new_head);
			}
			else
				ancestor = (block)father (new_head);

			new_block = connect_inside_label (first, ancestor);
		 }
		 else //first.size () > 1
//Add edges between sons of first
//Update bctree BC_.
			new_block = join_pendants (first);
	}
	else { //sec is planar to first
		new_block = connect_labels (first, sec);
		if ( first->size () == 0)
			delete_label (first, false);
// sec is removed inside of connect_labels
	}

// Root Updatind
// Conventions: The root is a bc_node with "large" degree (not
// necessarily maximal). If exist bc_nodes with bcn degree (bcn) > 2,
// then set root to such bc-node. If all bc-nodes have degree <=2, then
// the root is a cutpoint, if possible a head of a label
	if ( bcn_degree (BC_.root ()) < 3 )
		update_dfs_numbering ();

	if ( bcn_degree (new_block) == 1) {
		pendants_.push (new_block);
		if ( belongs_to_.defined (new_block) && belongs_to_[new_block]->size () == 1)
			reduce_chain (new_block, (belongs_to_[new_block])->head (), belongs_to_[new_block]);
		else
			reduce_chain (new_block, (cutpoint)father (new_block));
	}

  }
 return terminate ();
}
//*************************************************************
STOP_CAUSE l_matcher::follow_path(cutpoint cp, cutpoint &last)
{
  bc_node bcn = cp;  
  while ( bcn != nil )  {
  	if ( bcn_degree (bcn) != 2 ) {
  		if ( is_cutpoint (bcn)) {
  			last = (cutpoint)bcn;
			return C_DEGREE;
  		}
  		else   			
			return B_DEGREE;
        }
  	if ( is_cutpoint (bcn))   last = (cutpoint)bcn;
   	else if ( number_of_nodes (bcn) > 4 && !PLANAR ((cutpoint)first_adj (bcn), (cutpoint)last_adj (bcn), false, false))
   		 return PLANARITY;
		 
	bcn = father (bcn);		
  }
  return ROOT;
}
//**************************************************************
bool l_matcher::connect_condition(label a, label b)
{
 list<bc_node> path = BC_.path (a->head (), b->head ());
 bool one_found = false;
 if ( a->is_blabel () && b->size () == 1)
	 one_found = true;
 int deg1 = bcn_degree (a->head ()) - b->size () + 1;
 int deg2 = bcn_degree (b->head ()) - b->size () + 1;

 if ( deg1 > 2 && deg2 > 2)
   	return true;
 if ( deg1 > 2 || deg2 > 2) {
	 if ( one_found )
		 return true;
	 else
		 one_found = true;
  }
   	
 bc_node bcn;
 forall (bcn, path) {
	 if ( bcn == a->parent () || bcn == b->parent ())
		 continue;
	 if ( bcn_degree (bcn) > 2) {
		 if ( one_found )
			 return true;
		 else
			 one_found = true;
         }
	 if ( !is_cutpoint (bcn) && bcn_degree (bcn) > 3)
		 return true;
 }
 if ( one_found )
	 return false;
 else // Ein Block mit dem Grad 0 entsteht
	 return true;
}
//***************************************************************
// Select as first a label with max size
// Select as sec a label with maximal size, which is planar to first
// If there are several such labels then select one of them,
// which satisfies connect_condition with respect to first
bool l_matcher::size_strategy(label &first, label &sec)
{
  first = labels_.head ();
  label l = nil;// l is the label with max size, which is planar to sec
  forall (sec, labels_)  {
	  if (sec == first)
		  continue;
	  if ( l && sec->size () < l->size ()) {
		  sec = l;
		  return true;
	  }
	  if ( l ) {// We have one planar label already
		  if (connect_condition (sec, first) && PLANAR (sec->head (), first->head ()))
			  return true; // a good label is found
	  }
	  else
		  if ( PLANAR (sec->head (), first->head ()))  {
			  if ( connect_condition (sec, first))
				return true;
			  l = sec;
		  }
  }
  if ( !l ) 
	  return false;
  
  sec = l;
  return true;
}
//***************************************************************
// Select as first a label with max size
// Select as sec a label with maximal size, which satisfies
// connect_condition with respect to first (and is planar to first).
// If there are no such labels, then select one label which is planar to
// first

bool l_matcher::concond_strategy(label &first, label &sec)
{
  first = labels_.head ();
  label l = nil;// Das label mit maximaler Gre das Planar zu first ist
  forall (sec, labels_)  {
	  if (sec == first)
		  continue;

	  if (connect_condition (sec, first)) {
		  if ( PLANAR (sec->head (), first->head ()))
			  return true;
	  }
	  else
		  if ( !l && PLANAR (sec->head (), first->head ()))
			  l = sec;
  }
  if (!l)
	  return false;

  sec = l;
  return true;
}
//************************************************************
// This function short up the path in bctree, it 
// start in cutpoint start and goes up as long as all nodes
// have degree of 2 and planaroty is not lost.

// Preconditions : p is already pushed in pendants_

// Return value: a label of p ( it can be an early generated label or
// a label of size 1
label l_matcher::reduce_chain (block p, cutpoint start, label l_old)
{
  if ( l_old && l_old->size () != 1)
	  error_handler (1, "l_matcher::reduce_chain: A label with size() > 1 in reduce_chain!");
  if ( bcn_degree (p) != 1)
	  error_handler (1, "l_matcher::reduce_chain: A non-pendant in reduce_chain!");
  cutpoint last;
  STOP_CAUSE why_stop = follow_path (start, last);
  if ( why_stop == PLANARITY ) {// lost planarity
//We can connect p to an ancestor block immediately
//and apply this procedure to the new pendant
	block new_p = connect_inside_pendant (p, (block)father(last), last);
	if ( l_old )
		delete_label (l_old, false);
	pendants_.push (new_p);
	return reduce_chain (new_p, (cutpoint)father(new_p));
  }

  label l;
  if (why_stop == C_DEGREE || why_stop == ROOT) {
	if ( l_old ) {
		if ( l_old->head () == last ) {
			l_old->stop_cause (why_stop);
			return l_old; //Nothing is changed
		}
		else
			delete_label (l_old, false);
	 }
	l = is_label (last);
	if (l) {
		add_pendant (l, p);
		l->stop_cause (why_stop);
		return l;
	}
	else
		return new_label (last, p, why_stop);
  }
// why_stop = B_DEGREE
  if ( l_old )
	  if (l_old->head_ != last ) {
		  delete_label (l_old, false);
		  l = new_label (last, p, B_DEGREE);
	  }
	  else { // Nothing is changed
		  l = l_old;
		  belongs_to_[l->first_pendant ()] = l;
		  l_old->stop_cause (B_DEGREE);
	  }
  else
	  l = new_label (last, p, B_DEGREE);
	  
  if ( !use_blabels_ || bcn_degree (last) != 2)
	return l;

// Try to combine 2 c-labels (one of them is last) into a new b-label
   block bl = (block)first_adj (last);
   if (bcn_degree (bl) < 3 )
	   bl = (block)last_adj (last);

   if (bcn_degree (bl) != 3)
	   return l;

   label tmp = is_label (bl);
   if (tmp)
//Der Fall: wir haben im Graphen nur einen Block vom Grad 3 und an ihm "hngen"
//drei Pendants. Zwei davon gehren bereits zu einem b-label. Man kann l lschen
//und p zu tmp addieren - dadurch entsteht ein b-label der Gre 3. Dieser Weg
//ist riskant, weil tmp in diesem Fall ein einziges label ist und man wird im nchsten 
//Schritt 2 Kanten zwischen 3 Pendants von tmp addieren, was unter Umstnden 
//Planarittsverlust bedeuten kann. Deswegen die Lsung:
	   return l;

// Check whether one aof cutpoint adj to bl is a c-label
   list<bc_node> adj = BC_.adj_nodes (bl);
   adj.remove (last);
   cutpoint cp1 = (cutpoint)adj.head (), cp2 = (cutpoint)adj.tail ();
   cutpoint new_head = cp1;
   label l2 = is_label (cp2);
   if (!l2)  {
	   l2 = is_label (cp1);
	   if (!l2 )
		   return l;//neither cp1 nor cp2 are labels
	   else {
		   if (l2->size () != 1 || bcn_degree (l2->head ()) != 2)
			   return l;
		   new_head = cp2;
	   }
   }
   else {
	   if ( l2->size () != 1 || bcn_degree (l2->head ()) != 2) {
		   label l1 = is_label (cp1);
		   if ( l1 && l1->size () == 1 && bcn_degree (l1->head ()) == 2 ) {
			   new_head = l2->head ();
			   l2 = l1;
		   }
		   else
			   return l;//
	   }
	   else
		   if ( is_label (cp1) && is_label (cp1)->size () == 1 && bcn_degree (is_label (cp1)->head ()) == 2 )
			   // Wir haben schon versucht aus bl ein b-label zu machen, aber das ging aus 
			   // Planarittsgrnden nicht - es wird auch jetzt nicht gehen
			   return l;
   }
// l2 is the second label that can be evtl. combined with l
// new_head is the third cutpoint adj to bl, it`s the new head
// of b-label that we can get (if planarity tests will be
// successfull).

   if ( PLANAR (last, l2->head (), false, true))
	   if ( PLANAR (last, new_head, false, true))
		   if (PLANAR (l2->head (), new_head, false, true )) {
// Die beiden Kanten mssen auch GLEICHZEITIG planar sein zu einem
// Knoten der mit new_head mit einer Kante verknpft ist (damit
// kein K_3,3 entstehen kann)
			   node n = G_->new_node ();
			   G_->new_edge ( n, vertex (new_head));
			   G_->new_edge (vertex (l2->head ()), n);
			   G_->new_edge (vertex (last), n);
			   bool planar = leda::PLANAR (*G_);
			   nTests_++;
			   G_->del_node (n);
			   if ( planar )
				   return make_blabel (l, l2, bl, new_head);
			   else
				   return l;
		   }
		   else {
// l(last) ist planar zu new_head, l2 aber nicht -> Verknpfe
// die labels vom l und vom l2 miteinander, es entsteht ein 
// neues Pendant new_p und rufe reduce_chain fr new_p auf.
			   block new_p = connect_labels (l, l2);
			   delete_label (l, false);
			   pendants_.push (new_p);

			   if ( dfs_number (bl) == 1 )
// bl war root, jetzt ist es Pendant geworden -> aktualisiere DFS-Numerierung
					update_dfs_numbering ();

			   return reduce_chain (new_p, new_head);
		   }
	   else {
// Analog zum letzten Fall
		   block new_p = connect_labels (l, l2);
		   delete_label (l, false);
		   pendants_.push (new_p);

		   if ( dfs_number (bl) == 1 )
// bl war root, jetzt ist es Pendant geworden -> aktualisiere DFS-Numerierung
			   update_dfs_numbering ();

		   return reduce_chain (new_p, new_head);
		}
	 else  // PLANAR (last, l2->head ()) == false
		if ( PLANAR (last, new_head, false, true)) 
			if ( !PLANAR (l2->head (), new_head, false, true)) {
// l2 ist weder zu l noch zu new_head planar = inside_ label
// lsche l2 und versuche l zu erweitern ( degree (bl) ist jetzt 2
				connect_inside_label (l2, bl);
				if (dfs_number (bl) == 1)// bl war ein root == alle Schnittknoten im Graphen,
// haben den Grad 2 und jetzt auch degree (bl) = 2
					update_dfs_numbering ();
				return reduce_chain (p, l->head (), l);
			}
			else
//l und l2 sind nicht planar zueinander aber beide sind planar zu 
// new_head: ndere nichts, lasse zwei c-labels sein
				return l;
		else   {// last (l) ist ein inside label
				connect_inside_label (l, bl);
				if (dfs_number (bl) == 1)
// bl war ein root == alle Schnittknoten im Graphen,
// haben den Grad 2 und jetzt auch degree (bl) = 2
					update_dfs_numbering ();
				return reduce_chain (l2->first_pendant(), l2->head (), l2);
		}
}
//***********************************************************
// 0) Add edges between pendants of l (join them to one block)
// 1) Reduce size of l to 1
// 2) Update BC_
block l_matcher::join_pendants(label& l)
{
	block bl2, bl1 = l->first_pendant ();

    forall (bl2, l->pendants_) {
    	if ( bl2 ==	bl1) continue; // Pass first pendant
		connect_pendants (bl1, bl2);
    	bl1 = bl2;
    }
	block b;
	forall (b, l->pendants_)
		delete_pendant (b);
	if ( l->is_clabel ()) {
		b = BC_.union_pendants (l->head (), l->pendants_);
		l->delete_all_pendants ();
		if ( bcn_degree (b) == 1 )
			add_pendant (l, b);// Problem mit STOP_CAUSE
		return b;
	}
// l ist ein b-label
	b = BC_.union_nodes (l->first_pendant (), l->last_pendant ());
	if ( l->size () == 3) {
		b = BC_.union_nodes (b, l->get_pendant (1));
		l->delete_all_pendants ();
		return b;//Eigentlich terminate
	}
	l->delete_all_pendants ();

	change2clabel (l, l->head ());//STOP_CAUSE ??
	if ( bcn_degree (b) == 1)
		add_pendant (l, b);
	
	return b;
}
//*************************************************************
label l_matcher::is_head(cutpoint cp)
{
	if ( is_label_.defined (cp))  {
		label l = labels_.inf (is_label_[cp]);
		if ( l->head () != cp)
			error_handler (1, "l_matcher::is_head: Die Eintrge in label_struct und is_label_ stimmen nicht berein");
		return l;
	}
	return nil;
}
//**************************************************************
label l_matcher::new_label(cutpoint cp, block p, STOP_CAUSE why_stop)
{
// Dont push p in pendants_ 
	label l = new label_struct (cp, p, why_stop);
	belongs_to_[p] = l;
	is_label_[cp] = labels_.append (l);
	return l;
}
//*************************************************************
block l_matcher::connect_inside_pendant(block p, block anc, cutpoint c)
{
 edge e = G_->new_edge (BC_.adj_to_cutpoint_node (p), BC_.adj_to_cutpoint_node (anc, c));
 result_.push (e);
 delete_pendant (p);
 return BC_.union_nodes (p, anc);
}
//*************************************************************
void l_matcher::delete_label(label& l, bool remove_pendants)
{
  list_item li = is_label_[l->parent ()];
  labels_.del_item (li);
  is_label_.undefine (l->parent ());
  
  if ( !remove_pendants ) {
	  delete l;
	  l = nil;
	  return;
  }

  block p;
  forall ( p, l->pendants_)
	  delete_pendant (p);
  delete l;
  l = nil;
}
//***********************************************************
// 1) Adds edges between pendants of first and sec
// 2) Removes connected pendants
// 3) Reduces size of first
// 4) Deletes sec
// 5) Evtl. tranfors first to a c-label
// 6) Updates bctree
// 7) Return value: the new arised block
block l_matcher::connect_labels(label first, label sec)
{
	block p1, p2;
	list<block> f_connected;//pendants of first that are matched
	int n = 0;

	forall (p2, sec->pendants_ )  {
		p1 = first->get_pendant (n);
		f_connected.push (p1);
		connect_pendants (p1, p2);
		n++;
	}
	block first_bundle = nil, sec_bundle = nil;
	if ( sec->size () == 2 && use_blabels_ ) {
		if ( sec->is_blabel ())
			sec_bundle = BC_.union_nodes (sec->first_pendant (), sec->last_pendant ());
		if ( first->is_blabel ())
			first_bundle = BC_.union_nodes (first->first_pendant (), first->last_pendant ());
	}
	if ( !first_bundle )
		first_bundle = BC_.union_pendants (first->head (), f_connected);
	if ( !sec_bundle )
		sec_bundle = BC_.union_pendants (sec->head(), sec->pendants_);

	block new_block = BC_.union_nodes (first_bundle, sec_bundle);
	delete_label (sec);
	decrease_size (first, f_connected);
	if ( first->size () == 1 && first->is_blabel ())
		change2clabel (first, (cutpoint)find_last_before (first->first_pendant (), new_block));
	return new_block;
}
//************************************************************
// Removes pendants in the 2. argument from l and all other members
// (pendants_, ...).Restore the size order in labels_ 
// by moving of l to the end of list
void l_matcher::decrease_size (label l, list<block> pendants)
{
  block p;
  forall (p, pendants) {
	  l->remove_pendant (p);
	  delete_pendant (p);
  }
  if ( l->size () == 0)
	  return;

  labels_.del_item (is_label_[l->parent ()]);
  is_label_[l->parent ()] = insert_label (l);
}
//*************************************************************
label l_matcher::add_pendant(label l, block p)
{
  belongs_to_[p] = l;
  l->add_pendant (p);
  bc_node bcn = l->parent ();

  labels_.del_item (is_label_[bcn]);
  is_label_[bcn] = insert_label (l);
  return l;
}
//*************************************************************
// This function combines two c-labels of size 1
// to a new b_label and returns it
label l_matcher::make_blabel(label c1, label c2, block b, cutpoint head)
{
	label l = new label_struct (c1->first_pendant (), c2->first_pendant (), b, head);
	delete_label (c2, false);
	delete_label (c1, false);
	is_label_[b] = insert_label (l);
	belongs_to_[l->first_pendant ()] = l;
	belongs_to_[l->last_pendant ()] = l;
	return l;
}
//**************************************************************
list_item l_matcher::insert_label(label l)
{
	if ( l->size () == 1 )
		return labels_.append (l);
	list_item li_next;
	forall_items (li_next, labels_)
		if ( (labels_.inf (li_next))->size () <= l->size ())
			return labels_.insert (l, li_next, leda::before);

	return labels_.append (l);//all labels are alarger then l
}
//**************************************************************
list<edge> l_matcher::terminate ()
{
	planar_.clear ();

	block p;
	forall_defined (p, belongs_to_)
		belongs_to_.undefine (p);
	while ( !labels_.empty ())
		delete labels_.pop ();

	bc_node bcn;
	forall_defined (bcn, is_label_)
		is_label_.undefine (bcn);

	pendants_.clear ();
	return result_;
}
//*************************************************************
bool l_matcher::PLANAR(cutpoint c1, cutpoint c2, bool explore_planar, bool add_infos)
{
  if ( vertex (c1) == vertex (c2) || planar_(c1, c2) == PLANAR_ || planar_(c2, c1) == PLANAR_)
	  return true;

  if ( planar_(c1, c2) == N_PLANAR || planar_(c1, c2) == N_PLANAR_CHECKED)
	  return false;

  if ( planar_(c2, c1) == N_PLANAR || planar_(c2, c1) == N_PLANAR_CHECKED)
	  return false;

  if ( are_adj (vertex (c1),vertex (c2))) {
	  planar_(c1, c2) = PLANAR_;
	  return true;
  }

  edge e = G_->new_edge (vertex (c1), vertex (c2));
  bool planar = leda::PLANAR (*G_, false);
  nTests_++;
  G_->del_edge (e);

  if ( !planar ) {
	  if ( add_infos )
		  planar_(c1, c2) = N_PLANAR;
	  if ( explore_planar )
		  explore_planarity (c1, c2);
	  return false;
  }
  return true;
}
//************************************************************
bool l_matcher::are_adj(node v, node w)
{
 edge e;
 forall_inout_edges ( e, v )
 	if ( opposite ( v, e ) == w )
 		return true;
 return false;
}
//*************************************************************
// Diese Funktion verfolgt den Pfad beginnend im Pendant p
// solange bis der aktuelle Knoten nicht bcn ist.
// Alle Knoten im Pfad mssen den Grad 2 haben
bc_node l_matcher::find_last_before (block p, bc_node bcn)
{
  bc_node b = p;
  while ( father (b) != bcn)
	  b = father (b);
  return b;
}
//*************************************************************
void l_matcher::check_DFS_numbers(bc_node bcn, bc_node parent)
{
  if ( bcn_degree (bcn) == 1 )
	  return;

  bc_node bcnod;
  list<bc_node> adj = BC_.adj_nodes (bcn);
  forall (bcnod, adj ) {
	  if ( bcnod != parent && dfs_number(bcnod) <= dfs_number (bcn))
		  error_handler (1, "l_matcher::check_DFS_numbers: DFS-Numerierung stimmt nicht");
	  if ( bcnod != parent )
		  check_DFS_numbers (bcnod, bcn);
  }
}
//**************************************************************************************
bool l_matcher::try_extend(label& l)
{
  if ( l->size () > 1 )
	  return false;
  if ( bcn_degree (l->head ()) == 2 && BC_.root () != l->head ()) {
	  if ( bcn_degree (father (l->head ())) == 2) {
		  l = reduce_chain (l->first_pendant (), l->head (), l);
		  return true;
	  }
	  if ( use_blabels_ && bcn_degree (father (l->head ())) == 3) {
		  l = reduce_chain (l->first_pendant (), l->head (), l);
		  return true;
	  }
  }
  return false;	  
}
//*************************************************************************************
void l_matcher::change2clabel(label& l, cutpoint new_head)
{
  	list_item li = is_label_[l->parent ()];
	is_label_.undefine (l->parent ());
	l->change2clabel (new_head);
	l->stop_cause (B_DEGREE); // ???
	is_label_[new_head] = li;
}
//****************************************************************
void l_matcher::explore_planarity(cutpoint c1, cutpoint c2)
{
// eine Kante c1 <-> c2 ist nicht planar, D.h. auf dem pfad (c1, c2)
// gibt es groe Blcke, die Planaritt "zerstren
  list<bc_node> pfad = BC_.path (c1, c2);
  bc_node bcn;
  cutpoint left_cp = nil, right_cp = nil;
  forall (bcn , pfad)
	  if ( !is_cutpoint (bcn) && number_of_nodes (bcn) > 4) {
// bcn is possibly a big non planar block
		  list<bc_node> adj = BC_.adj_nodes (bcn);
		  bc_node adj_cp;
// Search for two cutpoints adj to bcn and belonging to pfad
		  forall ( adj_cp, adj)
			  if ( pfad.search (adj_cp)) {
				  if ( left_cp ) {
					  right_cp = (cutpoint)adj_cp;
					  break;
				  }
				  else
					  left_cp = (cutpoint)adj_cp;
			  }
//left_cp and right_cp are found cutpoints
// Now check whether the edge (left_cp, right_cp) are non planar
			  if ( !PLANAR (left_cp, right_cp, false, true)) {
// Collect in following two lists all cutpoints belonging to 
// to subtrees rooted in left_cp and right_cp. bcn is now the
// ancestor of both cutpoints (is tabu node)
				  list<cutpoint> lefts, rights;
				  explore_subtree (left_cp, (block)bcn, lefts);
				  explore_subtree (right_cp, (block)bcn, rights);
				  cutpoint l_cp, r_cp;
// Each cutpoint in lefts is non planar to each cp in rights
				  forall ( l_cp, lefts)
					  forall (r_cp, rights) {
							planar_(l_cp, r_cp) = N_PLANAR;
#ifndef _DEBUG
							edge e = G_->new_edge (vertex (l_cp), vertex (r_cp));
							if (leda::PLANAR (*G_))
								error_handler (1, "explore_planarity sammelt falsche Informationen");
							G_->del_edge (e);
							planar_(l_cp, r_cp) = N_PLANAR_CHECKED;
#endif
				  }
			  }
			  left_cp = right_cp = nil;
	  }
}
//********************************************************************
void l_matcher::explore_subtree(bc_node start, bc_node tabu, list<cutpoint>& container)
{
	if ( bcn_degree (start) == 1 )
		return;
	if ( is_cutpoint (start)) {
		if ( bcn_degree (start) > 2 )
			container.push ((cutpoint)start);
		else
			if ( number_of_nodes (first_adj (start)) > 4 || number_of_nodes (last_adj (start)) > 4)
				container.push ((cutpoint)start);
        }

	list<bc_node> adj = BC_.adj_nodes (start);
	bc_node bcn;
	forall ( bcn, adj ) {
		if ( bcn == tabu || bcn_degree (bcn) == 1 )
			continue;
		explore_subtree (bcn, start, container);
	}
}
//***************************************************************************
int l_matcher::number_of_nodes(bc_node b)
{
   if ( is_cutpoint (b))
	   return 1;
   else
	   return bcn_degree (b) + ::number_of_nodes (b);
}
//******************************************************************
block l_matcher::connect_inside_label(label& l, block anc)
{
// if ( l->size () != 1)	 error_handler ("Fehler in connect_inside_label (label l): l->size () > 1", 1);
 edge e = G_->new_edge (BC_.adj_to_cutpoint_node (l->first_pendant()), BC_.adj_to_cutpoint_node (anc, l->head ()));
 result_.push (e);
 block result = BC_.union_nodes (l->first_pendant (), anc);
 delete_label (l);
 return result;
}
//**********************************************************************
void l_matcher::update_dfs_numbering()
{
	bc_node old_root = BC_.root ();
	bc_node bcn = BC_.bcnode_maximal_degree ();
	if ( bcn_degree (bcn) > 2 || !is_cutpoint (old_root))
		BC_.DFS_num (bcn);
}
//*********************************************************************
int l_matcher::nPlanarityTests ()
{
	return nTests_;
}
