/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGraphWinInterface.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                     CGraphWinInterface.c
//
//    author: Dirk Luetke-Huettmann
//    date:   November 1998
//-------------------------------------------------------------------


#include <AGD/CGGridLayout.h>
#include <AGD/CGraphWinInterface.h>
#include <AGD/CGLayoutInterface.h>

using namespace leda;

CGraphWinInterface::CGraphWinInterface (CGraphWin& CGW) :
	CGLayoutInterface(CGW.get_clustergraph())
{
	cgw_p = &CGW;
	rc_p = cgw_p->get_clustergraph().get_rootcluster();
	old_flush = cgw_p->set_flush(false);
}

void CGraphWinInterface::init(CGraphWin &CGW)
{
  cgw_p = &CGW;
  rc_p = cgw_p->get_clustergraph().get_rootcluster();
  old_flush = cgw_p->set_flush(false);
}

CGraphWinInterface::~CGraphWinInterface()
{
  cgw_p->set_flush(old_flush);
}

void CGraphWinInterface::update()
{
  cgw_p->set_flush(true);
  cgw_p->redraw();
  cgw_p->fill_window();
  cgw_p->set_flush(false);

}

void CGraphWinInterface::fill_window(void)
{
  cgw_p->fill_window();
}

void CGraphWinInterface::set_cluster_region(Cluster* cl, 
					    double x,
					    double y,
					    double width,
					    double height)
{
  list<point> cl_bends;
  point lu, ru, ro, lo;

  cluster_info* cli=cgw_p->get_cluster_info(cl);
  if(cli == nil) {
	  error_handler(1, "CGraphWinInterface::fill_window: no cluster info.");
  }
  
  cli->set_x(x);
  cli->set_y(y);
  cli->set_width(width);
  cli->set_height(height);

  lu = point(x,y);
  ru = point(x+width, y);
  ro = point(x+width, y+height);
  lo = point(x, y+height);

  cl_bends.append(lu);
  cl_bends.append(ru);
  cl_bends.append(ro);
  cl_bends.append(lo);
  
  gen_polygon cl_region(cl_bends, gen_polygon::SIMPLE, gen_polygon::DISREGARD_ORIENTATION);

  cli->set_region(cl_region);
}
