/*******************************************************************************
+
+  AGD 1.2
+
+
+  ClusterGraph.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


#include <AGD/ClusterGraph.h>

using namespace leda;

//////////////////////////////////////////////////////////////////////
//
// creates only a new empty clustertree with a root-cluster

ClusterGraph::ClusterGraph() {
  rc_p = new Cluster();
  cl_node_map.clear();
}

//////////////////////////////////////////////////////////////////////////////
//
// default destructor

ClusterGraph::~ClusterGraph() {
  if(rc_p != NULL) empty_cluster_tree(rc_p);
}

//////////////////////////////////////////////////////////////////////////////
//
// clears the clustertree

void ClusterGraph::clear_clustertree(void) {
  node v;
  if(rc_p != NULL) empty_cluster_tree(rc_p);
  rc_p = new Cluster();
  cl_node_map.clear();
  forall_nodes(v, *this) cl_node_map[v] = rc_p;
}

//////////////////////////////////////////////////////////////////////////////
//
// creates a new node and makes it a member of cluster <cl>

node ClusterGraph::new_node(Cluster* cl) {
  node v = graph::new_node();
  rc_p->remove_node(v);
  cl->add_node(v);
  cl_node_map[v] = cl;
  return(v);
}

//////////////////////////////////////////////////////////////////////////////
//
// add a new cluster to the clustertree with the rootcluster as supercluster

Cluster* ClusterGraph::new_cluster(void) {
  Cluster* cl = new Cluster();
  cl->set_supercluster(rc_p);
  rc_p->add_subcluster(cl);
  
  return cl;
}

//////////////////////////////////////////////////////////////////////////////
//
// add a new cluster to the clustertree with supercluster <cl_parent>

Cluster* ClusterGraph::new_cluster(Cluster* cl_parent) {
  Cluster* cl = new Cluster();
  cl->set_supercluster(cl_parent);
  cl_parent->add_subcluster(cl);

  return cl;
}

//////////////////////////////////////////////////////////////////////////////
//
// makes the clustertree the empty clustertree

void ClusterGraph::empty_cluster_tree(Cluster* cl_parent) {
  list<Cluster*> cl_children = cl_parent->get_subclusters() ;
  Cluster* cl_node; 

  forall(cl_node, cl_children) {
    if(cl_node != NULL) {
      if(!cl_node->get_subclusters().empty()) empty_cluster_tree(cl_node);
      else delete cl_node;
    }
  }

  if(cl_parent != rc_p) { 
    if(cl_parent != NULL) delete cl_parent;
  } else {
    cl_parent->clear_subclusters();
    cl_parent->clear_clusternodes();
  }
}

//////////////////////////////////////////////////////////////////////////////
//
// returns the cluster node v belongs too

Cluster* ClusterGraph::cluster_of(node v) const {
  return cl_node_map[v];
}

//////////////////////////////////////////////////////////////////////////////
//
// post node handler to catch up the creating of a node

void ClusterGraph::post_new_node_handler(node v) {
  move_node(rc_p, v);
}

//////////////////////////////////////////////////////////////////////////////
//
// pre node handler to catch up the deletion of a node

void ClusterGraph::pre_del_node_handler(node v) {
  remove_node(v);
}


void ClusterGraph::_all_clusters(Cluster *c, list<Cluster *> &all) const
{
	const list<Cluster *> &childs = c->get_subclusters();
	Cluster *child;

	forall(child,childs)
		_all_clusters(child,all);
	
	if(c->get_supercluster() != NULL) 
	  all.append(c);	

}

bool ClusterGraph::is_member(Cluster* cl, leda::node v) const {
  leda::list<Cluster*> subcl;
  Cluster* n_cl, *s_cl;
  
  n_cl = cluster_of(v);
  if(n_cl == cl) return(true);
  else {
    subcl = all_subcluster(cl);
    forall(s_cl, subcl) 
      if(s_cl == n_cl) return(true);
  }
  return(false);
}
