/*******************************************************************************
+
+  AGD 1.2
+
+
+  Keymapper.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         Keymapper.cpp
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#include <AGD/DefaultKeymapper.h>
#include <AGD/AGD.h>

using namespace leda;


// define hash function for strings (used by Keymapper)
// no longer necessary since LEDA-5.0
/*
LEDA_BEGIN_NAMESPACE
int Hash (const leda::string& s)
{
	int i, sum;

	for (i = 0, sum = 0; i < s.length(); i++)
		sum += (int)(s[i]);
	return sum;
}
LEDA_END_NAMESPACE
*/

//-------------------------------------------------------------------
//                            AgdKey
//-------------------------------------------------------------------

AgdKey::AgdKey(key::Predefined k)
{
	keyp = AGD::predefined(k).keyp;
}

AgdKey::AgdKey(const string &str)
{
	keyp = AGD::get_key(str).keyp;
}


//-------------------------------------------------------------------
//                            Keymapper
//-------------------------------------------------------------------

Keymapper::Keymapper()
{
	stringToKey = h_array<string,AgdKey>();
}

Keymapper::~Keymapper()
{
	AgdKey k;

	forall (k, _keys)
		delete k.keyp;
}

AgdKey Keymapper::get_key (const string& str)
{
	AgdKey k = stringToKey[str];

	if (!k.valid()) { // str not in hash table ?
		k = new AgdKeyStruct (str, _keys.length()); // create a new key
		_keys.append (k);
		stringToKey[str] = k; // put str into hash table
	}

	return k;
}


//-------------------------------------------------------------------
//                         DefaultKeymapper
//-------------------------------------------------------------------

DefaultKeymapper::DefaultKeymapper()
{
	predef = new AgdKey[key::MAX_PREDEFINED+1];
	predef[key::planar]          = get_key("planar");
	predef[key::simple]          = get_key("simple");
	predef[key::connected]       = get_key("connected");
	predef[key::biconnected]     = get_key("biconnected");
	predef[key::triconnected]    = get_key("triconnected");
	predef[key::four_graph]      = get_key("four-graph");
	predef[key::no_self_loops]   = get_key("no self loops");
	predef[key::directed]        = get_key("directed");
	predef[key::undirected]      = get_key("undirected");
	predef[key::no_reversals]    = get_key("no reversals");
	predef[key::acyclic]         = get_key("acyclic");
	predef[key::tree]            = get_key("tree");
	predef[key::forest]          = get_key("forest");
	predef[key::single_source]   = get_key("single source");
	predef[key::single_sink]     = get_key("single sink");
	predef[key::bipartite]       = get_key("bipartite");
	predef[key::straight_line]   = get_key("straight-line");
	predef[key::orthogonal]      = get_key("orthogonal");
	predef[key::no_crossings]    = get_key("no crossings");
	predef[key::upward_planar]   = get_key("upward planar");
	predef[key::convex]          = get_key("convex");
	predef[key::maximal_planar]  = get_key("maximal planar");
	predef[key::maximum_planar]  = get_key("maximum planar");
	predef[key::maximal_acyclic] = get_key("maximal acyclic");
	predef[key::maximum_acyclic] = get_key("maximum acyclic");
	predef[key::reduced_acyclic] = get_key("reduced acyclic");
	predef[key::c_planar]        = get_key("c-planar");
	predef[key::st_graph]        = get_key("st-graph");
	predef[key::st_planar]       = get_key("st-planar");
	predef[key::planar_map_callable] = get_key("planar-map-callable");
	predef[key::PlaneGraphCopy_callable] = get_key("PlaneGraphCopy-callable");
	predef[key::MAX_PREDEFINED]  = get_key("MAX PREDEFINED");
}
