/*******************************************************************************
+
+  AGD 1.2
+
+
+  PSInterface.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      PSInterface.c
//
//    author: David Alberts
//    date:   December 1997
//-------------------------------------------------------------------


#include<AGD/PSInterface.h>
#include <LEDA/graphics/ps_file.h>

using namespace leda;


PSInterface::PSInterface (const graph &G, string name) :
	LayoutInterface(G)
{
	_init(G,name);
}

void PSInterface::init(const graph &G, string name)
{
	set_graph(G);
	_init(G,name);
}

void PSInterface::_init(const graph &G, const string &name)
{
	filename = name;
	xcoord.init(G);
	ycoord.init(G);
	node_width.init(G,5);
	node_height.init(G,5);
	bends.init(G);
	sanch.init(G);
	tanch.init(G);
}

void PSInterface::update()
// gets automatically called immediately after a layout
// algorithm is executed, converts collected data into PostScript
{
  // NOTE: In PostScript y coordinates increase from the bottom
  //       of the page to the top. AGD uses the screen-oriented
  //       model of increasing y coordinates from top to bottom.
  //       Thus we transform the y coordinates.

#if __LEDA__ >= 380
  const double ASPECT = PS_HEIGHT/PS_WIDTH;
#else
  const double ASPECT = HEIGHT/WIDTH;
#endif
  
  const graph &G = get_graph();

  ps_file psf(filename);  // underlying ps_file
  psf.set_draw_bb(false); // don't draw bounding box

  // --- calculate clipping parameters --- //
  DRect bb(bounding_box());
  double l = bb.left();
  double r = bb.right();
  double t = bb.top();
  double b = bb.bottom();
  double aspect = (t-b)/(r-l);            // bottom >= top in AGD!
  if(aspect <= ASPECT) psf.init(l,r,b);
  else                       psf.init(l,r*(aspect/(ASPECT)),b);

  // --- draw edges --- //
  edge e;
  forall_edges(e,G)
  {
    DPoint dp1,dp2;

    // handle source anchor
    node s = source(e);
    double source_x = xcoord[s] + (sanch[e].xcoord() * node_width[s]/2);
    double source_y = ycoord[s] + (sanch[e].ycoord() * node_height[s]/2);
    dp1 = DPoint(source_x,source_y); // beginning of current edge
                                     // without y transformation

    // draw edge segments determined by its bends
    point p1,p2;
    forall(dp2,bends[e])
    {
      p1 = point(dp1.xcoord(),/*b-*/dp1.ycoord());  // last visited point
      p2 = point(dp2.xcoord(),/*b-*/dp2.ycoord());  // take care of y coordinates
      psf.draw_segment(p1,p2);
      dp1 = dp2;
    }

    // handle target anchor
    node t = target(e);
    p1 = point(dp1.xcoord(),/*b-*/dp1.ycoord());    // last visited point
    double target_x = xcoord[t] + (tanch[e].xcoord() * node_width[t]/2);
    double target_y = ycoord[t] + (tanch[e].ycoord() * node_height[t]/2);
    p2 = point(target_x,/*b-*/target_y);
    psf.draw_segment(p1,p2);
  }

  // --- draw nodes --- //
  node v;
  forall_nodes(v,G)
  {
    double lower = ycoord[v] - node_height[v]/2;
    double left_ = xcoord[v] - node_width[v]/2;
    point ll(left_,lower);
    double upper  = ycoord[v] + node_height[v]/2;
    double right_ = xcoord[v] + node_width[v]/2;
    point ur(right_,upper);
    psf.draw_box(ll,ur);
  }
}
