/*******************************************************************************
+
+  AGD 1.2
+
+
+  PlaneClusterGraphCopy.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                          PlaneClusterGraphCopy.c
//
//    author: Dirk Luetke-Huettmann
//    date:   june 1999
//-------------------------------------------------------------------


#include <AGD/PlaneClusterGraphCopy.h>

using namespace leda;

PlaneClusterGraphCopy::PlaneClusterGraphCopy() 	
  : PlaneGraphCopy() 
{
  cg_p = new ClusterGraph();
  rc_p = cg_p->get_rootcluster();
}

PlaneClusterGraphCopy::PlaneClusterGraphCopy (const PlaneClusterGraphCopy& PCG) 
  : PlaneGraphCopy(PCG)
{
  cg_p = new ClusterGraph();
  rc_p = cg_p->get_rootcluster();
  init_clustertree(PCG.get_rootcluster());
}

PlaneClusterGraphCopy::PlaneClusterGraphCopy(const ClusterGraph &CG) 
  : PlaneGraphCopy(CG)
{ 
  cg_p = new ClusterGraph();
  rc_p = cg_p->get_rootcluster();
  init_clustertree(CG.get_rootcluster());
}

PlaneClusterGraphCopy::PlaneClusterGraphCopy(const ClusterGraph &CG, list<leda::edge>& E) 
  : PlaneGraphCopy (CG,E)
{
  cg_p = new ClusterGraph();
  rc_p = cg_p->get_rootcluster();
  init_clustertree(CG.get_rootcluster());
}

PlaneClusterGraphCopy::~PlaneClusterGraphCopy()
{
  delete cg_p;
}


PlaneClusterGraphCopy& PlaneClusterGraphCopy::operator= (const PlaneClusterGraphCopy& PCG)
{
  PlaneGraphCopy::operator=(PCG);
  init_clustertree(PCG.get_rootcluster());   
  return *this;
}

node PlaneClusterGraphCopy::new_node(Cluster* cl) {
  return  cg_p->new_node(cl);
}

Cluster* PlaneClusterGraphCopy::new_cluster(void) {
  return cg_p->new_cluster();
}

Cluster* PlaneClusterGraphCopy::new_cluster(Cluster* cl_parent) {
  return cg_p->new_cluster(cl_parent);
}

Cluster* PlaneClusterGraphCopy::cluster_of(node v) const {
  return cg_p->cluster_of(v);
}

void PlaneClusterGraphCopy::post_new_node_handler(node v) {
  cg_p->move_node(rc_p, v);
}

void PlaneClusterGraphCopy::pre_del_node_handler(node v) {
  cg_p->remove_node(v);
}

Cluster* PlaneClusterGraphCopy::original(Cluster* cl) {
  return cl_orig[cl];
}

Cluster* PlaneClusterGraphCopy::copy(Cluster* cl) {
  return cl_copy[cl];
}

void PlaneClusterGraphCopy::init_clustertree(Cluster* o_cl_parent) {
  list<Cluster*> o_cl_children = o_cl_parent->get_subclusters();
  list<node> o_cl_nodes;
  list<edge> o_cl_incident_edges;
  Cluster* o_cl, *cl;

  node v, w;
  edge e, e_t;

  if(o_cl_parent->get_supercluster() == NULL) {
    cl_copy[o_cl_parent] = rc_p;
    cl_orig[rc_p] = o_cl_parent;
    o_cl_nodes = o_cl_parent->get_nodes();
    forall(v, o_cl_nodes) cg_p->move_node(rc_p, copy(v));
  } else {
    cl = new_cluster(cl_copy[o_cl_parent->get_supercluster()]);
    cl_copy[o_cl_parent] = cl;
    cl_orig[cl] = o_cl_parent;
    o_cl_nodes = o_cl_parent->get_nodes();	  
    forall(v, o_cl_nodes) cg_p->move_node(cl, copy(v));
    o_cl_incident_edges = o_cl_parent->get_edge_order();
    forall(e, o_cl_incident_edges) {
      v = original().source(e);
      w = original().target(e);
      forall_adj_edges(e_t, copy(v))
        if(copy(w) == target(e_t)) cl->append_to_edge_order(e_t);
    }
  }
  
  forall(o_cl, o_cl_children) {
    if(!o_cl->get_subclusters().empty()) init_clustertree(o_cl);
    else {
      cl = new_cluster(cl_copy[o_cl_parent]);
      cl_copy[o_cl] = cl;
      cl_orig[cl] = o_cl;
      o_cl_nodes = o_cl->get_nodes();
      forall(v, o_cl_nodes) cg_p->move_node(cl, copy(v));
      o_cl_incident_edges = o_cl->get_edge_order();
      forall(e, o_cl_incident_edges) {
	v = original().source(e);
	w = original().target(e);
        forall_adj_edges(e_t, copy(v)) 
	  if(copy(w) == target(e_t)) cl->append_to_edge_order(e_t);
      }
    }
  }
}

void PlaneClusterGraphCopy::_all_clusters(Cluster *c, list<Cluster *> &all) const
{
  const list<Cluster *> &childs = c->get_subclusters();
  Cluster *child;
  
  forall(child,childs)
    _all_clusters(child,all);

  if(c->get_supercluster() != NULL)
    all.append(c);
}

bool PlaneClusterGraphCopy::is_member(Cluster* cl, leda::node v) const {
  leda::list<Cluster*> subcl;
  Cluster* n_cl, *s_cl;
  
  n_cl = cluster_of(v);
  if(n_cl == cl) return(true);
  else {
    subcl = all_subcluster(cl);
    forall(s_cl, subcl) 
      if(s_cl == n_cl) return(true);
  }
  return(false);
}
