/*******************************************************************************
+
+  AGD 1.2
+
+
+  concept.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


#include <AGD/CGGridLayoutModule.h>
#include <AGD/CGPlaneGraphCopyGridConcept.h>
#include <AGD/impl/CGGraphCopyGridConceptBase.h>
#include <AGD/PlaneGraphCopy.h>
#include <AGD/PlaneClusterGraphCopy.h>
#include <AGD/GraphCopyGLI.h>
#include <AGD/GraphGLI.h>
#include <AGD/MaxAllMapper.h>
#include <AGD/CGGridLayout.h>

#include <AGD/ClusterGraph.h>

using namespace leda;
//--------------------------------------------------------------------
//                       CGGraphCopyGridConceptBase
//--------------------------------------------------------------------

void CGGraphCopyGridConceptBase::transform (CGGridLayout &gl)
{
  CGGridLayout *dest = (CGGridLayout *) &gl;
  const graph& G = _gcp->original();

  node v;
  forall_nodes (v,G) {
    node vc = _gcp->copy(v);
    dest->x(v) = _glp->x(vc);
    dest->y(v) = _glp->y(vc);
    dest->width (v, _glp->width (vc));
    dest->height(v, _glp->height(vc));
  }
  
  edge e;
  forall_edges (e,G) {
    dest->bends(e) = get_bends(e);
  }

  Cluster* cl;
  list<Cluster*> cl_list = _pgcp->all_clusters();

  forall(cl, cl_list) {
    Cluster* o_cl = _pgcp->original(cl);
    dest->x(o_cl, _cgglp->x(cl));
    dest->y(o_cl, _cgglp->y(cl));
    dest->width(o_cl, _cgglp->width(cl));
    dest->height(o_cl, _cgglp->height(cl));

  }
}

//-------------------------------------------------------------------
//                       CGPlaneGraphCopyGridConcept
//-------------------------------------------------------------------

// changed GridLayout to CGGridLayout

CGPlaneGraphCopyGridConcept::CGPlaneGraphCopyGridConcept (const ClusterGraph &CG,
							  CGLayoutInterface &) : _ext_face(nil), new_graph(true), new_layout(true)
{
  _gcp = _pgcp = new PlaneClusterGraphCopy(CG);
  _glp = _cgglp = new CGGridLayout(*_pgcp);
}
	
CGPlaneGraphCopyGridConcept::CGPlaneGraphCopyGridConcept (const ClusterGraph &CG,
							  face f, 
							  CGLayoutInterface &) : new_graph(true), new_layout(true)
{
   _gcp = _pgcp = new PlaneClusterGraphCopy(CG);
   _glp =_cgglp = new CGGridLayout(*_pgcp);
   _ext_face = (f != nil) ? _pgcp->copy_of_face(f) : nil;
}

CGPlaneGraphCopyGridConcept::CGPlaneGraphCopyGridConcept (const ClusterGraph &CG,
							  CGGridLayout &) : _ext_face(nil), new_graph(true), new_layout(true)
{
  _gcp = _pgcp = new PlaneClusterGraphCopy(CG);
  _glp =_cgglp = new CGGridLayout(*_pgcp);
}

CGPlaneGraphCopyGridConcept::CGPlaneGraphCopyGridConcept (const ClusterGraph &CG,
							  face f, 
							  CGGridLayout &) : new_graph(true), new_layout(true)
{
  _gcp = _pgcp = new PlaneClusterGraphCopy(CG);
  _glp =_cgglp = new CGGridLayout(*_pgcp);
  _ext_face = (f != nil) ? _pgcp->copy_of_face(f) : nil;
}

CGPlaneGraphCopyGridConcept::~CGPlaneGraphCopyGridConcept()
{
  if (new_graph)  delete _pgcp;
  if (new_layout) delete _glp;
}


void CGPlaneGraphCopyGridConcept::get_node_grid_size (GridCoordinateMapper &gcm,
						      const LayoutInterface &A)
{
  if (new_graph) {
    GraphCopyGLI gli(*this);
    gcm.get_node_grid_size (gli, A);
  } else {
    GraphGLI gli(*_glp);
    gcm.get_node_grid_size (gli, A);
  }
}

void CGPlaneGraphCopyGridConcept::update (GridCoordinateMapper &gcm,
					  LayoutInterface &A)
{
  if (new_graph) {
    GraphCopyGLI gli(*this);
    gcm.call (gli, A);
  } else {
    GraphGLI gli(*_glp);
    gcm.call (gli, A);
  }
}

void CGPlaneGraphCopyGridConcept::update (CGGridLayout &gl)
{
  if (!new_layout) return;
  
  transform (gl);
}

//-------------------------------------------------------------------
//                           LayoutModule
//-------------------------------------------------------------------

const int CGLayoutModule::in_graph = 1;
const int CGLayoutModule::out_layout = 1;


//-------------------------------------------------------------------
//                         GridLayoutModule
//-------------------------------------------------------------------

CGGridLayoutModule::CGGridLayoutModule ()
{
  mapper.init(PreCond(),PostCond(),MaxAllMapper());
}

CGGridLayoutModule::CGGridLayoutModule (const PreCond &P, 
					const PostCond &Q,
					const GridCoordinateMapper &M)
{
  mapper.init(P,Q,M);
}
