/*******************************************************************************
+
+  AGD 1.2
+
+
+  cgraphwin.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <AGD/CGraphWin.h>
#include <LEDA/internal/std/stdio.h>

using namespace leda;

static void cb_after_create_new_graph(GraphWin &gw);
static bool cb_before_create_new_graph(GraphWin &gw);
static void cb_new_node(GraphWin &gw, leda::node v);
static void cb_start_move_node(GraphWin &gw, leda::node v);
static void cb_end_move_node(GraphWin &gw, leda::node v);

/*
static void draw_background(leda::window* wp, double x, double y, double w, double h);
*/

cluster_info::cluster_info() {
  index = 0;
  text_fg_color = black;
  cluster_color = black;
  cluster_style = cgw_rectangle;
  cluster_fill_color = invisible;
  cluster_label_color = black;
  linestyle = dashed;
  borderwidth = 2;
}

//------------------------------------------------------------------------
// Constructors

CGraphWin::CGraphWin(const char* win_label) 
  : GraphWin(win_label) 
{
  cg_p = new ClusterGraph();
  rc_p = cg_p->get_rootcluster();

  GraphWin::set_graph(*cg_p);

  init_window();
  
  set_handler_functions();
  cl_index_counter = 0;
}

/////////////////////////////////////////////////////////////////////////////7
//
//  setups the information for clusters in the given graph

void CGraphWin::init_clustertree(Cluster* cl_parent) {
  list<Cluster*> cl_children = cl_parent->get_subclusters() ;
  Cluster* cl_node;

  cluster_info* cli;

  forall(cl_node, cl_children) {
    if(!cl_node->get_subclusters().empty()) init_clustertree(cl_node);
    else {
      cli = new cluster_info();
      cli->x = 0;
      cli->y = 0;
      cli->w = 0;
      cli->h = 0;
      //      cli->set_region(cl_region);
      cli->index = get_next_index();
      cli->label = string("C%d", cli->index);
      cli->cl = cl_node;
      insert_to_cli_cl_map(cli, cl_node);
    }
  }
  if(cl_parent != rc_p) {
    cli = new cluster_info();
    cli->x = 0;
    cli->y = 0;
    cli->w = 0;
    cli->h = 0;
    //    cli->set_region(cl_region);
    cli->index = get_next_index();
    cli->label = string("C%d", cli->index);
    cli->cl = cl_parent;
    insert_to_cli_cl_map(cli, cl_parent);
  }
}

///////////////////////////////////////////////////////////////////////////////
//
//

ClusterGraph& CGraphWin::get_clustergraph() { 
  return *cg_p;
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::clear_clustertree() {
  // creation of a new clustertree
  cg_p->clear_clustertree();
  rc_p = cg_p->get_rootcluster();

  // restarting cluster ids
  cl_index_counter = 0;
  missing_ids.clear();
  
  // deleting all clusters from graphwin window
  //del_all_clusters();
  clear_shapes();
  
  // restoring cluster-cluster_info map
  cluster_info* cli;
  forall(cli, cli_cl_map) delete cli;
  cli_cl_map.clear();
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::init_window() {
  _xmin = get_xmin();
  _xmax = get_xmax();
  _ymin = get_ymin();
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::set_gml_filename(const string &name) {
	GraphWin::set_gml_filename(name);
    set_frame_label("ClusterGraphWin - " + name);
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::set_handler_functions() {
  int submenu_id;

  // background function for cluster boundaries
  // set_bg_redraw(draw_background);

  // called AFTER generation of a new graph
  set_init_graph_handler(cb_after_create_new_graph);
  set_init_graph_handler(cb_before_create_new_graph);

  set_new_node_handler(cb_new_node);
  set_start_move_node_handler(cb_start_move_node);
  set_end_move_node_handler(cb_end_move_node);

  // create a submenu of the EDIT menu called Cluster
  submenu_id = gw_add_menu(*this, "Cluster");

  gw_add_simple_call(*this, create_cluster, "New Cluster", submenu_id);
  gw_add_simple_call(*this, delete_cluster, "Delete ", submenu_id);
  add_separator(submenu_id);
  gw_add_simple_call(*this, load_gml, "Load clustergraph (*.gml)", submenu_id);
  gw_add_simple_call(*this, save_gml, "Save clustergraph (*.gml)", submenu_id);
}

//////////////////////////////////////////////////////////////////////////////
//
//

int CGraphWin::get_next_index() {
  list_item id_item;
  int id;

  if(missing_ids.empty()) id = ++cl_index_counter;
  else {
    id_item = missing_ids.min();
    id = missing_ids.inf(id_item);
    missing_ids.erase(id_item);
  }

  return id;
}

///////////////////////////////////////////////////////////////////////////////
//
// public member: finds the cluster that contains the point <p> at the lowest
//                level

Cluster* CGraphWin::find_bounding_cluster(point p) {
  return(rec_fb_cluster(rc_p, p));
}

///////////////////////////////////////////////////////////////////////////////
//
// private member: runs through the cluster hierarchie to find the 
//                 bounding cluster

Cluster* CGraphWin::rec_fb_cluster(Cluster* cl_parent, point p) {
  list<Cluster*> cl_children = cl_parent->get_subclusters() ;
  Cluster* cl;
  cluster_info* cli;

  forall(cl, cl_children) {
    cli = cli_cl_map[cl];
    if(cli->get_region().contains(p)) return(rec_fb_cluster(cl, p));
  }
  return cl_parent;
}

///////////////////////////////////////////////////////////////////////////////
// handler functions
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// is called after a new node has been created

void CGraphWin::setup_node(node v) {
  point p = get_position(v);
  Cluster *cl;

  //char index_label[5];
  char index_label[16];
  sprintf(index_label, "v%d", get_index(v));
  set_user_label(v, index_label);

  cl = find_bounding_cluster(p);
  rc_p->remove_node(v);
  cl->add_node(v);
  cg_p->cl_node_map[v] = cl;
}


///////////////////////////////////////////////////////////////////////////////
//
//

static void cb_new_node(GraphWin &gw, node v)
{
  CGraphWin* cgw = (CGraphWin*) &gw;
  cgw->setup_node(v);
}

///////////////////////////////////////////////////////////////////////////////
//
//

static void cb_start_move_node(GraphWin &gw, node v)
{
  CGraphWin* cgw = (CGraphWin*) &gw;
  point p = cgw->get_position(v);
  Cluster* cl;

  cl = cgw->find_bounding_cluster(p);
  cl->remove_node(v);
}

///////////////////////////////////////////////////////////////////////////////
//
//

static void cb_end_move_node(GraphWin &gw, node v)
{
  CGraphWin* cgw = (CGraphWin*) &gw;
  point p = cgw->get_position(v);
  Cluster* cl;

  cl = cgw->find_bounding_cluster(p);
  cl->add_node(v);
}

///////////////////////////////////////////////////////////////////////////////
//
//

static bool cb_before_create_new_graph(GraphWin &gw) {
  CGraphWin* cgw = (CGraphWin*) &gw;

  cgw->clear_clustertree();

  return true;
}

///////////////////////////////////////////////////////////////////////////////
//
//

static void cb_after_create_new_graph(GraphWin &gw) {
  CGraphWin* cgw = (CGraphWin*) &gw;
  ClusterGraph& CG = cgw->get_clustergraph();

  node v;
  //char index_label[5];
  char index_label[16];
  
  if(CG.number_of_nodes() == 0) {
    cgw->clear_clustertree(); 
  } else {
    forall_nodes(v, CG) {
      sprintf(index_label, "v%d", cgw->get_index(v));
      cgw->set_user_label(v,index_label);
    }
  }
  // clears also the background...
  cgw->update_graph();
}

void CGraphWin::redraw()
{
  redraw_clusters();
  GraphWin::redraw();
}
