/*******************************************************************************
+
+  AGD 1.2
+
+
+  draw.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <AGD/CGraphWin.h>

using namespace leda;

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::create_cluster(GraphWin& gw) {
  CGraphWin* cgw = (CGraphWin*) &gw;

  node v, v2;
  double x1,x2,y1,y2;

  list<node> cluster_node_list, parent_node_list ;
  list<point> cl_bends;
  Cluster *cl_node;
  cluster_info *cli;

 
  cgw->define_area(x1,y1,x2,y2,"Please select the new cluster area");

  //------------------------------------------
  // Creates a cluster using ONE node with width/heigth 1 (better 0)
  // and ONE selfloop edge

  if(x1==x2 && y1==y2) {
    cgw->acknowledge("To create a cluster press and hold the \
left mouse button and drag the mouse to \
select the area for your new cluster!");
    return;
  }

  point p1,p2,p3,p4;

  p1 = point(x1,y1);
  p2 = point(x1,y2);
  p3 = point(x2,y2);
  p4 = point(x2,y1);

  cl_bends.append(p1);
  cl_bends.append(p2);
  cl_bends.append(p3);
  cl_bends.append(p4);

  gen_polygon cl_region(cl_bends, gen_polygon::SIMPLE, gen_polygon::DISREGARD_ORIENTATION);

  //--------------------------------
  // check if new cluster is valid and add it
  // cl_node is the pointer to the added cluster

  ClusterGraph &cg = cgw->get_clustergraph();
  Cluster* root_cluster = cg.get_rootcluster();

  list<Cluster*> cl_children = root_cluster->get_subclusters();
  list<Cluster*> supercluster;

  gen_polygon poly_inter, poly_diff, poly2_diff;

  Cluster *cl_supercluster, *cl_parent;

  list_item cl_item;

  bool outside = false;

  if(cl_children.empty()) {
    cl_node = cg.new_cluster(root_cluster);
  }
  else {
    cl_item = cl_children.first();
    
    while (cl_item != nil) {
      cl_node = cl_children.inf(cl_item);
      cli = cgw->cli_cl_map[cl_node];
      poly_inter = cl_region.intersection(cli->get_region());
      if(poly_inter.size()==0) {
	// Cluster is outside: New Cluster is outside of cl_node
	cl_item = cl_children.succ(cl_item);
	outside = true;
      } else {
	poly_diff = cl_region.diff(poly_inter);
	if(poly_diff.size()==0) {
	  // new cluster is complete inside actual cluster
	  cl_children = cl_node->get_subclusters();
	  cl_item = cl_children.first();
	  outside = false;  
	} else {
	  poly2_diff = cli->get_region().diff(poly_diff);
	  if(cl_region.diff(poly_diff).size()== 4 && poly2_diff.size() == 4) {
	    // cluster is a supercluster
	    cl_item = cl_children.succ(cl_item);
	    supercluster.append(cl_node);
	  } else {
	    cl_node = NULL;
	    break;
	  }
	}
      } 

      if(!supercluster.empty() && cl_item==nil) {
	// creates super cluster
	cl_supercluster = cg.new_cluster(cl_node->get_supercluster());

	// for all cluster inside the temorary super cluster
	forall(cl_node, supercluster) {
	  // add's cluster <cl_node> to the supercluster
	  cl_supercluster->add_subcluster(cl_node);
	  // get the parent of <cl_node>
	  cl_parent = cl_node->get_supercluster();
	  // removes <cl_node> from the child list of cluster <cl_parent>
	  cl_parent->remove_subcluster(cl_node);
	  // make <cl_supercluster> the new parent of cluster <cl_node>
	  cl_node->set_supercluster(cl_supercluster);
	}
        cl_node = cl_supercluster;
      } else if(cl_item==nil && outside==true) { 
	cl_node = cg.new_cluster(cl_node->get_supercluster());
      } else if (cl_item==nil && outside==false) {
	cl_node = cg.new_cluster(cl_node); 
      }
    } // end-while 
  }

//--------------------------------------------------------------------------------------

  if(cl_node!=NULL) {
    cli = new cluster_info();
    cli->x = x1<x2 ? x1 : x2;
    cli->y = y1<y2 ? y1 : y2;
    cli->w = fabs(x1 - x2);
    cli->h = fabs(y1 - y2);
    cli->set_region(cl_region);

    // assignment of an index 
    cli->index = cgw->get_next_index();
    cli->label = string("C%d", cli->index);

    cli->cl = cl_node;
    cgw->insert_to_cli_cl_map(cli, cl_node);
    
    // get nodes inside cluster
    cluster_node_list = cgw->get_nodes_in_area(x1,y1,x2,y2);

    Cluster* cl_parent = cl_node->get_supercluster();
    ClusterGraph& cg = cgw->get_clustergraph();

    parent_node_list = cl_parent->get_nodes();

    forall(v2, parent_node_list) {
      forall(v, cluster_node_list) {
	if(v2 == v && cl_region.contains(cgw->get_position(v))) {
	  cl_node->add_node(v);
	  cg.cl_node_map[v] = cl_node;

	  cl_node->get_supercluster()->remove_node(v);
	}
      }
    }
    cgw->redraw_clusters();
  } else cgw->acknowledge("For clusters its forbidden to intersect. Try again!");
  cgw->del_messages();
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::delete_cluster(GraphWin& gw) {
  CGraphWin* cgw = (CGraphWin*) &gw;

  cgw->acknowledge("Delete cluster function has to be re-written\n and is not \
available at the moment.");
}

/////////////////////////////////////////////////////////////////////////
// 
// redraws cluster hierachie

void CGraphWin::redraw_clusters() {
  //del_all_clusters();
  clear_shapes();
  rec_redraw_cluster(rc_p);
}

void CGraphWin::rec_redraw_cluster(Cluster* cl_parent) {
  list<Cluster*> cl_children = cl_parent->get_subclusters() ;
  Cluster* cl_node;
  cluster_info* cli;
  
  forall(cl_node, cl_children) {
    if(!cl_node->get_subclusters().empty()) rec_redraw_cluster(cl_node);
    else {
      cli = cli_cl_map[cl_node];
      if(cli != NULL) {
	if(cli->y + cli->h < cli->y) {
	  new_shape(cli->x, 
		      cli->y + cli->h,
		      cli->x + cli->w, 
		      cli->y, 
		      cli->label, 
		      cli->cluster_color,
		      cli->cluster_fill_color,
                      cli->cluster_label_color,
                      cli->borderwidth, 
                      cli->linestyle); 
	} else { 
	  new_shape(cli->x, 
		      cli->y, 
		      cli->x + cli->w, 
		      cli->y + cli->h, 
		      cli->label, 
		      cli->cluster_color,
		      cli->cluster_fill_color,
                      cli->cluster_label_color,
                      cli->borderwidth, 
                      cli->linestyle); 
	}
      }
    }
  }
  if(cl_parent != rc_p) {
    cli = cli_cl_map[cl_parent];
    if(cli != NULL) {
      if(cli->y + cli->h < cli->y) {
	new_shape(cli->x, 
		    cli->y + cli->h, 
		    cli->x + cli->w, 
		    cli->y, 
		    cli->label, 
		    cli->cluster_color,
		    cli->cluster_fill_color,
                    cli->cluster_label_color,
                    cli->borderwidth, 
                    cli->linestyle); 
      } else { 
	new_shape(cli->x, 
		    cli->y, 
		    cli->x + cli->w, 
		    cli->y + cli->h, 
		    cli->label, 
		    cli->cluster_color,
		    cli->cluster_fill_color,
                    cli->cluster_label_color,
                    cli->borderwidth, 
                    cli->linestyle); 
      }
    }
  }
}
