/*******************************************************************************
+
+  AGD 1.2
+
+
+  io.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <LEDA/graphics/file_panel.h>
#include <AGD/CGraphWin.h>

using namespace leda;

// missing define in LEDA/graphics/file_panel.h
enum { file_load, file_save, file_exit, file_all, file_cancel };

///////////////////////////////////////////////////////////////////////////////
//
// Opens a filepanel and loads a cluster graph from the selected file

static CGraphWin *cgw;

void CGraphWin::read_file(string filename)
{
	cgw->restore_dim();
	cgw->set_flush(false);
	cgw->read_gml(filename);
	cgw->read_cluster_gml(filename);
	cgw->redraw_clusters();

	cgw->fill_window();
	cgw->set_flush(true);
}

void CGraphWin::load_gml_handler()
{
	cgw = this;
	window& W=get_window();

	string fname = get_gml_filename();
	string dname = get_dirname();

	W.disable_panel();
	file_panel fp(W,fname,dname);
	fp.set_load_handler(read_file);
	fp.set_load_string("Load Cluster-Graph from GML File");
	fp.set_pattern("Cluster Graphs (*.gml)","*.gml");
	fp.open();

	W.enable_panel();
	set_dirname(dname);
	set_gml_filename(fname);
}

void CGraphWin::load_gml(GraphWin& gw)
{
	CGraphWin &cgw = (CGraphWin &) gw;
	cgw.load_gml_handler();
}


///////////////////////////////////////////////////////////////////////////////
//
// reads only the cluster hierachie from a *.gml file

void CGraphWin::read_cluster_gml(const string &filename) {

  file_istream fp(filename);
  string key, value, ref;

  node v;

  clear_clustertree();

  do {
    fp >> key;
    if(key == "rootcluster") {
      fp >> value;
      cl_index_counter = 0;
      break;
    } 
  } while(fp);

  while(fp) {
    fp >> key;
    if(key == "cluster") { 
      fp >> value;
      read_cluster(fp, rc_p);
    }
    if(key == "vertex") {
      fp >> value;
      forall_nodes(v, *cg_p) {
	ref = "\"" + get_user_label(v) + "\"";
	if(ref == value) {
	  rc_p->add_node(v);
	  cg_p->cl_node_map[v] = rc_p;
	}
      }
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
//
//

void CGraphWin::read_cluster(file_istream& fp, Cluster* cl_parent) {
  string key, value, ref;
  node v;
  list<point> cl_bends;
  string label;

  Cluster* cl = new Cluster;
  cluster_info *cli = new cluster_info;

  insert_to_cli_cl_map(cli, cl);
  cli->cl = cl;

  cl->set_supercluster(cl_parent);
  cl_parent->add_subcluster(cl);

  do {
    cl_bends.clear();
    fp >> key;
    if(key == "id") {
      fp >> value;
      cli->index = get_next_index();
    }
    

    if(key == "label") {
      fp >> value;
      cli->label = value(1,value.length()-2);
    }

    if(key == "graphics") {
      fp >> value;

      fp >> key >> cli->x;
      fp >> key >> cli->y;
      fp >> key >> cli->w;
      fp >> key >> cli->h;

      point lu(cli->x,cli->y);
      point ru(cli->x + cli->w, cli->y);
      point ro(cli->x + cli->w, cli->y + cli->h);
      point lo(cli->x, cli->y + cli->h);
      
      cl_bends.append(lu);
      cl_bends.append(ru);
      cl_bends.append(ro);
      cl_bends.append(lo);

      gen_polygon cl_region(cl_bends, gen_polygon::SIMPLE, gen_polygon::DISREGARD_ORIENTATION);

      cli->set_region(cl_region);

      fp >> key >> value;
      if(value == "\"red\"") cli->cluster_color = red;
      else if(value == "\"blue\"") cli->cluster_color = blue;
      else cli->cluster_color = black;

      fp >> key >> value;
      if(value == "\"rectangle\"") cli->cluster_style = cgw_rectangle;
      else if(value == "\"circle\"") cli->cluster_style = cgw_circle;

      fp >> key >> cli->borderwidth;
      fp >> value;
    }

    if(key == "vertex") {
      fp >> value;
      forall_nodes(v,*cg_p) {
	ref = "\"" + get_user_label(v) + "\"";
	if(ref == value) {
	  cl->add_node(v);
	  cg_p->cl_node_map[v] = cl;
	}
      }
    }

    if(key == "cluster") {
      fp >> value;
      read_cluster(fp,cl);
    }
  } while(key != "]");
}

///////////////////////////////////////////////////////////////////////////////
//
// Opens a filepanel and saves the actual graph as a cluster graph 

void CGraphWin::save_file(string filename)
{
    ofstream out(filename);

	if (!out) return;

    cgw->GraphWin::save_gml(out);

	out << "\nrootcluster [\n";
    cgw->print_gml(out);
    out << endl << "]\n";
    out.close();
}

void CGraphWin::save_gml_handler()
{
	cgw = this;
	window& W=get_window();

	string fname = get_gml_filename();
	string dname = get_dirname();

	W.disable_panel();
	file_panel fp(W,fname,dname);
	fp.set_save_handler(save_file);
	fp.set_save_string("Save Cluster-Graph to GML File");
	fp.set_pattern("Cluster Graphs (*.gml)","*.gml");
	fp.open();

	W.enable_panel();
	set_dirname(dname);
	set_gml_filename(fname);
}

void CGraphWin::save_gml(GraphWin& gw)
{
	CGraphWin &cgw = (CGraphWin &) gw;
	cgw.save_gml_handler();
}
