/*******************************************************************************
+
+  AGD 1.2
+
+
+  zoom.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <AGD/CGraphWin.h>

using namespace leda;

void CGraphWin::get_bounding_box(double &xmin, double &ymin, double &xmax, double &ymax) {
  list<Cluster*> cl_sub = rc_p->get_subclusters();
  Cluster* cl;
  cluster_info* cli;
  
  GraphWin::get_bounding_box(xmin,ymin,xmax,ymax);
  
  forall(cl, cl_sub) {
    cli = cli_cl_map[cl];
    if(cli->x < xmin) xmin = cli->x;
    if(cli->y < ymin) ymin = cli->y;
    if(cli->w + cli->x > xmax) xmax = cli->w + cli->x;
    if(cli->h + cli->y > ymax) ymax = cli->h + cli->y;
  }
}

void  CGraphWin::fill_window() {
  zoom_graph();
}

void CGraphWin::zoom_graph()
{
  graph&  G = get_graph();
  window& W = get_window();

  if (!W.is_open())
    error_handler(1,"GraphWin::zoom_graph: window has to be displayed.");

  if (G.number_of_nodes() == 0) return;

  double xmin;
  double xmax;
  double ymin;
  double ymax;
  
  get_bounding_box(xmin,ymin,xmax,ymax);

  double f = (xmax-xmin)/(W.xmax()-W.xmin());

  double dx = f*W.pix_to_real(48); 
  double dy = f*W.pix_to_real(48); 

  if (!get_zoom_objects())
  { double r1,r2;
    get_extreme_coords(xmin,ymin,xmax,ymax,r1,r2);
    f = (xmax-xmin)/(W.xmax()-W.xmin());
    dx = f*(W.pix_to_real(48) + r1);
    dy = f*(W.pix_to_real(48) + r2);
   }

  xmin -= dx;
  xmax += dx;
  ymin -= dy;
  ymax += dy;

  double x0 = xmin;
  double y0 = ymin;
  double x1 = xmax;
  double y1 = ymax;

  adjust_zoom_rect(x0,y0,x1,y1);

  dx = x1 - xmax; 
  dy = y1 - ymax;
  zoom_area(x0-dx/2,y0-dy/2, x1-dx/2, y1-dy/2);
}

void CGraphWin::adjust_zoom_rect(double& x0, double& y0, double& x1, double& y1) 
{ 
  window& W = get_window();

  point p0(x0,y0);
  

  double wdx = W.xmax()-W.xmin();
  double wdy = W.ymax()-W.ymin();

  if (x0 > x1) wdy = -wdy;
  if (y0 > y1) wdx = -wdx;

  line L(p0,p0.translate(wdx,wdy));

  if (orientation(L,point(x1,y1)) < 0)
    y1 = L.y_proj(x1);
  else
    x1 = L.x_proj(y1);
}

void CGraphWin::get_extreme_coords(double& xmin, double& ymin, 
				   double& xmax, double& ymax)
{
  graph&  G = get_graph();
  node    v = G.first_node();

  embed_edges();

  xmin = xmax = ymin = ymax = 0;

  if (v == nil) return;

  point p = get_position(v);

  xmin = p.xcoord();
  xmax = p.xcoord();
  ymin = p.ycoord();
  ymax = p.ycoord();

  forall_nodes(v,G) {
    point  p  = get_position(v);
    double x  = p.xcoord();
    double y  = p.ycoord();
    if (x < xmin) xmin = x;
    if (x > xmax) xmax = x;
    if (y < ymin) ymin = y;
    if (y > ymax) ymax = y;
  }


  edge e;
  forall_edges(e,G) {
    list<point> P = get_bends(e);
    
    list_item it;
    forall_items(it,P) {
      if (it == P.first() || it == P.last()) continue;
      point p = P[it];
      double x = p.xcoord();
      double y = p.ycoord();
      if (x < xmin) xmin = x;
      if (x > xmax) xmax = x;
      if (y < ymin) ymin = y; 
      if (y > ymax) ymax = y;
    }
  }

  list<Cluster*> cl_sub = rc_p->get_subclusters();
  Cluster* cl;
  cluster_info* cli;
  
  forall(cl, cl_sub) {
    cli = cli_cl_map[cl];
    if(cli->x < xmin) xmin = cli->x;  
    if(cli->x > xmax) xmax = cli->x;
    if(cli->y < ymin) ymin = cli->y;
    if(cli->y > ymax) ymax = cli->y;  
    if(cli->w + cli->x > xmax) xmax = cli->w + cli->x; 
    if(cli->w + cli->x < xmin) xmin = cli->w + cli->x; 
    if(cli->h + cli->y > ymax) ymax = cli->h + cli->y; 
    if(cli->h + cli->y < ymin) ymin = cli->h + cli->y; 
  }
}

void CGraphWin::get_extreme_coords(double& xmin, double& ymin, 
				   double& xmax, double& ymax, 
				   double& r1, double& r2)
{ embed_edges();
  double x0,y0,x1,y1;
  get_bounding_box(x0,y0,x1,y1);
  get_extreme_coords(xmin,ymin,xmax,ymax);
  r1 = leda::max(xmin-x0,x1-xmax);
  r2 = leda::max(ymin-y0,y1-ymax);
}
