/*******************************************************************************
+
+  AGD 1.2
+
+
+  inclusionTree.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


/*********************************************************************

  FILENAME  : inclusionTree.cc
  VERSION   : 1.0.0
  AUTHOR    : Christoph Liebel
  LANGUAGE  : C++

*********************************************************************/

#include <LEDA/system/basic.h>
#include <LEDA/internal/std/iostream.h>
#include "AGD/impl/cplanarity/inclusionTree.h"


/*****************************************************************************/
/*****************************               *********************************/
/***************************** inclusionTree *********************************/
/*****************************               *********************************/
/*****************************************************************************/

inclusionTree::inclusionTree()
{
    count = 0;
    actual = 0;
    root = 0;
}


/*****************************************************************************/
/*****************************                ********************************/
/***************************** ~inclusionTree ********************************/
/*****************************                ********************************/
/*****************************************************************************/

inclusionTree::~inclusionTree()
{
    clear(root);
}


/*****************************************************************************/
/*****************************       *****************************************/
/***************************** clear *****************************************/
/*****************************       *****************************************/
/*****************************************************************************/

void inclusionTree::clear(treeElement* helpPtr)
{
  if (helpPtr)
      {
          clear(helpPtr->child);
          clear(helpPtr->next);

          if (helpPtr == root)
              {   
                  delete helpPtr;
                  root = actual = 0;
              }
          else
              delete helpPtr;

          count--;
      }
}


/*****************************************************************************/
/*****************************           *************************************/
/***************************** getActual *************************************/
/*****************************           *************************************/
/*****************************************************************************/

treeElement* inclusionTree::getActual() const
{
    return actual;
}


/*****************************************************************************/
/*****************************          **************************************/
/***************************** getChild **************************************/
/*****************************          **************************************/
/*****************************************************************************/

treeElement* inclusionTree::getChild(treeElement* elementPtr) const
{
    return elementPtr->child;
}


/*****************************************************************************/
/*****************************          **************************************/
/***************************** getCount **************************************/
/*****************************          **************************************/
/*****************************************************************************/

int inclusionTree::getCount() const
{
    return count;
}


/*****************************************************************************/
/*****************************         ***************************************/
/***************************** getItem ***************************************/
/*****************************         ***************************************/
/*****************************************************************************/

int inclusionTree::getItem(treeElement* elementPtr) const
{
    return elementPtr->item;
}


/*****************************************************************************/
/*****************************         ***************************************/
/***************************** getNext ***************************************/
/*****************************         ***************************************/
/*****************************************************************************/

treeElement* inclusionTree::getNext(treeElement* elementPtr) const
{
    return elementPtr->next;
}


/*****************************************************************************/
/*****************************           *************************************/
/***************************** getParent *************************************/
/*****************************           *************************************/
/*****************************************************************************/

treeElement* inclusionTree::getParent(treeElement* elementPtr) const
{
    return elementPtr->parent;
}


/*****************************************************************************/
/*****************************         ***************************************/
/***************************** getRoot ***************************************/
/*****************************         ***************************************/
/*****************************************************************************/

treeElement* inclusionTree::getRoot() const
{
    return root;
}


/*****************************************************************************/
/*****************************        ****************************************/
/***************************** insert ****************************************/
/*****************************        ****************************************/
/*****************************************************************************/

void inclusionTree::insert(int newItem, int sibling)
{
    treeElement* newElement = new treeElement;
    newElement->item = newItem;
    newElement->next = 0;
    newElement->child = 0;

    if (!root)    // tree is empty
        {
            newElement->parent = 0;
            root = newElement;
        }
    else         // tree is not empty
        if (sibling)    // insertion as sibling of actual
            {
                newElement->parent = actual->parent;
                actual->next = newElement;
            }
        else           // insertion as child of actual
            {
                newElement->parent = actual;
                actual->child = newElement;
            }

    actual = newElement;
    count++;

}


/*****************************************************************************/
/*****************************              **********************************/
/***************************** postorderOut **********************************/
/*****************************              **********************************/
/*****************************************************************************/

void inclusionTree::postorderOut(treeElement* pointer) const
{
    if (pointer)
        {
            postorderOut(pointer->child);
            postorderOut(pointer->next);
            cout << "element = " << pointer->item << endl;
        }
}




