/*******************************************************************************
+
+  AGD 1.2
+
+
+  MixedModelLayout.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                     MixedModelLayout.cpp
//
//    author: Carsten Gutwenger
//    date:   september 1998
//-------------------------------------------------------------------


#include <AGD/MixedModelLayout.h>
#include "MixedModelBase.h"

#include <AGD/impl/misc.h>
#include <AGD/PlanAug.h>


//ofstream os;


void MMOrder::init(PlaneGraphCopy &PG, CanonicalOrderType ct, face f,
	bool prefer_nodes, double base_ratio)
{
	lmc = LmcOrder (PG, ct, f, prefer_nodes, base_ratio);
	left = right = array<node>(1,lmc.length());
}


/********************************************************************
							constructor

  set precondition and properties, set options to defaults
********************************************************************/

MixedModelLayout::MixedModelLayout () :
	the_prefer_nodes(false), the_base_ratio(0.33)
{
	add_precondition (key::planar);
	add_precondition (key::simple);
	add_precondition (key::no_self_loops);

	add_post_rule (key::no_crossings);

	PreCond  PRE;
	PostCond POST;
	PRE [1] << key::planar << key::simple << key::no_self_loops
		    << key::directed;
	POST[1] << key::planar << key::simple << key::no_self_loops
			<< key::directed << key::biconnected;
	augmenter.init (PRE,POST,PlanAug());
}


/********************************************************************
							clone

  dynamic copy function
********************************************************************/

AGDModule *MixedModelLayout::clone() const
{
	MixedModelLayout *M = new MixedModelLayout;

	M->prefer_nodes (prefer_nodes());
	M->base_ratio (base_ratio());

	M->set_augmenter (augmenter.get());
	M->set_mapper (mapper.get());

	return M;
}


/********************************************************************
						   do_call_grid

  Implements the algorithm call.
********************************************************************/

void MixedModelLayout::do_call (PlaneGraphCopyGridConcept &C)
{
	PlaneGraphCopy &PG = C.get_graph();
	GridLayout &gl = C.get_layout();

	// handle graphs with less than 3 nodes
	node v1, v2;
	switch (PG.number_of_nodes()) {
	case 0:
		return;
	case 1:
		v1 = PG.first_node();
		gl.x(v1) = gl.y(v1) = 0;
		return;
	case 2:
		v1 = PG.first_node();
		v2 = PG.succ_node (v1);
		gl.x(v1) = gl.y(v1) = gl.y(v2) = 0;
		gl.x(v2) = 1;
		return;
	}


	//os.open("output.txt");

	MixedModelBase mm(PG,C.get_exterior_face(),gl);

	mm.compute_order(augmenter.get(),prefer_nodes(),base_ratio());

	// begin debugging output
	//mm.printMMOrder(os);
	// end debugging output
	
	mm.assign_iop_coords();

	// begin debugging output
	//mm.printInOutPoints(os);
	// end debugging output

	mm.place_nodes();

	mm.postprocessing1();
	mm.set_bends();
	mm.postprocessing2();

	// begin debugging output
	//mm.printNodeCoords(os);
	// end debugging output


	//os.close();
}



/********************************************************************
					   required I/O operators
********************************************************************/

ostream& operator<< (ostream& os, const InOutPoint &)
{
	return os;
}

istream& operator>> (istream& is, InOutPoint&)
{
	return is;
}

