/*******************************************************************************
+
+  AGD 1.2
+
+
+  OrthogonalRepresentation.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 304 "Lweb/OrthoRep.lw"
#include <AGD/OrthogonalRepresentation.h>
#include <LEDA/core/stack.h>


using namespace leda;

const string OrthogonalRepresentation::epsilon;


OrthogonalRepresentation::OrthogonalRepresentation(const graph &G, face f) :
  PlaneGraphCopy(G) {

    if (f) f_0_ = copy_of_face(f);
    else f_0_ = max_face();

    a_.init(*this, 0); s_.init(*this, epsilon);
    zeroes_.init(*this, 0);
    dir_.init(*this, unexplored_); 
    L_.init(*this, nil);
    scaff_.init(*this, false);
    screw_.init(*this, false);
    number_of_bends_ = 0;
    output_level_ = 0;
}


//#line 329 "Lweb/OrthoRep.lw"
OrthogonalRepresentation::OrthogonalRepresentation(const PlaneGraphCopy &G, 
                                                   face f) :
  PlaneGraphCopy(G) {

    if (f) {
      edge epg = G.first_face_edge(f);
      node epgs = source(epg);
      node egs = G.original(epgs);
      node ehs = copy(egs);
      
      edge fpg = G.first_adj_edge(epgs);
      edge fh = first_adj_edge(ehs);

      while (fpg != epg) { 
        fpg = G.adj_succ(fpg);
        fh = adj_succ(fh);
      }

      f_0_ = face_of(fh);
   
    } else f_0_ = max_face();
    
    a_.init(*this, 0); s_.init(*this, epsilon);
    zeroes_.init(*this, 0);
    dir_.init(*this, unexplored_); 
    L_.init(*this, nil);
    scaff_.init(*this, false);
    screw_.init(*this, false);
    number_of_bends_ = 0;
    output_level_ = 0;
}



//#line 366 "Lweb/OrthoRep.lw"
OrthogonalRepresentation::OrthogonalRepresentation(const OrthogonalRepresentation &H) :
  PlaneGraphCopy(H) {

  edge e_0 = H.first_face_edge(H.f_0_);    // get edge on H's outer face...

  a_.init(*this, 0); s_.init(*this, epsilon);
  zeroes_.init(*this, 0);
  dir_.init(*this, unexplored_); 
  L_.init(*this, nil);
  scaff_.init(*this, false);
  screw_.init(*this, false);
  
  node v, vc;
  edge e, ec;
  
  vc = first_node();
  forall_nodes (v, H) {
    screw_[vc] = H.screw_[v];
    ec = _first_adjacent(vc);
    for (e = _first_adjacent(v); e != nil; e = _succ_adjacent(e,v)) {

      a_[ec] = H.a_[e];
      s_[ec] = H.s_[e];
      zeroes_[ec] = H.zeroes_[e];
      dir_[ec] = H.dir_[e];
      L_[ec] = H.L_[e];
      scaff_[ec] = H.scaff_[e];

      if (e == e_0) f_0_ = face_of(ec);
      
      ec = _succ_adjacent(ec,vc);
    }
    vc = succ_node(vc);
  }

  number_of_bends_ = H.number_of_bends_;
  output_level_ = H.output_level_;
}


//#line 411 "Lweb/OrthoRep.lw"
OrthogonalRepresentation::OrthogonalRepresentation(const OrthogonalRepresentation &H, bool ) : PlaneGraphCopy((const graph&) H) {

  edge e_0 = H.first_face_edge(H.f_0_);    // get edge on H's outer face...

  a_.init(*this, 0); s_.init(*this, epsilon);
  zeroes_.init(*this, 0);
  dir_.init(*this, unexplored_); 
  L_.init(*this, nil);
  scaff_.init(*this, false);
  screw_.init(*this, false);
  
  node v, vc;
  edge e, ec;
  
  vc = first_node();
  forall_nodes (v, H) {
    screw_[vc] = H.screw_[v];
    ec = _first_adjacent(vc);
    for (e = _first_adjacent(v); e != nil; e = _succ_adjacent(e,v)) {

      a_[ec] = H.a_[e];
      s_[ec] = H.s_[e];
      zeroes_[ec] = H.zeroes_[e];
      dir_[ec] = H.dir_[e];
      L_[ec] = H.L_[e];
      scaff_[ec] = H.scaff_[e];

      if (e == e_0) f_0_ = face_of(ec);
      
      ec = _succ_adjacent(ec,vc);
    }
    vc = succ_node(vc);
  }

  number_of_bends_ = H.number_of_bends_;
  output_level_ = H.output_level_;
}



//#line 456 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::set_string(leda::edge e, char c, int n) {
  if (c != '0' && c != '1') 
    error_handler(1, "OrthogonalRepresentation::set_string. No bits");
  for (int i = 0; i < n; i++) s_[e] += c;
  if (c == '0') { zeroes_[e] += n; number_of_bends_ += n; }
}

void OrthogonalRepresentation::add_convex_bend(leda::edge e) {
  s_[e] += '0'; 
  zeroes_[e]++; number_of_bends_++;
}

void OrthogonalRepresentation::add_concave_bend(leda::edge e) {
  s_[e] += '1';
}



//#line 481 "Lweb/OrthoRep.lw"
face OrthogonalRepresentation::expand(node v) {
  node l = original(v);
  face c = PlaneGraphCopy::expand(v); 
  edge e;
  forall_face_edges(e, c) {
    a_[e] = 0; s_[e] = epsilon; zeroes_[e] = 0; dir_[e] = unexplored_;
    scaff_[e] = screw_[source(e)] = false;
    L_[e] = l;
  }
  return c;
}

node OrthogonalRepresentation::collapse(face c) {
  node l = L_[first_face_edge(c)];
  if (!l) error_handler(1, 
    "OrthogonalRepresentation::collapse. Face has not been expanded.");
  return PlaneGraphCopy::collapse(c, l);
}

 

//#line 507 "Lweb/OrthoRep.lw"
edge OrthogonalRepresentation::split(edge e) {
  
  edge s = PlaneGraphCopy::split(e);
  edge re = reversal(e), rs = reversal(s);

  dir_[s] = dir_[e]; dir_[re] = dir_[rs];
  scaff_[s] = scaff_[e]; scaff_[re] = scaff_[rs];
  a_[s] = a_[e]; a_[re] = a_[rs];
  a_[e] = a_[rs] = 180;
  L_[s] = L_[e]; L_[re] = L_[rs];
  zeroes_[re] = zeroes_[rs]; zeroes_[rs] = 0;
  screw_[target(e)] = false;
  return s;
  
}


//#line 538 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::normalize() {

  edge e;
  forall_edges(e, *this) if (s_[e] != epsilon) {

      int ae = a_[e], ar = a_[reversal(e)];
      edge s = PlaneGraphCopy::split(e);

      edge r = reversal(e), rs = reversal(s);
      a_[s] = ae; a_[r] = ar;

      switch (s_[e][0]) {
      case '0': 
        a_[e] = 90;
        a_[rs] = 270;
        s_[s] = s_[e].del(0); 
        s_[rs] = s_[r].del(s_[r].length() - 1);
        s_[e] = s_[r] = epsilon; 
        zeroes_[s] = zeroes_[e] - 1;
        break;
      case '1':
        a_[e] = 270;
        a_[rs] = 90;
        s_[s] = s_[e].del(0);
        s_[rs] = s_[r].del(s_[e].length() - 1);
        s_[e] = s_[r] = epsilon;
        zeroes_[s] = zeroes_[e];
        --zeroes_[rs];
        break;
      }
      
      zeroes_[e] = zeroes_[r] = 0;
      --number_of_bends_;
  }
}


//#line 586 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::dissect() {

  enum state { nothing_read, read_10, read_1 };

  if (number_of_bends_ != 0) 
    error_handler(1, "OrthogonalRepresentation::dissect. Shape must be normalized");
  
  face f; forall_faces(f, *this) {
    
    edge e_act = first_face_edge(f); 
    state z = nothing_read; stack<edge> S;
    int n_steps = 0;

    edge e; forall_face_edges(e, f) if (a_[e] >= 270) {
      e_act = e;
      S.push(e); 
      if (a_[e] == 360) S.push(e); 
      z = read_1;
      break;
    }
    
    const int max_steps = 10 * adj_edges(f).size(); //XXX: muss unbedingt
    //gendert werden, absolut unschn
   
    while (n_steps++ < max_steps) {
      
      if (z == nothing_read && !S.empty()) { e_act = S.top(); z = read_1; } 

      e_act = face_cycle_succ(e_act);
      
      switch (a_[e_act]) {          
      case 90:  
        if (z == read_10) { 
//#line 638 "Lweb/OrthoRep.lw"
edge e_stack = S.pop(), e_rect_beg = face_cycle_succ(e_stack),
     e_rect_end = face_cycle_succ(e_act);

edge e_split = PlaneGraphCopy::split(e_rect_end);

scaff_[e_split] = scaff_[e_rect_end];
scaff_[reversal(e_rect_end)] = scaff_[reversal(e_split)];
dir_[e_split] = dir_[e_rect_end];
dir_[reversal(e_rect_end)] = dir_[reversal(e_split)];

edge e_scaff = split_face(e_rect_beg, e_split);

screw_[source(e_split)] = true; scaff_[e_scaff] = true;
dir_[reversal(e_scaff)] = next_(dir_[e_split]);
dir_[e_scaff] = opp_(dir_[reversal(e_scaff)]);

a_[e_stack] -= 90;
a_[e_split] = a_[e_rect_end]; a_[reversal(e_rect_end)] = a_[reversal(e_split)];
a_[e_rect_end] = 90; a_[reversal(e_split)] = 180;
a_[e_scaff] = 90; a_[reversal(e_scaff)] = 90; 

L_[e_split] = L_[e_rect_end];
L_[reversal(e_rect_end)] = L_[reversal(e_split)];
L_[e_scaff] = L_[reversal(e_scaff)] = NULL;

e_act = e_split; z = nothing_read; 

//#line 618 "Lweb/OrthoRep.lw"
                                                 ; }
        else if (z == read_1) z = read_10;
        break;
      case 180: break;
      case 270:
      case 360: 
        z = read_1; S.push(e_act); 
        if (a_[e_act] == 360) S.push(e_act); 
        break;
      default: error_handler(1, "angle not in {90, 180, 270, 360}");}
      
    }
  }
}


//#line 675 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::undissect() {
  
  list<edge> E_s;
  edge e; forall_edges(e, *this) if (scaff_[e]) E_s.append(e);
  forall(e, E_s) { del_edge(reversal(e)); del_edge(e); }

  node v; forall_nodes(v, *this) if (screw_[v]) unsplit(v); 

}


//#line 694 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::orientate() { 
  dir_.init(*this, unexplored_);
  orientate(first_edge(), N_); 
}

void OrthogonalRepresentation::orientate(edge e, direction d) {

  if (dir_[e] != unexplored_) return;
  while (dir_[e] == unexplored_) {
    dir_[e] = d; edge r = reversal(e);
    if (dir_[r] == unexplored_) orientate(r, opp_(d));
    switch (a_[e]) { 
      // case 0: nothing to do (d stays the same)
    case 0:   d = opp_(d); break; 
    case  90: d = next_(d); break;
    case 270: d = prev_(d); break;
    case 360: d = opp_(d); break;
    }
    e = face_cycle_succ(e);
  }

}


//#line 724 "Lweb/OrthoRep.lw"
bool OrthogonalRepresentation::check(string& msg) const {

  if (!Is_Planar_Map(*this)) { 
    msg = "OrthogonalRepresentation::check. The graph is no planar map";
    return false;
  }


//#line 735 "Lweb/OrthoRep.lw"
  edge e; forall_edges(e, *this) {
    string s = s_[e], r = s_[reversal(e)]; 
    int l = s.length() - 1;
    for (int i = 0; i <= l; i++) if (s[i] != flip(r[l-i])) {
      print_edge(e);
      msg = "OrthogonalRepresentation::check. The bit strings are inconsistent";
      return false;
    }
  }                     

//#line 749 "Lweb/OrthoRep.lw"
  
  face f; forall_faces(f, *this) {
    int rho = 0;
    forall_face_edges(e, f) rho += (zeroes_[e] - ones(e) + 2 - a_[e]/90);
        
    if (rho != ((f == f_0_) ? -4 : 4)) {
      msg = "OrthogonalRepresentation::check. Face shapes are not rectagonal";
      return false;
    }
  }


//#line 764 "Lweb/OrthoRep.lw"
  node v; forall_nodes(v, *this) {    
    int angle_sum = 0;
    forall_in_edges(e, v) angle_sum += a_[e];
    if (angle_sum != 360) {
      msg = "OrthogonalRepresentation::check. Angle sums corrupted";
      return false;
    }
  }


//#line 777 "Lweb/OrthoRep.lw"
/* modification done by Thomas Ziegler, date: 09-Jan-2001
    problem: direction test causes warning for graph with two nodes
  and no edges. This problem should be fixed in new version of AGD.*/
  /*
  msg = "OrthogonalRepresentation::check. Directions not consistent";

  forall_faces(f, *this) {
    edge e; forall_face_edges(e, f) {
      if (dir_[e] != opp_(dir_[reversal(e)])) return false;
   
      direction d = dir_[face_cycle_succ(e)];

      switch (a_[e]) {
      case 0:   if (d != opp_(dir_[e])) return false; break;
      case 90:  if (d != next_(dir_[e])) return false; break;
      case 180: if (d != dir_[e]) return false; break;
      case 270: if (d != prev_(dir_[e])) return false; break;
      case 360: if (d != opp_(dir_[e])) return false; break;
      default:
        error_handler(1, 
          string("OrthogonalRepresenation::check. Angle is %d", a_[e])); 
        return false;
      }
    }
  }*/
    
  msg = ""; return true;

}


//#line 810 "Lweb/OrthoRep.lw"
void OrthogonalRepresentation::print_edge(edge e, ostream& O) const {
  graph::print_edge(e, O); 
  O << string("\ta = %d\ts = ", a_[e]) << s_[e] 
  << string("\tdir = %d\tscaff =  %d |0| = %d, cage = ", 
            (int)dir_[e], scaff_[e], zeroes_[e]);
  if (L_[e]) print_node(L_[e]);
  cout << endl;
}

ostream& operator<< (ostream& O, const OrthogonalRepresentation& H) {
  O << string("OrthogonalRepresentation with %d nodes, %d edges and %d bends", 
    H.number_of_nodes(), H.number_of_edges(), 
    H.number_of_bends_) << endl;
    face f; forall_faces(f, H) {
    O << ((f == H.f_0_) ? "external " : "internal ") << "face" << endl;
    edge e; forall_face_edges(e, f) {
      O << "\t";
      H.print_edge(e); 
    }
  }
  return O;
}

