/*******************************************************************************
+
+  AGD 1.2
+
+
+  QuasiOrthogonalLayout.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 539 "Lweb/OrthoLayout.lw"
#include <AGD/QuasiOrthogonalLayout.h>

using namespace leda;

QuasiOrthogonalLayout::QuasiOrthogonalLayout() : Ovid() {
  
  _blowUp = true;
  _containsCages = true;
  _allowBends = true;

}

AGDModule* QuasiOrthogonalLayout::clone() const { 

  QuasiOrthogonalLayout *O = new QuasiOrthogonalLayout;
    
  O->variant(variant());
  O->output_level(output_level());
  O->traditional(traditional());
  O->progressive(progressive());
  O->verify(verify());

  O->set_compactor(compactor.get());

  return O; 
 
}


//#line 570 "Lweb/OrthoLayout.lw"
void QuasiOrthogonalLayout::preprocessing(OrthogonalRepresentation &H) {

  if (!Is_Connected(H)) { 
//#line 582 "Lweb/OrthoLayout.lw"
if (_outputLevel > 3) INF("making the input graph temporarily connected...");
edge e_ref = nil;
if (H.number_of_edges() > 0) e_ref = H.first_face_edge(H.get_external_face());
Make_Connected(H);
list<edge> R; H.make_map(R); H.compute_faces();
if (e_ref) H.set_external_face(H.face_of(e_ref));
else H.set_external_face(H.first_face());

//#line 572 "Lweb/OrthoLayout.lw"
                                                             } 
  
//#line 596 "Lweb/OrthoLayout.lw"
  
INF("transforming highdeg vertices into cages... ");

node v; forall_nodes(v, H) if (outdeg(v) > 4) H.expand(v);
  
DONE();

//#line 573 "Lweb/OrthoLayout.lw"
                                               ;

}


//#line 609 "Lweb/OrthoLayout.lw"
void QuasiOrthogonalLayout::postprocessing(OrthogonalRepresentation &H,
                                           GridLayout &L) {

  INF("putting highdeg vertices in their cages... "); 

  face f; list<face> all_faces = H.all_faces(); // workaround
  forall(f, all_faces) if (H.is_cage(f)) {
  
    int xmin = MAXINT, ymin = MAXINT, xmax = -MAXINT, ymax = -MAXINT;
  
    edge e; forall_face_edges(e, f) { 
      node c = source(e); 
      if (L.x()[c] < xmin) xmin = L.x()[c]; 
      if (L.x()[c] > xmax) xmax = L.x()[c]; 
      if (L.y()[c] < ymin) ymin = L.y()[c]; 
      if (L.y()[c] > ymax) ymax = L.y()[c]; 
    }
  
    node v = H.collapse(f);
   
    L.x()[v] = (xmin + xmax)/2; 
    L.y()[v] = (ymin + ymax)/2;
    
  }

  DONE();

}

