/*******************************************************************************
+
+  AGD 1.2
+
+
+  NaiveMaximalPlanarSubgraph.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                        NaiveMaximalPlanarSubgraph.cpp
//
//
// author: Thomas Ziegler
//        
// date:   july 2000
//-------------------------------------------------------------------


#include <AGD/NaiveMaximalPlanarSubgraph.h>
#include <AGD/impl/misc.h>
#include <LEDA/graph/graph_misc.h>

using namespace leda;

NaiveMaximalPlanarSubgraph::NaiveMaximalPlanarSubgraph() : _runs(4)
{
	add_post_rule (key::planar);
	add_post_rule (key::simple, 1, key::simple);
	add_post_rule (key::no_self_loops, 1, key::no_self_loops);
	add_post_rule (key::no_reversals, 1, key::no_reversals);
}



void NaiveMaximalPlanarSubgraph::do_call 
(const graph& G, list<edge>& UnplanarEdges)
{
  if (G.number_of_edges() == 0||Is_Planar(G)) return;
  
  UnplanarEdges.clear();

  list<edge> all_edges=G.all_edges();

  int r;
  for (r=1;r<=runs();r++) {  
    list<edge> current_unplanar;

    permute(all_edges);

    graph G_help;
    node_array<node> corr_help_node(G);
    
    node v;
    forall_nodes(v,G) {
      corr_help_node[v]=G_help.new_node();
    }
    
    edge e;
    forall(e,all_edges) {
      node s_help=corr_help_node[G.source(e)];
      node t_help=corr_help_node[G.target(e)];    
      edge e_help=G_help.new_edge(s_help,t_help);
      
      if (!Is_Planar(G_help)) {
	G_help.del_edge(e_help);
	current_unplanar.append(e);
      }
    }
    
    if (UnplanarEdges.empty()||
	UnplanarEdges.size()>current_unplanar.size()) {
      UnplanarEdges.clear();
      while (!current_unplanar.empty()) 
	UnplanarEdges.append(current_unplanar.pop()); 
    }
  }
}

AGDModule *NaiveMaximalPlanarSubgraph::clone() const 
{
  NaiveMaximalPlanarSubgraph *S = new NaiveMaximalPlanarSubgraph;
  S->runs(runs());
  return S;
}
