/*******************************************************************************
+
+  AGD 1.2
+
+
+  planGraph.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



/************************************************************************

         Filename   :   planGraph.cc
         Version    :   1.0.2
         Author     :   Christoph Buchheim, Sebastian Leipert
         Date       :   19.01.2000
         Language   :   C++

************************************************************************/

#if defined(__BORLANDC__)
// turn off some warnings
#pragma option -w-inl -w-ccc -w-aus -w-eff -w-par -w-rch -w-pia
#endif



#ifdef NOAGD

#include "planGraph.h"
#include "PQdirected.h"

#else

#include "AGD/impl/planarity/planGraph.h"
#include "AGD/impl/planarity/PQdirected.h"

#endif




planGraph::planGraph():embedded(1) {
  embeddedAdjacencyList = 0;
  D = 0;
  stNode = 0;
  marked = 0;
  PQ = 0;
}

planGraph::~planGraph() {};

void planGraph::addSTEdge(int lower,outEdge higherEdge) {
  int higher=higherEdge.getNode();
  int newEdge=higherEdge.getNumber();
  D[higher].lowerNumbered[D[higher].lowerCount++]=
    D[lower].higherNumbered[D[lower].higherCount++]=
      new leafKey<outEdge,int,int>(outEdge(lower,newEdge,higherEdge.label()));
  marked[newEdge]=1;
}

pqStack<int>* planGraph::findPath(int toStartAt) {
  outEdge toFind,toPass;
  pqStack<int>* path=new pqStack<int>;
  while(!D[toStartAt].aEdges.stackEmpty()) {
    toFind=D[toStartAt].aEdges.pop();
    if(!marked[toFind.getNumber()]) {
      path->push(toStartAt);
      addSTEdge(toStartAt,toFind);
      return path;
    }
  }
  while(!D[toStartAt].bEdges.stackEmpty()) {
    toFind=D[toStartAt].bEdges.pop();
    if(!marked[toFind.getNumber()]) {
      path->push(toStartAt);
      addSTEdge(toStartAt,toFind);
      while(!D[toFind.getNode()].marked) {
        toPass=D[toFind.getNode()].toLowpoint;
        D[toFind.getNode()].marked=1;
        addSTEdge(toFind.getNode(),toPass);
        path->push(toFind.getNode());
        toFind=toPass;
      }
      return path;
    }
  }
  while(!D[toStartAt].cEdges.stackEmpty()) {
    toFind=D[toStartAt].cEdges.pop();
    if(!marked[toFind.getNumber()]) {
      path->push(toStartAt);
      addSTEdge(toStartAt,toFind);
      while(!D[toFind.getNode()].marked) {
        toPass=D[toFind.getNode()].father;
        D[toFind.getNode()].marked=1;
        addSTEdge(toFind.getNode(),toPass);
        path->push(toFind.getNode());
        toFind=toPass;
      }
      return path;
    }
  }
  stNode[stCount++]=toStartAt;
  return path;
}

void planGraph::findSTEdges(int source) {
  stCount=0;
  outEdge sinkEdge=D[source].bEdges.pop();
  outEdge sourceEdge(source,sinkEdge.getNumber(),sinkEdge.label());
  int sink=sinkEdge.getNode();
  pqStack<int> *P,*S=new pqStack<int>;
  D[sink].marked=D[source].marked=1;
  addSTEdge(sink,sourceEdge);
  S->push(source);
  S->push(sink);
  while(S->count()>1) {
    P=findPath(S->pop());
    while(!P->stackEmpty()) S->push(P->pop());
    delete P;
  }
  stNode[stCount++]=S->pop();
  delete S;
}

int planGraph::upwardEmbed(int runThrough) {
  PQdirected<outEdge,int,int>* PQ_Tree = new PQdirected<outEdge,int,int>;
  nodeData<outEdge,int,int>* d=&D[stNode[0]];
  PQ_Tree->Initialize(d->higherCount,d->higherNumbered);
  d->higherCount=0;
  int st,keyNumber,ok=1;
  for(st=1;st<stCount;st++) 
      if ((!runThrough && stNode[st]) || runThrough) {
          d=&D[stNode[st]];
          if(ok) ok=PQ_Tree->Reduction(d->lowerCount,d->lowerNumbered);
          if(ok) {
              PQ_Tree->addition(stNode[st],d->higherCount,d->higherNumbered,
                                d->upwardAdjacencyList,
                                d->toBeReversed,d->notToBeReversed);
              PQ_Tree->emptyAllPertinentNodes();
          }
          for(keyNumber=0;keyNumber<d->lowerCount;keyNumber++)
              delete d->lowerNumbered[keyNumber];
          d->higherCount=d->lowerCount=0;
      }
  if(!runThrough && !stNode[st-1]) {
      PQ = PQ_Tree;
      return ok;
  }
  if(!ok) return 0;
  int reverse;
  pqStack<int>* help;
  for(st=stCount-1;st;st--) {
    d=&D[stNode[st]];
    while(!d->toBeReversed->stackEmpty()) {
      reverse=d->toBeReversed->pop();
      D[reverse].upwardAdjacencyList->reverse();
      help=D[reverse].toBeReversed;
      D[reverse].toBeReversed=D[reverse].notToBeReversed;
      D[reverse].notToBeReversed=help;
    }
    d->notToBeReversed->cleanupStack();
  }
  return 1;
}

void planGraph::entireEmbed(int newNode) {
  embeddedAdjacencyList[newNode].
    append(D[newNode].upwardAdjacencyList);
  D[newNode].marked=0;
  D[newNode].upwardAdjacencyList->reset();
  outEdge toBeExtended;
  while(D[newNode].upwardAdjacencyList->out(toBeExtended)) {
    if(D[toBeExtended.getNode()].marked)
      entireEmbed(toBeExtended.getNode());
    outEdge toBeAdded(newNode,toBeExtended.getNumber(),toBeExtended.label());
    embeddedAdjacencyList[toBeExtended.getNode()].in(toBeAdded);
  }
  D[newNode].upwardAdjacencyList->clear();
}

int planGraph::search(int root,int& count) {
  int nextNode;
  outEdge nextEdge;
  D[root].lowpoint=D[root].dfsNumber=++count;
  adjacencyList[root].reset();
  while(adjacencyList[root].out(nextEdge)) {
    nextNode=nextEdge.getNode();
    if(!D[nextNode].dfsNumber) {
      D[nextNode].father=outEdge(root,nextEdge.getNumber(),nextEdge.label());
      if(!search(nextNode,count)) return 0;
      D[root].bEdges.push(nextEdge);
      if(D[nextNode].lowpoint<=D[root].lowpoint) {
        D[root].lowpoint=D[nextNode].lowpoint;
        D[root].toLowpoint=nextEdge;
      }
      if(D[nextNode].lowpoint>=D[root].dfsNumber) {
        findSTEdges(root);
        if(!upwardEmbed()) return 0;
        entireEmbed(root);
      }
    }
    else if(nextNode==root) embeddedAdjacencyList[root].in(nextEdge);
    else if(D[nextNode].dfsNumber<D[root].dfsNumber) {
      if(D[nextNode].dfsNumber<=D[root].lowpoint) {
        D[root].lowpoint=D[nextNode].dfsNumber;
        D[root].toLowpoint=nextEdge;
      }
      D[root].aEdges.push(nextEdge);
      outEdge reversed(root,nextEdge.getNumber(),nextEdge.label());
      D[nextNode].cEdges.push(reversed);
    }
  }
  return 1;
}

int planGraph::search(int root,int& count,int _dontStop) {   // Needed for c-planarity test in AGD
  int nextNode;
  outEdge nextEdge;
  D[root].lowpoint=D[root].dfsNumber=++count;
  adjacencyList[root].reset();
  while(adjacencyList[root].out(nextEdge)) {
    nextNode=nextEdge.getNode();
    if(!D[nextNode].dfsNumber) {
      D[nextNode].father=outEdge(root,nextEdge.getNumber(),nextEdge.label());
      if(!search(nextNode,count,_dontStop)) return 0;
      D[root].bEdges.push(nextEdge);
      if(D[nextNode].lowpoint<=D[root].lowpoint) {
        D[root].lowpoint=D[nextNode].lowpoint;
        D[root].toLowpoint=nextEdge;
      }
      if(D[nextNode].lowpoint>=D[root].dfsNumber) {
        findSTEdges(root);
        if(!upwardEmbed(_dontStop)) return 0;
        entireEmbed(root);
      }
    }
    else if(nextNode==root) embeddedAdjacencyList[root].in(nextEdge);
    else if(D[nextNode].dfsNumber<D[root].dfsNumber) {
      if(D[nextNode].dfsNumber<=D[root].lowpoint) {
        D[root].lowpoint=D[nextNode].dfsNumber;
        D[root].toLowpoint=nextEdge;
      }
      D[root].aEdges.push(nextEdge);
      outEdge reversed(root,nextEdge.getNumber(),nextEdge.label());
      D[nextNode].cEdges.push(reversed);
    }
  }
  return 1;
}



int planGraph::embedGraph(int reallyEmbed,int* st) {
  if(n>2&&mSingle>3*n-6) return 0;
  D=new nodeData<outEdge,int,int>[n];
  stNode=new int[n];
  marked=new int[m];
  int nodeNumber,edgeNumber,count;
  outEdge nextEdge;
  for(nodeNumber=0;nodeNumber<n;nodeNumber++)
  if((count=adjacencyList[nodeNumber].count())) {
    D[nodeNumber].higherNumbered=new leafKey<outEdge,int,int>*[count];
    D[nodeNumber].lowerNumbered=new leafKey<outEdge,int,int>*[count];
  }
  for(edgeNumber=0;edgeNumber<m;edgeNumber++)
    marked[edgeNumber]=0;
  embeddedAdjacencyList=new basicList<outEdge>[n];
  int planar=1;
  if(st) {
    for(nodeNumber=0;nodeNumber<n;nodeNumber++) {
      stNode[st[nodeNumber]]=nodeNumber;
      adjacencyList[nodeNumber].reset();
      while(adjacencyList[nodeNumber].out(nextEdge)) 
        if(st[nextEdge.getNode()]>st[nodeNumber]) 
            addSTEdge(nodeNumber,nextEdge);
      D[nodeNumber].marked=1;
    }
    stCount=n;
    planar=upwardEmbed();
    if(planar) entireEmbed(stNode[n-1]);
  }
  else {
    for(nodeNumber=0;nodeNumber<n&&planar;nodeNumber++)
    if(!D[nodeNumber].dfsNumber) {
      count=0;
      planar=search(nodeNumber,count);
    }
  }
  if(planar&&reallyEmbed) {
    delete[] adjacencyList;
    adjacencyList=embeddedAdjacencyList;
  }
  else delete[] embeddedAdjacencyList;
  delete[] D;
  delete[] stNode;
  delete[] marked;
  return planar;
}


  // Needed for c-planarity testing in AGD:
int planGraph::embedGraph(PQdirected<outEdge,int,int>* & P,int reallyEmbed, 
                          int dontStop,int* st) {
  if(embedded) return 1;
  if(mSingle>3*n-6) return 0;
  D=new nodeData<outEdge,int,int>[n];
  stNode=new int[n];
  marked=new int[m];
  int nodeNumber,edgeNumber,count;
  outEdge nextEdge;
  for(nodeNumber=0;nodeNumber<n;nodeNumber++)
  if((count=adjacencyList[nodeNumber].count())) {
    D[nodeNumber].higherNumbered=new leafKey<outEdge,int,int>*[count];
    D[nodeNumber].lowerNumbered=new leafKey<outEdge,int,int>*[count];
  }
  for(edgeNumber=0;edgeNumber<m;edgeNumber++)
    marked[edgeNumber]=0;
  embeddedAdjacencyList=new basicList<outEdge>[n];
  int planar=1;
  if(st) {
    for(nodeNumber=0;nodeNumber<n;nodeNumber++) {
      stNode[st[nodeNumber]]=nodeNumber;
      adjacencyList[nodeNumber].reset();
      while(adjacencyList[nodeNumber].out(nextEdge)) 
        if(st[nextEdge.getNode()]>st[nodeNumber]) 
          addSTEdge(nodeNumber,nextEdge);
    }
    stCount=n;
    planar=upwardEmbed(dontStop);
    entireEmbed(0);
  }
  else {
    for(nodeNumber=0;nodeNumber<n&&planar;nodeNumber++)
    if(!D[nodeNumber].dfsNumber) {
      count=0;
      planar=search(nodeNumber,count,dontStop);
    }
  }
  P = PQ;
  if(planar&&reallyEmbed) {
    delete[] adjacencyList;
    adjacencyList=embeddedAdjacencyList;
  }
  else {
          //delete[] embeddedAdjacencyList;
          //embeddedAdjacencyList = 0;
  }
  return planar;
}

int planGraph::isPlanar() {
  return embedGraph(0,(int*)0);
}

  // Needed for c-planarity testing in AGD: 
int planGraph::isPlanarC() {
  PQdirected<outEdge,int,int>* PQT = 0;
  int ergebnis = embedGraph(PQT,0,1,(int*)0);
  if (PQT) {
      delete PQT;
      PQ = PQT = 0;
  }
  return ergebnis;
}


void planGraph::deleteGraph() {
  basicGraph::deleteGraph();
  embedded=1;
}

int planGraph::readGraph(char* filename) {
  if(!basicGraph::readGraph(filename)) return 0;
  embedded=m<3;
  return 1;
}





