/*******************************************************************************
+
+  AGD 1.2
+
+
+  GreedyCycleRemoval.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      GreedyCycleRemoval.cpp
//
//    author: Carsten Gutwenger
//    date:   decembre 1998
//-------------------------------------------------------------------


#include <AGD/GreedyCycleRemoval.h>


using namespace leda;

//-------------------------------------------------------------------
//                          GreedyCycleRemoval
//-------------------------------------------------------------------

GreedyCycleRemoval::GreedyCycleRemoval()
{
	add_precondition (key::directed);
	add_post_rule (key::directed);
	add_post_rule (key::maximal_acyclic);
}


AGDModule *GreedyCycleRemoval::clone() const
{
	return new GreedyCycleRemoval;
}


void GreedyCycleRemoval::dfs (node v, const graph &G)
{
	visited[v] = true;

	int i;
	if (outdeg(v) == 0) i = min;
	else if (indeg(v) == 0) i = max;
	else i = outdeg(v) - indeg(v);

	item[v] = B[index[v] = i].push_back(v);
	in [v] = indeg (v);
	out[v] = outdeg(v);
	counter++;

	edge e;
	forall_inout_edges(e,v) {
		node u = G.opposite(v,e);
		if (!visited[u])
			dfs(u,G);
	}
}


void GreedyCycleRemoval::do_call (const graph& G, list<edge>& feedback)
{
	node u, v, w;
	edge e;

	max = min = 0;
	forall_nodes(v,G) {
		if (-indeg (v) < min) min = -indeg(v);
		if ( outdeg(v) > max) max = outdeg(v);
	}

	if (G.number_of_edges() <= 1) return;

	visited.init(G,false); item.init(G);
	in     .init(G);       out .init(G);
	index  .init(G);
	B = array<list<node> >(min,max);
	
	list<node> S_l, S_r;
	node_array<int> pos(G);

	counter = 0;
	forall_nodes (v, G) {
		if (!visited[v]) {
			dfs(v,G);

			int i, max_i = max-1, min_i = min+1;

			for ( ; counter > 0; counter--) {
				if (!B[min].empty()) {
					u = B[min].pop();
					S_r.push_front(u);

				} else if (!B[max].empty()) {
					u = B[max].pop();
					S_l.push_back(u);

				} else {
					while (B[max_i].empty())
						max_i--;
					while (B[min_i].empty())
						min_i++;

					if (abs(max_i) > abs(min_i)) {
						u = B[max_i].pop();
						S_l.push_back(u);
					} else {
						u = B[min_i].pop();
						S_r.push_front(u);
					}
				}

				item[u] = nil;

				forall_in_edges(e,u) {
					w = source(e);
					if (item[w]) {
						out[w]--; i = index[w];
						B[i].del_item(item[w]);
						if (out[w] == 0) i = min;
						else if (in[w] == 0) i = max;
						else i--;
						item[w] = B[index[w] = i].push_back(w);

						if (index[w] < min_i)
							min_i = index[w];
					}
				}
				forall_out_edges(e,u) {
					w = target(e);
					if (item[w]) {
						in[w]--; i = index[w];
						B[i].del_item(item[w]);
						if (out[w] == 0) i = min;
						else if (in[w] == 0) i = max;
						else i++;
						item[w] = B[index[w] = i].push_back(w);

						if (index[w] > max_i)
							max_i = index[w];
					}
				}
			}

			i = 0;
			while (!S_l.empty())
				pos[S_l.pop_front()] = i++;
			while (!S_r.empty())
				pos[S_r.pop_front()] = i++;
		}
	}

	forall_edges (e, G)
		if (pos[source(e)] >= pos[target(e)])
			feedback.push_back(e);

	visited.init(); item.init();
	in     .init(); out .init();
	index  .init();
	B = array<list<node> >();
}

