/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  _gxl_x11.c
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


// ---------------------------------------------------------------------------
// LEDA OpenGL Support
// ---------------------------------------------------------------------------

#include "../x11/_x_basic.h"

#include <GL/glx.h>

LEDA_BEGIN_NAMESPACE

// -----------------------
// from ../x11/_x_basic.c
// -----------------------

extern x11_win* wlist[MAXWIN];
extern Display* display;
extern int      screen;


void x_opengl_init(int win)
{ x11_win* wp = wlist[win];

  int bits = 8;

  int attrib[] = { GLX_RGBA,
                   GLX_RED_SIZE,   bits,
		               GLX_GREEN_SIZE, bits,
		               GLX_BLUE_SIZE,  bits,
		               GLX_DEPTH_SIZE, bits,
		               GLX_DOUBLEBUFFER,
		               None };
  
  XVisualInfo* vis_info = 0;
  while (bits > 1)
  {
    if ((vis_info = glXChooseVisual(display,screen,attrib)) != NULL) break;    
    fprintf(stderr,"\nCannot allocate a RGB double buffered visual with depth size %d!",bits--);   
    
    attrib[2] = bits;
    attrib[4] = bits;
    attrib[6] = bits;
  }

  if (!vis_info) 
  { fprintf(stderr,"\nOpenGL color allocation failed!");
    return;
  }

  GLXContext glc = glXCreateContext(display,vis_info,NULL,True);   
  if (!glc)
  { fprintf(stderr,"\nCannot create a rendering context!");
    return; 
  }
  
  if (!glXMakeCurrent(display,wp->win,glc))
  { fprintf(stderr,"\nCannot enable the rendering context!");
    return;   
  }  
  
  wp->glc = glc;
}

void x_opengl_start(int win)
{ x11_win* wp = wlist[win];

  wp->win_ret = glXGetCurrentDrawable();
  wp->glc_ret = glXGetCurrentContext();
  
  if (wp->glc != wp->glc_ret) 
  { GLXContext glc = (GLXContext) wp->glc;     
    glXMakeCurrent(display,wp->win,glc);  
  }
}

void x_opengl_flush(int win)
{ x11_win* wp = wlist[win];  
  glXSwapBuffers(display,wp->win);
}

void x_opengl_finish(int win)
{ x11_win* wp = wlist[win];

  if (wp->glc != wp->glc_ret) 
  { GLXContext glc = (GLXContext) wp->glc_ret;     
    glXMakeCurrent(display,wp->win_ret,glc);  
  }
}

void* x_opengl_win_struct(int win) { return wlist[win]; }

LEDA_END_NAMESPACE
