/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  _wgl_win32.c
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


// ---------------------------------------------------------------------------
// LEDA OpenGL Support
// ---------------------------------------------------------------------------

#include "../mswin/_x_basic.h"

LEDA_BEGIN_NAMESPACE

// -----------------------
// from ../mswin/_x_basic.c
// -----------------------

extern win_struct* wlist[MAX_WIN];

void x_opengl_init(int win)
{ win_struct* wp = wlist[win];

  int bits = GetDeviceCaps(wp->hdc,BITSPIXEL); 
  
  static PIXELFORMATDESCRIPTOR pfd =
  { sizeof(PIXELFORMATDESCRIPTOR),                                
    1,                                                                              
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
    PFD_TYPE_RGBA,                          
    bits,                                                               
    0, 0, 0, 0, 0, 0,                       
    0,                                                                              
    0,                                                                              
    0,                                                                              
    0, 0, 0, 0,                                             
    bits,                                                                     
    0,                                                                              
    0,                                                                              
    PFD_MAIN_PLANE,                         
    0,                                                                              
    0, 0, 0                                                         
  };

  int PixelFormat = ChoosePixelFormat(wp->hdc,&pfd);
  if (!PixelFormat)
  { fprintf(stderr,"\nCannot choose the pixel format!");
    return;
  }
  
  if (!SetPixelFormat(wp->hdc,PixelFormat,&pfd))
  { fprintf(stderr,"\nCannot set the pixel format!");
    return;
  }                         
  
  wp->hRC = wglCreateContext(wp->hdc);  
  if (wp->hRC == NULL)
  { fprintf(stderr,"\nCannot create a rendering context!");
    return;  
  }
  
  if (!wglMakeCurrent(wp->hdc,wp->hRC))
  { fprintf(stderr,"\nCannot enable the context!");
    return;  
  }   
}

void x_opengl_start(int win)
{ win_struct* wp = wlist[win];
  
  wp->hDCret = wglGetCurrentDC();
  wp->hRCret = wglGetCurrentContext();    
  
  if (wp->hdc != wp->hDCret) 
  { wp->hrgnret = (HRGN) SelectClipRgn(wp->hdc,NULL);
    wglMakeCurrent(wp->hdc,wp->hRC); 
  }
}

void x_opengl_flush(int win)
{ win_struct* wp=wlist[win];  
  SwapBuffers(wp->hdc);
}

void x_opengl_finish(int win)
{ win_struct* wp=wlist[win];

  if (wp->hdc != wp->hDCret) 
  { wglMakeCurrent(wp->hDCret,wp->hRCret);  
    SelectClipRgn(wp->hdc,wp->hrgnret);
  }
}

void* x_opengl_win_struct(int win) 
{ return wlist[win]; }

LEDA_END_NAMESPACE
