#include <LEDA/graphics/d3_gl_scene.h>
#include <LEDA/graphics/d3_gl_window.h>


LEDA_BEGIN_NAMESPACE 


/*
#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::endl;
#endif
*/


d3_gl_scene::~d3_gl_scene()
{	
	delete pos1;
	delete col_node_item_array;
	delete col_edge_item_array;
	delete col_face_item_array;
	delete tex_face_item_array;

	col_node_seq.clear();
	col_edge_seq.clear();
	col_face_seq.clear();
	tex_face_seq.clear();
	
	while(!seq_mat.empty())
		delete[] seq_mat.pop();

	while(!sequences.empty()){
		transformation_list& transforms = sequences[sequences.first()];
		while(!transforms.empty())
			delete[] transforms.pop().second();
		sequences.erase(sequences.first());
	}		
}

void d3_gl_scene::init()
{
  draw_nodes  = true;
  draw_edges  = true;
  draw_faces  = true; 
  visible     = true;  
  draw_bounding_box = false;
  texturing  = false;
  node_width = 2.0f;  
  edge_width = 2.0f;  
  
  clear_matrix();       
  
  sequences.clear();
  seq_mat.clear();    
  
  d3_gl_win = NULL;   
  
  bb_line_width = 5.0f;
  bb_line_color.set_color(green);   
  
  pos1 = NULL;
  col_node_item_array = NULL;
  col_edge_item_array = NULL;
  col_face_item_array = NULL;
  tex_face_item_array = NULL;  
}

void d3_gl_scene::init_col_tex_seq()
{  
  // col nodes
  col_node_seq.clear();
  point_pointer_set S(compare);  
  nodes_to_points(Hp->all_nodes(),S);
  seq_item it = col_node_seq.insert(d3_gl_colF(red),S);  

  if(col_node_item_array)
	  delete col_node_item_array;
  col_node_item_array = new node_array<seq_item>(*Hp,it);
 
  //col edges
  if(col_edge_item_array)
	  delete col_edge_item_array;
  col_edge_item_array = new edge_array<seq_item>(*Hp,nil);

  col_edge_seq.clear();
  seq_item it2 = col_edge_seq.insert(d3_gl_colF(green),pp_tuple_set());
  pp_tuple_set& S2 = col_edge_seq[it2];    
  
  sortseq<node,node_set> node_to_adj_nodes;  
  edge e;
  forall_edges(e,*Hp){
	  it = node_to_adj_nodes.lookup(Hp->target(e)); 
	  if(it){
		  if(node_to_adj_nodes[it].member(Hp->source(e))) // if reverse edge already 
			  continue;                                   // has a color then continue
	  }

	  it = node_to_adj_nodes.lookup(Hp->source(e));
	  if(it)
		  node_to_adj_nodes[it].insert(Hp->target(e));
	  else{
		  node_set ns(compare);
		  ns.insert(Hp->target(e));
		  node_to_adj_nodes.insert(Hp->source(e),ns);		  
	  }

	  (*col_edge_item_array)[e] = it2;
	  edges_to_points(list<edge>(e),S2);
  }  

  //col faces
  col_face_seq.clear();
  pps_list S3;
  list<face> face_list = Hp->all_faces();
  faces_to_points(face_list,S3);
  seq_item it3 = col_face_seq.insert(d3_gl_colF(blue),S3);

  if(col_face_item_array)
	  delete col_face_item_array;
  col_face_item_array = new face_array<item_tuple>(*Hp); 

  list_item face_item = face_list.first();
  pps_list& S4 = col_face_seq[it3];
  list_item set_item = S4.first();  

  while(face_item != nil){
	  face& f = face_list[face_item];
	  (*col_face_item_array)[f] = item_tuple(it3,set_item);

	  face_item = face_list.succ(face_item);
	  set_item  = S4.succ(set_item);	  
  }  

  //tex faces 
  tex_face_seq.clear();  

  if(tex_face_item_array)
	  delete tex_face_item_array;
  tex_face_item_array = new face_array<item_tuple>(*Hp,item_tuple(nil,nil));    
}

void d3_gl_scene::exchange_graph(graph& G, const node_array<vector>& pos)
{
  Hp = &G;  
	
  if(pos1 != NULL) delete pos1;
  pos1 = new node_array<vector>(pos);  

  vector cg(0,0,0);
  node v;

  forall_nodes(v,*Hp) cg += (*pos1)[v];
  g_center = cg/Hp->number_of_nodes(); 
  
  comp_coord_extrema(xmin,xmax,ymin,ymax,zmin,zmax); 

  init_col_tex_seq();
}

void d3_gl_scene::exchange_graph(graph& G, const node_array<rat_vector>& pos)
{  
  Hp = &G;    

  if(pos1 != NULL) delete pos1;
  pos1 = new node_array<vector>(*Hp,vector(3));
  
  vector cg(0,0,0);
  node v;

  forall_nodes(v,*Hp) 
  { (*pos1)[v] = pos[v].to_vector();
    cg += (*pos1)[v];
   }
  
  g_center = cg/Hp->number_of_nodes();  
  comp_coord_extrema(xmin,xmax,ymin,ymax,zmin,zmax);

  init_col_tex_seq();
}


void d3_gl_scene::comp_coord_extrema(double& x0, double& x1, double& y0 ,
									 double& y1, double& z0, double& z1) const
{
	x0 = 0.0;	x1 = 0.0;
	y0 = 0.0;	y1 = 0.0;
	z0 = 0.0;	z1 = 0.0;  

	vector vec;
	node v;

	forall_nodes(v,*Hp){
		vec = (*pos1)[v];
		
		if(vec[0]>x1)
			x1 = vec[0];
		if(vec[0]<x0)
		    x0 = vec[0];

		if(vec[1]>y1)
			y1 = vec[1];
		if(vec[1]<y0)
			y0 = vec[1];

		if(vec[2]>z1)
			z1 = vec[2];
		if(vec[2]<z0)
			z0 = vec[2];
	}
}

void d3_gl_scene::comp_optimal_coord(double& x0, double& x1, double& y0 ,
									 double& y1, double& z0, double& z1) const
{
	comp_coord_extrema(x0,x1,y0,y1,z0,z1);
	double help;

	help = x0;
	x0 = (x0 - 2.0f) - ((x1-x0)*0.5f);
	x1 = (x1 + 2.0f) + ((x1-help)*0.5f);
	help = y0;
	y0 = (y0 - 2.0f) - ((y1-y0)*0.5f);
	y1 = (y1 + 2.0f) + ((y1-help)*0.5f);
	help = z0;
	z0 = (z0 - 2.0f) - ((z1-z0)*0.5f);
	z1 = (z1 + 2.0f) + ((z1-help)*0.5f);		
}

void d3_gl_scene::nodes_to_points(const list<node>& n_list, point_pointer_set& S) const
{
	node n;
	forall(n,n_list){
		vector& v = (*pos1)[n];
		S.insert(&v);
	}
}

void d3_gl_scene::edges_to_points(const list<edge>& e_list, pp_tuple_set& S) const
{	
	edge e;
	forall(e,e_list){
		node n1 = Hp->source(e);
		node n2 = Hp->target(e);
		vector& v1 = (*pos1)[n1];
		vector& v2 = (*pos1)[n2];
		S.insert(pp_tuple(&v1,&v2));		
	}	
}

void d3_gl_scene::faces_to_points(const list<face>& f_list, pps_list& L) const
{
	face f;
	forall(f,f_list){
		point_pointer_set temp_set(compare);
		edge first = Hp->first_face_edge(f);
		edge e = first;

		do{
			vector& v1 = (*pos1)[source(e)];			
			temp_set.insert(&v1);			
			e = Hp->face_cycle_succ(e);
		}while(e!=first);

		L.append(temp_set);
	}	
}


void d3_gl_scene::set_color(const d3_gl_colF& c, const list<node>& node_list)
{
	seq_item new_col_it = col_node_seq.lookup(c);
	if(new_col_it == nil)
		new_col_it = col_node_seq.insert(c,point_pointer_set(compare));

	set<seq_item> empty(compare);

	node n;
	forall(n,node_list){	
		seq_item old_col_it = (*col_node_item_array)[n];
		if(old_col_it != new_col_it){
			vector& v = (*pos1)[n];
			point_pointer_set& S = col_node_seq[old_col_it];
			S.del(&v);

			if(S.empty() && !empty.member(old_col_it))
				empty.insert(old_col_it);			

			col_node_seq[new_col_it].insert(&v);

			(*col_node_item_array)[n] = new_col_it;
		}
	}

	seq_item seq_it;
	forall(seq_it,empty)
		col_edge_seq.del_item(seq_it);
	
	if(col_node_seq[new_col_it].empty())
		col_node_seq.del_item(new_col_it);
}

void d3_gl_scene::set_color(const d3_gl_colF& c, const list<edge>& edge_list)
{
	seq_item new_col_it = col_edge_seq.lookup(c);
	if(new_col_it == nil)
		new_col_it = col_edge_seq.insert(c,pp_tuple_set(compare));	

	set<seq_item> empty(compare);

	edge e;
	forall(e,edge_list){	
		seq_item old_col_it = (*col_edge_item_array)[e];

		if(old_col_it == nil){
			list<edge> edges = Hp->adj_edges(Hp->target(e));
			edge e2;
			forall(e2,edges){
				if(Hp->target(e2) == Hp->source(e)){
					old_col_it = (*col_edge_item_array)[e2];

					if(old_col_it == nil)
						LEDA_EXCEPTION(1,"d3_gl_scene:: invalid item in set_color");

					break;
				}
			}
		}
		
		if(old_col_it != new_col_it){			
			node n1 = Hp->source(e);
			node n2 = Hp->target(e);
			vector& v1 = (*pos1)[n1];
			vector& v2 = (*pos1)[n2];			

			pp_tuple_set& S = col_edge_seq[old_col_it];
			S.del(pp_tuple(&v1,&v2));

			if(S.empty() && !empty.member(old_col_it))
				empty.insert(old_col_it);				

			col_edge_seq[new_col_it].insert(pp_tuple(&v1,&v2));

			(*col_edge_item_array)[e] = new_col_it;
		}		
	}
	
	seq_item seq_it;
	forall(seq_it,empty)
		col_edge_seq.del_item(seq_it);
	
	if(col_edge_seq[new_col_it].empty())
		col_edge_seq.del_item(new_col_it);
}

void d3_gl_scene::set_color(const d3_gl_colF& c, const list<face>& face_list)
{
	seq_item new_col_it = col_face_seq.lookup(c);
	if(new_col_it == nil)
		new_col_it = col_face_seq.insert(c,pps_list());	

	face f;
	forall(f,face_list){	
		seq_item old_col_it = (*col_face_item_array)[f].first();
		if(old_col_it != new_col_it){
			pps_list& L = col_face_seq[old_col_it];
			list_item old_item = (*col_face_item_array)[f].second();
			list_item new_item = col_face_seq[new_col_it].append(L[old_item]);
			L.erase(old_item);

			if(L.empty())
				col_face_seq.del_item(old_col_it);

			(*col_face_item_array)[f].first() = new_col_it;
			(*col_face_item_array)[f].second() = new_item;
		}
	}
	
	if(col_face_seq[new_col_it].empty())
		col_face_seq.del_item(new_col_it);
}


void d3_gl_scene::set_color(const color& c, const list<node>& node_list, const float& a)
{
	if(a < 0.0f || a > 1.0f) return;
	set_color(d3_gl_colF(c,a),node_list);
}

void d3_gl_scene::set_color(const color& c, const list<edge>& edge_list, const float& a)
{	
	if(a < 0.0f || a > 1.0f) return;
	set_color(d3_gl_colF(c,a),edge_list);
}

void d3_gl_scene::set_color(const color& c, const list<face>& face_list, const float& a)
{	
	if(a < 0.0f || a > 1.0f) return;
	set_color(d3_gl_colF(c,a),face_list);
}


d3_gl_colF d3_gl_scene::get_d3gl_color(const node& n) const
{	return col_node_seq.key((*col_node_item_array)[n]);	  }

d3_gl_colF d3_gl_scene::get_d3gl_color(const edge& e) const
{
	if((*col_edge_item_array)[e] == nil){
		list<edge> edges = Hp->adj_edges(Hp->target(e));
		edge e2;
		forall(e2,edges){
			if(Hp->target(e2) == Hp->source(e)){
				return col_edge_seq.key((*col_edge_item_array)[e]);
			}
		}

		LEDA_EXCEPTION(1,"d3_gl_scene:: wrong edge color");
		return d3_gl_colF(white); 
	}
	else
		return col_edge_seq.key((*col_edge_item_array)[e]);
}

d3_gl_colF d3_gl_scene::get_d3gl_color(const face& f) const
{	return col_face_seq.key((*col_face_item_array)[f].first());	  }


color d3_gl_scene::get_color(const node& n) const
{	return ((color)col_node_seq.key((*col_node_item_array)[n]));	  }

color d3_gl_scene::get_color(const edge& e) const
{	
	d3_gl_colF c = get_d3gl_color(e);
    return ((color)c); 
}

color d3_gl_scene::get_color(const face& f) const
{	return ((color)col_face_seq.key((*col_face_item_array)[f].first()));	  }


int d3_gl_scene::reg_tex(char* t_data, const int& w, const int& h,const GLint& filter)
{
	GLuint tex_num;

	if(d3_gl_win){
		if(d3_gl_win->gl_win){	
			d3_gl_win->reg_scene_tex(&tex_num,t_data,w,h,filter);					
		}
	}
	
	tex_info T(t_data,w,h,filter,tex_num);
	tex_face_seq.insert(tex_counter,tex_points_tuple(pps_list(),T)); // empty list			
	return tex_counter++;
}

int d3_gl_scene::reg_tex(char* t_data, const int& w, const int& h,
			             const GLint& filter, const list<face>& face_list)
{	
	int i = reg_tex(t_data,w,h,filter);
	set_texture(i,face_list);		
	return i;
}

bool d3_gl_scene::unreg_tex(int tex_num)
{	
	seq_item it = tex_face_seq.lookup(tex_num);
	if(it){
		face f;
		forall_faces(f,*Hp){
			if((*tex_face_item_array)[f].first() == it)
				(*tex_face_item_array)[f] = item_tuple(nil,nil);
		}

		if(d3_gl_win){
			if(d3_gl_win->gl_win){	
				d3_gl_win->gl_win->gl_start();
				GLuint gl_n = tex_face_seq[it].second().gl_num;
				glDeleteTextures(1,&gl_n);
				d3_gl_win->gl_win->gl_finish();
			}
		}
		
		tex_face_seq.del_item(it);

		return true;
	}
	else
		return false;
}

bool d3_gl_scene::set_texture(int tex_num, const list<face>& face_list)
{
	seq_item new_tex_it = tex_face_seq.lookup(tex_num);
	if(new_tex_it == nil) return false;		

	face f;
	forall(f,face_list){
		if((*tex_face_item_array)[f] == item_tuple(nil,nil)){
			pps_list L;
			faces_to_points(list<face>(f),L);
			list_item it = tex_face_seq[new_tex_it].first().append(L.pop());
			(*tex_face_item_array)[f].first()  = new_tex_it;
			(*tex_face_item_array)[f].second() = it;
		}
		else{		
			seq_item old_tex_it = (*tex_face_item_array)[f].first();
			if(old_tex_it != new_tex_it){
				pps_list& L = tex_face_seq[old_tex_it].first();
				list_item old_item = (*tex_face_item_array)[f].second();
				list_item new_item = tex_face_seq[new_tex_it].first().append(L[old_item]);
				L.erase(old_item);
				if(L.empty())
					tex_face_seq.del_item(old_tex_it);

				(*tex_face_item_array)[f].first()  = new_tex_it;
				(*tex_face_item_array)[f].second() = new_item;
			}
		}
	}	
	
	if(tex_face_seq[new_tex_it].first().empty())
		tex_face_seq.del_item(new_tex_it);

	return true;
}

int d3_gl_scene::get_texture(const face& f) const
{
	if((*tex_face_item_array)[f] == item_tuple(nil,nil) )
		return -1;
	else
		return tex_face_seq.key((*tex_face_item_array)[f].first());
}

GLint d3_gl_scene::get_tex_filter(int tex_num) const
{
	seq_item it = tex_face_seq.lookup(tex_num);
	if(it == nil) return -1;

	return tex_face_seq[it].second().filter;
}

bool d3_gl_scene::set_tex_filter(int tex_num, const GLint& filter)
{	
	seq_item it = tex_face_seq.lookup(tex_num);
	if(it == nil) return false;

	tex_face_seq[it].second().filter = filter;

	if(d3_gl_win == NULL) return true;
	if(d3_gl_win->gl_win == NULL) return true;

	d3_gl_win->gl_win->gl_start();

	GLuint gl_n = tex_face_seq[it].second().gl_num;
	glBindTexture(GL_TEXTURE_2D,gl_n);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);

	d3_gl_win->gl_win->gl_finish();
	
	return true;
}

char* d3_gl_scene::get_tex_data(int tex_num) const
{ 
	seq_item it = tex_face_seq.lookup(tex_num);
	if(it == nil) return NULL;

	return tex_face_seq[it].second().data;	
}
 
int d3_gl_scene::number_of_textures() const
{ return tex_face_seq.size()-1; }


void d3_gl_scene::update_textures()
{
	GLuint gl_id;
	seq_item it = tex_face_seq.min_item();

	while(it != nil){		
		tex_info& T = tex_face_seq[it].second();
		d3_gl_win->reg_scene_tex(&gl_id,T.data,T.w,T.h,T.filter);
		d3_gl_win->textures.del(&gl_id);			
		T.gl_num = gl_id;

		it = tex_face_seq.succ(it);
	}	
}

void d3_gl_scene::draw() 
{   
  glPushMatrix();     
  
  if(seq_mat.length()){	  	  	  

	  list_item mat  = seq_mat.first();
	  list_item kind = sequences.first();	  

	  while(mat!=nil){		  
		  glMultMatrixd(seq_mat[mat]);
		  transformation_list& transforms = sequences[kind];

		  list_item it = transforms.first();
		  while(it != nil){
			  transformation& T = transforms[it];
			  GLdouble*& d = T.second();

			  if(T.first() == D3_GL_TRANS){				  
				  glTranslated(d[0],d[1],d[2]);				  
				  d[0] += d[3];
				  d[1] += d[4];
				  d[2] += d[5];
			  }
			  else if(T.first() == D3_GL_SCALE){				  
				  glScaled(d[0],d[1],d[2]);				  
				  d[0] += d[3];
				  d[1] += d[4];
				  d[2] += d[5];
			  }
			  else if(T.first() == D3_GL_ROT){				  
				  glRotated(d[0],d[2],d[3],d[4]);				  
				  d[0] += d[1];
			  }  

			  it = transforms.succ(it);
		  }          		  

		  mat  = seq_mat.succ(mat);
		  kind = sequences.succ(kind);
	  }	  	  
  }  

  glMultMatrixd(trans_mat);  
  
  
  if(draw_faces)
  {
    point_pointer P;
    seq_item it1 = col_face_seq.min_item();
	while(it1 != nil){		
		d3_gl_colF temp_col = col_face_seq.key(it1);
		temp_col();		

		pps_list& L = col_face_seq[it1];
		list_item it2 = L.first();
		while(it2 != nil){
			point_pointer_set& S = L[it2];			
			
			glBegin(GL_POLYGON);

			forall(P,S)
				gl_vertex(*P);									    										

			glEnd();			

			it2 = L.succ(it2);
		}

		it1 = col_face_seq.succ(it1);
	}     
  }  
  
  if(texturing)
  {	 	  
      glEnable(GL_TEXTURE_2D);
	  
	  seq_item it1 = tex_face_seq.min_item();

	  float a[4][2] = { { 1.0f,1.0f }, { 0.0f,1.0f }, { 0.0f,0.0f }, { 1.0f, 0.0f } };
	  int c = 0;
	  point_pointer P;

	  while(it1 != nil){		  
		  glBindTexture(GL_TEXTURE_2D, tex_face_seq[it1].second().gl_num);
		  pps_list& L = tex_face_seq[it1].first();			  

		  list_item it2 = L.first();
		  while(it2 != nil){
			  point_pointer_set& pps = L[it2];
			  it2 = L.succ(it2);
			  if(pps.size()>4) continue;
	
			  glBegin(GL_POLYGON);
			  
			  forall(P,pps){
				  glTexCoord2f(a[c][0],a[c][1]); gl_vertex(*P);
				  ++c;				  
			  }	
			  c = 0;

			  glEnd();			  
		  }			  
		  it1 = tex_face_seq.succ(it1);		  
	  }

	  glDisable(GL_TEXTURE_2D);	  	  
  }    
  
  if(draw_edges)
  {	
    glLineWidth(edge_width);
	glBegin(GL_LINES);	

	seq_item it = col_edge_seq.min_item();
	while(it != nil){		
		d3_gl_colF temp_col = col_edge_seq.key(it);
		temp_col();	

		pp_tuple_set& S = col_edge_seq[it];
		pp_tuple T;
		forall(T,S){
			gl_vertex(*T.first());
			gl_vertex(*T.second());			
		}	

		it = col_edge_seq.succ(it);
	}    		

	glEnd();	
   }      

  
  if(draw_nodes)
  {
	glPointSize(node_width);
	glBegin(GL_POINTS);

	seq_item it = col_node_seq.min_item();
	while(it != nil){		
		d3_gl_colF temp_col = col_node_seq.key(it);
		temp_col();	

		point_pointer_set& S = col_node_seq[it];
		point_pointer pp;
		forall(pp,S)
			gl_vertex(*pp);	

		it = col_node_seq.succ(it);
	} 		

	glEnd();	
  } 
  
  if(draw_bounding_box){
	  glLineWidth(bb_line_width);	  
	  bb_line_color();
	  
	  glBegin(GL_LINE_LOOP); // left
		glVertex3d(xmin,ymin,zmin); 
		glVertex3d(xmin,ymin,zmax); 
		glVertex3d(xmin,ymax,zmax); 
		glVertex3d(xmin,ymax,zmin); 
	  glEnd();

	  glBegin(GL_LINE_LOOP); // right	
	    glVertex3d(xmax,ymin,zmin); 
		glVertex3d(xmax,ymin,zmax); 
		glVertex3d(xmax,ymax,zmax); 
		glVertex3d(xmax,ymax,zmin); 
	  glEnd();

	  glBegin(GL_LINES);     // the rest	
	    glVertex3d(xmin,ymin,zmin); glVertex3d(xmax,ymin,zmin); 
		glVertex3d(xmin,ymin,zmax); glVertex3d(xmax,ymin,zmax);
		glVertex3d(xmin,ymax,zmax); glVertex3d(xmax,ymax,zmax);
		glVertex3d(xmin,ymax,zmin); glVertex3d(xmax,ymax,zmin);
	  glEnd();	  	       
  }  

  
  glPopMatrix();    
  
}


// ----------------------------------------------
// affine transformations

void d3_gl_scene::mult_matrix(const double* m)
{
	double result[] = { 0.0, 0.0, 0.0, 0.0,
		                0.0, 0.0, 0.0, 0.0,
					    0.0, 0.0, 0.0, 0.0,
					    0.0, 0.0, 0.0, 0.0 };

	int k;

	for(k=0; k<4; k++)
		for(int i=0; i<4; i++)
			for(int j=0; j<4; j++)
				result[i*4+k] += trans_mat[j*4+k]*m[j+i*4];		

	for(k=0; k<16; k++)
		trans_mat[k] = result[k];
}

void d3_gl_scene::clear_matrix()
{
	for(int i=0; i<16 ;i++){
	  if(i % 5 == 0)
		trans_mat[i] = 1.0;
	  else
		trans_mat[i] = 0.0;
	}
}

void d3_gl_scene::print_matrix() const
{	
	for(int j=0; j<4 ; j++){
	  for(int k=0; k<4; k++)
		  cout<<trans_mat[k*4+j]<<"  ";
	  cout<<"\n";
	}
	cout<<"\n";
}

void d3_gl_scene::set_matrix(const double* m)
{
	for(int i=0; i<16; i++)
		trans_mat[i] = m[i];
}

const GLdouble* d3_gl_scene::get_matrix() const
{ return trans_mat; }



void d3_gl_scene::translate(const double& x, const double& y, const double& z,
							const bool& first)
{
	GLdouble T[] = { 1.0, 0.0, 0.0, 0.0,
			   	     0.0, 1.0, 0.0, 0.0,
				     0.0, 0.0, 1.0, 0.0,
				     x  , y  , z  , 1.0 };
	if(first){
		GLdouble temp[16];
		for(int i=0; i<16; i++)
			temp[i] = trans_mat[i];

		set_matrix(T);
		mult_matrix(temp);
	}
	else
		mult_matrix(T);		
}


void d3_gl_scene::scale(const double& x, const double& y, const double& z, const bool& first)
{
	GLdouble S[] = { x  , 0.0, 0.0, 0.0,
				     0.0, y  , 0.0, 0.0,
				     0.0, 0.0, z  , 0.0,
				     0.0, 0.0, 0.0, 1.0 };
	
	if(first){
		GLdouble temp[16];
		for(int i=0; i<16; i++)
			temp[i] = trans_mat[i];

		set_matrix(S);
		mult_matrix(temp);
	}
	else
		mult_matrix(S);	
}

void d3_gl_scene::rotate(const double& alpha, double x, double y,double z, const bool& first)
{
	double l = ::sqrt(x*x + y*y + z*z);
	x = x/l;
	y = y/l;
	z = z/l;

	double cosa = cos(vgl::deg_to_rad(alpha));
	double sina = sin(vgl::deg_to_rad(alpha));	

	GLdouble R[] =
	{ x*x+cosa-cosa*x*x  , y*x-cosa*y*x+sina*z, z*x-cosa*z*x-sina*y, 0.0,
	  x*y-cosa*x*y-sina*z, y*y+cosa-cosa*y*y  , z*y-cosa*z*y+sina*x, 0.0,
	  x*z-cosa*x*z+sina*y, y*z-cosa*y*z-sina*x, z*z+cosa-cosa*z*z  , 0.0,
	  0.0                , 0.0                , 0.0                , 1.0 };

	if(first){
		GLdouble temp[16];
		for(int i=0; i<16; i++)
			temp[i] = trans_mat[i];

		set_matrix(R);
		mult_matrix(temp);
	}
	else
		mult_matrix(R);
}


void d3_gl_scene::get_modelview_matrix(GLdouble* matrix) 
{   
  if(d3_gl_win == NULL) return;
  if(d3_gl_win->gl_win == NULL) return;

  d3_gl_win->gl_win->gl_start();         
	
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();  

  glPushMatrix();   
  

  if(seq_mat.length()){	  	  	  

	  list_item mat  = seq_mat.first();
	  list_item kind = sequences.first();	  

	  while(mat!=nil){		  
		  glMultMatrixd(seq_mat[mat]);
		  transformation_list& transforms = sequences[kind];

		  list_item it = transforms.first();
		  while(it != nil){
			  transformation& T = transforms[it];
			  GLdouble*& d = T.second();

			  if(T.first() == D3_GL_TRANS){				  
				  glTranslated(d[0],d[1],d[2]);				  
				  d[0] += d[3];
				  d[1] += d[4];
				  d[2] += d[5];
			  }
			  else if(T.first() == D3_GL_SCALE){				  
				  glScaled(d[0],d[1],d[2]);				  
				  d[0] += d[3];
				  d[1] += d[4];
				  d[2] += d[5];
			  }
			  else if(T.first() == D3_GL_ROT){				  
				  glRotated(d[0],d[2],d[3],d[4]);				  
				  d[0] += d[1];
			  }  

			  it = transforms.succ(it);
		  }          		  

		  mat  = seq_mat.succ(mat);
		  kind = sequences.succ(kind);
	  }	  	  
  }    
  
  glMultMatrixd(trans_mat);	  
  
  glGetDoublev(GL_MODELVIEW_MATRIX,matrix);  

  glPopMatrix();

  d3_gl_win->gl_win->gl_finish();
  
}


int d3_gl_scene::begin_sequence()
{
	if(sequences.length()){
		if(sequences[sequences.last()].length() == 0)
			return (sequences.length()-1);
	}

	// add empty list 
	sequences.append(transformation_list());

	// store actual matrix
	GLdouble* trans_mat2 = new double[16];
	for(int i=0; i<16; i++)
		trans_mat2[i] = trans_mat[i];
	seq_mat.append(trans_mat2);

	// reset actual matrix
	clear_matrix();

	return (seq_mat.length()-1);
}

// has currently no function ; but it helps to structure the code
int d3_gl_scene::end_sequence()
{   return (seq_mat.length()-1); }


bool d3_gl_scene::remove_sequence(const int& seq_num)
{
	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return false;
	
	double* stored_mat = seq_mat[seq_mat.get_item(seq_num)];
	double  actual_mat[16];

	for(int i=0; i<16; i++)
		actual_mat[i] = trans_mat[i];

	set_matrix(stored_mat);	
	
	transformation t;
	transformation_list& transforms = sequences[sequences.get_item(seq_num)];	  	

	forall(t,transforms){
      GLdouble*& d = t.second();
	  if(t.first() == D3_GL_TRANS){		  
		  translate(d[0],d[1],d[2]);
		  d[0] += d[3];
		  d[1] += d[4];
		  d[2] += d[5];
	  }
	  else if(t.first() == D3_GL_SCALE){		  
		  scale(d[0],d[1],d[2]);
		  d[0] += d[3];
		  d[1] += d[4];
		  d[2] += d[5];
	  }
	  else if(t.first() == D3_GL_ROT){		  
		  rotate(d[0],d[2],d[3],d[4]);
		  d[0] += d[1];
	  }
	  delete[] d;
	}		  	  
	  
	transforms.clear();
	
	mult_matrix(actual_mat);

	delete[] seq_mat.del(seq_mat.get_item(seq_num));
	sequences.del(sequences.get_item(seq_num));

	return true;
}


int d3_gl_scene::seq_translate(const double* params, int seq_num)
{
	if(seq_num == -1) seq_num = sequences.length()-1;

	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return -1;

	GLdouble* trans_kind = new double[6];	
	for(int i=0; i<6; i++)
		trans_kind[i] = params[i];

	list_item it = sequences.get_item(seq_num);
	sequences[it].append(transformation(D3_GL_TRANS,trans_kind));
	return (sequences[it].length()-1);
}


int d3_gl_scene::seq_scale(const double* params, int seq_num)
{
	if(seq_num == -1) seq_num = sequences.length()-1;

	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return -1;

	GLdouble* trans_kind = new double[6];	
	for(int i=0; i<6; i++)
	 	trans_kind[i] = params[i];

	list_item it = sequences.get_item(seq_num);
	sequences[it].append(transformation(D3_GL_SCALE,trans_kind));
	return (sequences[it].length()-1);
}


int d3_gl_scene::seq_rotate(const double* params, int seq_num)
{
	if(seq_num == -1) seq_num = sequences.length()-1;

	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return -1;

    GLdouble* trans_kind = new double[5];	
	for(int i=0; i<5; i++)
		trans_kind[i] = params[i];

	list_item it = sequences.get_item(seq_num);
	sequences[it].append(transformation(D3_GL_ROT,trans_kind));
	return (sequences[it].length()-1);
}

int d3_gl_scene::seq_translate(const double& x , const double& y , const double& z,
	 						   const double& x2, const double& y2, const double& z2,
							   int seq_num)
{
	double params[6];
	params[0] = x;  params[1] = y;  params[2] = z;
	params[3] = x2; params[4] = y2; params[5] = z2;

	return seq_translate(params,seq_num);
}


int d3_gl_scene::seq_scale(const double& x , const double& y , const double& z,
						   const double& x2, const double& y2, const double& z2,
						   int seq_num)
{
	double params[6];
	params[0] = x;  params[1] = y;  params[2] = z;
	params[3] = x2; params[4] = y2; params[5] = z2;

	return seq_scale(params,seq_num);

}


int d3_gl_scene::seq_rotate(const double& start_angle, const double& alpha, const double& x,
						    const double& y, const double& z, int seq_num)
{
	double params[5];
	params[0] = start_angle;  params[1] = alpha;
	params[2] = x; params[3] = y; params[4] = z;

	return seq_rotate(params,seq_num);
}


bool d3_gl_scene::set_seq_transform(const int& seq_num, const int& transform_num,
								const double* params)
{
	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return false;

	transformation_list& trans_list = sequences[sequences.get_item(seq_num)];

	if(transform_num < 0 || transform_num > (trans_list.length()-1))
		return false;

	transformation& T = trans_list[trans_list.get_item(transform_num)];

	int l;
	if(T.first() == D3_GL_ROT)
		 l = 5;
	else l = 6;

	GLdouble*& d = T.second();

	for(int i=0; i<l; i++)
		d[i] = params[i];

	return true;
}

double* d3_gl_scene::get_seq_transform(const int& seq_num, const int& transform_num,
									   int& l) const
{
	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return NULL;

	const transformation_list& trans_list = sequences[sequences.get_item(seq_num)];

	if(transform_num < 0 || transform_num > (trans_list.length()-1))
		return NULL;

	const transformation& T = trans_list[trans_list.get_item(transform_num)];

	if(T.first() == D3_GL_ROT) l = 5;
	else                       l = 6; 

	return T.second();
}

bool d3_gl_scene::remove_seq_transform(const int& seq_num, const int& transform_num)
{
	if(seq_num < 0 || seq_num > (sequences.length()-1))
		return false;

	transformation_list& trans_list = sequences[sequences.get_item(seq_num)];

	if(transform_num < 0 || transform_num > (trans_list.length()-1))
		return false;

	trans_list.erase(trans_list.get_item(transform_num));	
	
	return true;
}


void d3_gl_scene::set_position(const node& n, const double& x, const double& y,
							   const double& z)
{
  (*pos1)[n] = vector(x,y,z); 
  
  if(x<ymin) xmin = x;
  if(x>ymax) xmax = x;
  if(y<ymin) ymin = y;
  if(y>ymax) ymax = y;
  if(z<zmin) zmin = z;
  if(z>zmax) zmax = z;  

  vector cg(0,0,0);
  node n2;

  forall_nodes(n2,*Hp) cg += (*pos1)[n2];
  g_center = cg/Hp->number_of_nodes();
}

void d3_gl_scene::set_position_array(const node_array<vector>& pos)
{
  const graph& G = pos.get_graph();
  if(&G != Hp) return;  

  vector cg(0,0,0);
  node v;

  forall_nodes(v,*Hp){
	  (*pos1)[v] = pos[v];
	  cg += pos[v];
  }
  g_center = cg/Hp->number_of_nodes(); 
  
  comp_coord_extrema(xmin,xmax,ymin,ymax,zmin,zmax);    
}


int d3_gl_scene::tex_counter = 0;


tex_info::tex_info()
{		
	data = NULL;
	w = h = 0;
	filter = 0;
	gl_num = 0;
}

tex_info::tex_info(char* d, const int& w2, const int& h2,
		const GLint& f, const GLuint& n)
{
	data = d;
	w = w2;
	h = h2;
	filter = f;
	gl_num = n;
}


int compare(const d3_gl_scene& scene1, const d3_gl_scene& scene2)
{ 
	if(scene1.get_position_array() < scene2.get_position_array())
		return -1;
	else if( scene1.get_position_array() > scene2.get_position_array())
		return 1;
	else
		return 0;
}

int compare(const SC_PTR& scene1, const SC_PTR& scene2)
{
	if(scene1 < scene2)
		return -1;
	else if( scene1 > scene2)
		return 1;
	else
		return 0;
} 

int compare(const point_pointer& p1, const point_pointer& p2)
{
	if(p1 < p2)
		return -1;
	else if( p1 > p2)
		return 1;
	else
		return 0;
}

int compare(const seq_item& it1, const seq_item& it2)
{
	if(it1 < it2)
		return -1;
	else if( it1 > it2)
		return 1;
	else
		return 0;
}


LEDA_END_NAMESPACE
