// compression module includes
#include <LEDA/coding/internal/streambuf_workaround.h>

// core LEDA includes
#include <string.h> // for memcpy (see below)

/*
#if defined(LEDA_DEBUG)
#include <LEDA/system/assert.h>
#else
#undef assert
#define	assert(cond) ((void)0)
#endif
*/

LEDA_BEGIN_NAMESPACE

#ifdef LEDA_COMPRESSION_WA1
streamsize local_streambuf_wa1::xsgetn(char* buffer, streamsize count)
{
	streamsize total_read = 0;

	while (count > 0) {
		int pending;
		if (gptr() && (pending = (int)(egptr() - gptr())) > 0) {
			// copy from internal buffer
			if (count < pending) pending = count;
			memcpy(buffer, gptr(), pending);
			buffer += pending; total_read += pending; count -= pending;
			gbump(pending);
			continue;
		}
		
		int c = uflow();
		if (c < 0) break;

		*buffer++ = (char) c; ++total_read; --count;
	}

	return total_read;
}


streamsize local_streambuf_wa1::xsputn(const char* buffer, streamsize count)
{
	streamsize total_written = 0;

	while (count > 0) {
		int space;
		if (pptr() && (space = (int)(epptr() - pptr())) > 0) {
			// copy from internal buffer
			if (count < space) space = count;
			memcpy(pptr(), buffer, space);
			buffer += space; total_written += space; count -= space;
			pbump(space);
			continue;
		}
		
		int c = overflow((unsigned char) *buffer);
		if (c < 0) break;

		++buffer; ++total_written; --count;
	}

	return total_written;
}

#else
// LEDA_COMPRESSION_WA1 not defined
// dummies to avoid "has no symbols" warning

streamsize local_streambuf_wa1::xsgetn(char*, streamsize) { return 0; }
streamsize local_streambuf_wa1::xsputn(const char*, streamsize) { return 0; }

#endif

LEDA_END_NAMESPACE
