#include <LEDA/core/list.h>
#include <LEDA/core/string.h>
#include <LEDA/core/array.h>
#include <LEDA/system/file.h>

#include <LEDA/coding/crypt.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::cerr;
using std::endl;
using std::ostream;
using std::istream;
using std::ofstream;
using std::ifstream;
using std::streambuf;
using std::filebuf;
using std::ios;
#endif

/*
#ifdef LEDA_COMPRESSION_WA1
#define openmode open_mode
#endif
*/

#define open_in  coder_base::binary_in()
#define open_out coder_base::binary_out()

/*
//typedef CoderPipe3<BWTCoder, MTFCoder, A0Coder> Coder;
typedef PPMIICoder Coder;
typedef CFBCoder<Rijndael> CipherCoder;
*/

// display short help message

istream* setup_istream(string archive_file)
{
  typedef decoding_istream<CryptAutoDecoder> AutoDecIStream;

  filebuf* p_raw_filebuf = new filebuf;
  if (! p_raw_filebuf->open(archive_file, open_in)) {
    cerr << "could not open arcrive " << archive_file << endl; return 0;
  }

  istream arc_raw(p_raw_filebuf); 
  // data read from this stream is not decoded


  // check for id "MZ."
  char id_buffer[3];
  arc_raw.read(id_buffer, 3);
  if (arc_raw.fail() || id_buffer[0] != 'M' 
                     || id_buffer[1] != 'Z' 
                     || id_buffer[2] > 1) 
 { 
    cerr << "archive corrupted" << endl; 
    return 0;
  }

  bool crypt = (id_buffer[2] & 0x01) != 0;

  if (crypt)
  { cerr << "error: crypted " << endl;
    return 0;
   }

  // prepare decoding (with CryptAutoDecoder this is easy)
  AutoDecIStream* p_arc = new AutoDecIStream(p_raw_filebuf, true);

  // display some coder info
  char ch; 
  p_arc->get(ch); 
  p_arc->putback(ch); // we have to read s.th. first!
  cout << "  using coder: " << p_arc->get_coder()->get_description() << endl;
  return p_arc;
}

int read_toc(istream& arc, array<string>& name_of_file, 
                           array<int>& size_of_file)
{
  // check TOC start marker
  char ch = 0;
  arc >> ch; if (arc.fail() || ch != '#') 
  { cerr << "archive table of contents corrupted" << endl; 
    return 1; 
   }

  // read table_of_contents
  int num_of_files; arc >> num_of_files; 
  arc.get(ch); 
  if (arc.fail() || ch != '\n') 
  { cerr << "archive table of contents corrupted" << endl; 
    return 1; 
   }

  if (num_of_files == 0) return 2;

  name_of_file.resize(1, num_of_files);
  size_of_file.resize(1, num_of_files);

  int i;
  for (i=1; i<=num_of_files; ++i) {
    arc >> name_of_file[i] >> size_of_file[i];
  }

  // check TOC end marker
  arc >> ch; 
  if (arc.fail() || ch != '#') 
 { cerr << "archive table of contents corrupted" << endl; 
   return 1; 
  }

  return 0;
}

int decode_files(string archive_file)
{
  cout << "Unpacking " << archive_file << " ... " << endl;

  // generate the archive-istream "arc"
  istream* p_arc = setup_istream(archive_file);
  if (! p_arc) return 1;
  istream& arc = *p_arc;

  // read table of contents
  array<string> name_of_file;
  array<int>    size_of_file;

  if (read_toc(arc, name_of_file, size_of_file) != 0) 
  { 
    return 1;
   }

  // decode the files

  for (int i=name_of_file.low(); i<=name_of_file.high(); ++i) {
    string filename = name_of_file[i];
    cout << i << ": " << filename << " ... "; 
    cout.flush();

    // open output file
    ofstream out(filename);

    if (out.fail())
    { cerr << filename << ": cannot open" << endl; 
      return 1; 
     }

    istream in(arc.rdbuf());

    char line[256];
    int sz = size_of_file[i];
    int n = 0;
    while (in) 
    { in.getline(line,256);
      out << line << endl;
      n += strlen(line);
      n++; // EOL
     }

/*
    char c1,c2,c3;
    in.get(c1); 
    in.get(c2); 
    in.get(c3); 
    if (in.fail() || c1 != '#' || c2 != 'L' || c3 != '#') 
    { cout << "archive corrupted" << endl; 
      return 1; 
    }
*/

    cout << n << " bytes" << endl;
  }

  delete p_arc;
  return 0;
}




int main(int argc, char* argv[])
{
  if (argc < 2) 
  { cerr << "usage: leda_unzip archive" << endl; 
    return 1; 
   }

  return decode_files(argv[1]);
}

