/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  md5sum.cpp
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


/*
	This program demonstrates the use of the LEDA checksummers.
 */

#include <LEDA/core/string.h>
#include <LEDA/system/file.h>
#include <LEDA/coding/compress.h>

typedef leda::MD5SumCoder checksummer;

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cerr;
using std::cin;
using std::endl;
using std::streambuf;
using std::ostream;
using std::istream;
#endif

#ifdef LEDA_COMPRESSION_WA1
#define openmode open_mode
#endif

static void usage(const char* my_name)
{
	checksummer coder;
	cerr << "usage: " << my_name << " file | directory" << endl;
	cerr << "  computes a checksum of the given file or files in the given directory" << endl;
	cerr << "  using " << coder.get_description() << endl;
	cerr << endl;
}

static void process_file(string path, int indent = 0)
{
	checksummer checksum(path);
	checksum.set_checksums_in_stream_flag(false);
	checksum.check();

	string delim = get_directory_delimiter();
	string filename = path;
	int last_pos = -1, pos;
	while ( (pos=filename.index(delim, last_pos+1)) >= 0 ) last_pos = pos; 
	filename = filename(last_pos+1, filename.length()-1);

	for(int i = 0; i < indent; ++i) cout << " ";
	if (checksum.failed()) cout << "check failed!";
	else                 cout << checksum.get_checksum_as_hex_string();
	cout << " <- " << filename << endl;
}

static void process_directory(string path, int indent = 0)
{
	for(int i = 0; i < indent; ++i) cout << " ";
	cout << "Processing directory " << path << " ..." << endl;

	list<string> files = get_files(path);
	string file;
	forall(file, files) {
		if (file == "." || file == "..") continue;
		process_file(path + get_directory_delimiter() + file, 2);
	}
}

int main(int argc, char* argv[]) 
{
	if (argc <= 1) { usage(argv[0]); return 1; }

	string path = argv[1];

	if (is_file(path)) process_file(path);
	else if (is_directory(path)) process_directory(path);
	else {
		cerr << "Path " << path << " not found!" << endl;
		return 1;
	}

	return 0;
}
