/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  simple_test.cpp
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


#include <LEDA/core/string.h>
#include <LEDA/system/file.h>
#include <LEDA/coding/compress.h>

#include <fstream>

typedef leda::A0Coder Coder;

using namespace leda;

using std::cout;
using std::endl;
using std::ostream;
using std::istream;
using std::filebuf;
using std::ios;

#ifdef LEDA_COMPRESSION_WA1
#define openmode open_mode
#endif

void file_test(const char* tmp_file_orig, const char* tmp_file_enc, const char* tmp_file_dec)
{
	string orig = "Hello";

	{
		filebuf out;
		out.open(tmp_file_orig, ios::openmode(ios::out | coder_base::binary));
		ostream o(&out);
		o << orig;
	}

	{
		filebuf in;
		in.open(tmp_file_orig, ios::openmode(ios::in | coder_base::binary));
		filebuf out;
		out.open(tmp_file_enc, ios::openmode(ios::out | coder_base::binary));

		Coder alg(&in, &out);
		alg.encode();
	}

	{
		filebuf in;
		in.open(tmp_file_enc, ios::openmode(ios::in | coder_base::binary));
		filebuf out;
		out.open(tmp_file_dec, ios::openmode(ios::out | coder_base::binary));

		Coder alg(&in, &out);
		alg.decode();
	}

	string decompr;
	{
		filebuf in;
		in.open(tmp_file_dec, ios::openmode(ios::in | coder_base::binary));
		istream i(&in);
		decompr.read(i);
	}

	cout << "file_test: " << orig << "#" << decompr << "# equal:" << (decompr == orig) << endl;

	if (decompr == orig) {
		delete_file(tmp_file_orig); delete_file(tmp_file_enc); delete_file(tmp_file_dec);
	}
}

void stream_test(const char* tmp_file_name)
{
	string orig = "Hello";

	{
		encoding_ofstream<Coder> os(tmp_file_name);
		cout << "os failed? " << os.fail() << endl;
		os << orig << endl;
		cout << "os failed? " << os.fail() << endl;
	}

	string decompr;
	{
		decoding_ifstream<Coder> is(tmp_file_name);
		cout << "is failed? " << is.fail() << endl;
		is >> decompr;
		cout << "is failed? " << is.fail() << endl;
	}

	cout << "stream_test: " << orig << "#" << decompr << "#" << endl;

	if (decompr == orig) {
		delete_file(tmp_file_name);
	}
}

int main(int argc, char* argv[]) 
{
	if (argc < 4) file_test(tmp_file_name(), tmp_file_name(), tmp_file_name());
	else          file_test(argv[1], argv[2], argv[3]);
	
	if (argc < 2) stream_test(tmp_file_name());
	else          stream_test(argv[1]);

	return 0;
}
