/*******************************************************************************
+
+  LEDA 7.0
+
+
+  alphabet.cpp
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

#include<LEDA/string/alphabet.h>

LEDA_BEGIN_NAMESPACE

void alphabet::setStandard() {
	letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	finished = true;
}

bool alphabet::isStandard() const {
	if ((letters == "ABCDEFGHIJKLMNOPQRSTUVWXYZ") || finished)
		return true;
	else
		return false;
}

void alphabet::setDNA() {
	letters = "ACGT";
	finished = true;
}

bool alphabet::isDNA() const {
	if ((letters == "ACGT") || finished)
		return true;
	else
		return false;
}


int alphabet::getLength() const {
	if (!finished)
		LEDA_EXCEPTION(1,"Attempt to call getLength on an unfinished alphabet!");
	return letters.length();
}

int alphabet::getSize() const {
	if (!finished)
		LEDA_EXCEPTION(1,"Attempt to call getLength on an unfinished alphabet!");
	return letters.length();
}

void alphabet::addLetter(char ch) {
	if (finished)
		LEDA_EXCEPTION(1,"Attempt to add character to finished alphabet!");
	else
		if (letter_set.lookup(int(ch)) == nil)
			letter_set.insert(int(ch), ch);
}

void alphabet::delLetter(char ch){
	if (finished)
		LEDA_EXCEPTION(1,"Attempt to delete character from finished alphabet!");
	else
		if (letter_set.lookup(int(ch)) != nil)
			letter_set.del(int(ch));
}

char alphabet::getLetter(unsigned int i){
	if (int(i) >= letters.length())
		LEDA_EXCEPTION(1,"Index in getLetter too big!");
	else
		if (!finished)
			LEDA_EXCEPTION(1,"Attempt to call getLetter on an unfinished alphabet!");
		else
			return letters[int(i)];
	return 'x';
}


void alphabet::setFinished() {
	finished = true;
	string word;
	char ch;
	forall(ch, letter_set)
		word += ch;
	letters = word;
}

bool alphabet::getFinished() const {
	return finished;
}

bool alphabet::isFinished() const {
	return finished;
}

string alphabet::getLetters() {
		if (!finished)
			LEDA_EXCEPTION(1,"Attempt to call getLetters on an unfinished alphabet!");
		return letters;
}

LEDA_END_NAMESPACE

