#!/bin/sh

if [ "$1" = "" ]; then
  echo "usage: leda.make cc"
  exit
fi

cc=$1

sys=`leda.sys`
host=`hostname`


ledadir=$LEDA/LINK/$sys/$cc
logfile=$LEDA/make/$host.$sys.$cc


if [ ! -d $ledadir ]; then
   continue;
fi


if [ -h $ledadir ]; then
  cc=`readlink $ledadir`
fi


echo " "  > $logfile

echo $ledadir

cd $ledadir

cc_conf=$cc

version=""
flags=""
mode=shared

targets="lib apps"

if [ $cc = g++ -o $cc = g++-mt ]; then 
  targets="$targets demos tests res dvi"
fi

 
case $cc in

     g++-O) cc=g++-O
            cc_conf=$cc
            ;;

    g++-O2) cc=g++-O2
            cc_conf=$cc
            ;;

    g++-m32) cc=g++
            cc_conf=g++-m32;
            ;;

  g++*-mt-spl) version=`echo $cc | sed s/g++// | sed s/-mt-spl//`
            cc=g++-mt-spl
            cc_conf=$cc;
            ;;
  g++*-mt-mtx) version=`echo $cc | sed s/g++// | sed s/-mt-mtx//`
            cc=g++-mt-mtx
            cc_conf=$cc;
            ;;
  g++*-mt-std) version=`echo $cc | sed s/g++// | sed s/-mt-std//`
            cc=g++-mt-std
            cc_conf=$cc;
            ;;
  g++*-mt-sys) version=`echo $cc | sed s/g++// | sed s/-mt-sys//`
            cc=g++-mt-sys
            cc_conf=$cc;
            ;;
   g++*-mt) version=`echo $cc | sed s/g++// | sed s/-mt//`
            cc=g++-mt
            cc_conf=$cc;
            ;;
g++*-mt-O2) version=`echo $cc | sed s/g++// | sed s/-mt-O2//`
            cc=g++-mt-O2
            cc_conf=$cc;
            ;;

 g++-geode) cc=g++
            cc_conf=g++-geode
            mode=static;
            ;;

 g++-debug) version=""
            cc_conf=$cc;
            cc=g++
            ;;

     g++-*) version=`echo $cc | sed s/g++-//`
            cc=g++
            cc_conf=$cc;
            ;;


      g++*) version=`echo $cc | sed s/g++//`
            cc=g++
            cc_conf=$cc;
            ;;

    CC*-mt) version=`echo $cc | sed s/CC// | sed s/-mt//`
            cc=CC-mt
            cc_conf=$cc;
            ;;
       CC*) version=`echo $cc | sed s/CC//`
            cc=CC
            cc_conf=$cc;
            ;;
esac



if [ $cc = g++ -a $sys = win32 ]; then
  mode=static
fi

if [ $cc = msc ]; then
  cc_conf=msc
  mode=shared
fi

if [ $cc = msc-s ]; then
  cc_conf=msc
  mode=static
fi

if [ $cc = msc-mt ]; then
  cc_conf=msc-mt
  mode=shared
fi

if [ $cc = msc-mts ]; then
  cc_conf=msc-mt
  mode=static
fi


if [ $cc = dmc ]; then
  mode=static
fi

if [ $cc = xlC ]; then
  mode=static
fi

if [ $cc = gdb ]; then
  cc_conf=gdb
  mode=static
  flags="CFLAGS=-g"
fi


#touch .license

echo "lconfig $cc_conf $mode $version" >> $logfile
echo " " >> $logfile

/bin/sh lconfig $cc_conf $mode $version >> $logfile 2>&1

rm -f *.tag

make -i $flags $targets >> $logfile 2>&1

#if [ ! -h $ledadir ]; then 
#  make -i install >> $logfile 2>&1
#fi


