#include <LEDA/plane_alg.h>
#include <LEDA/map.h>
#include <LEDA/window.h>

typedef rat_point POINT;
typedef rat_circle CIRCLE;


void segment_points(segment s, int n, list<POINT>& pl, map<POINT,segment>& D)
{ 
  integer x  = n*s.xcoord1();
  integer y  = n*s.ycoord1();
  integer dx = s.xcoord2() - s.xcoord1();
  integer dy = s.ycoord2() - s.ycoord1();

  for(int i=0; i <= n; i++)
  { POINT p(x,y,n);
    pl.append(p);
    D[p] = s; 
    x += dx;
    y += dy;
   }
}


int main()
{
  int n = 2;

  window W;

  W.init(0,1000,0);
  W.set_node_width(3);


  list<segment> sl;
  list<POINT>   pl;
  map<POINT,segment> D;

  segment s;
  while ( W >> s ) 
  { W << s;
    sl.append(s);
    segment_points(s,n,pl,D);

    GRAPH<POINT,int> G;
    DELAUNAY_TRIANG(pl,G);


    W.clear();

    node v;
    forall_nodes(v,G) 
      W.draw_filled_node(G[v].to_point(),blue);

    edge e;
    forall_edges(e,G) 
    { POINT p = G[source(e)];
      POINT q = G[target(e)];
      if (D[p] != D[q])
       { W.set_line_style(dashed);
         W.draw_segment(p.to_point(), q.to_point(),blue);
         W.set_line_style(solid);
        }
      else
        W.draw_segment(p.to_point(), q.to_point(),black);
     }

    GRAPH<CIRCLE,POINT> VD;
    DELAUNAY_TO_VORONOI(G,VD);

    forall_nodes(v,VD) 
    {  POINT p1 = VD[v].point1();
       POINT p2 = VD[v].point2();
       POINT p3 = VD[v].point3();
       if (VD[v].orientation() != 0)
       {  POINT c = VD[v].center();
          if (D[p1] != D[p2] && D[p1] != D[p3] && D[p2] != D[p3])
           { W.draw_filled_node(c.to_point(),red);
             W.draw_circle(VD[v].to_circle(),red);
            }
          else
           { W.draw_filled_node(c.to_point(),green);
             W.draw_circle(VD[v].to_circle(),green);
            }
        }
     }

  }

  return 0;
}
