// Programm zur Simulation eines endlichen Automaten
// Sascha Hunold
// Beginn: 9.6.1999
// Letze Aenderung: 4.7.1999


#include <iostream>

#include <LEDA/core/string.h>
#include <LEDA/core/list.h>
#include <LEDA/graphics/window.h>
#include <LEDA/graphics/graphwin.h>
#include <LEDA/graph/graph.h>
#include <LEDA/graphics/pixmaps/button32.h>

#include "xpm/start.xpm"
#include "xpm/final.xpm"
#include "xpm/input.xpm"
#include "xpm/about.xpm"
#include "xpm/next.xpm"
#include "xpm/prev.xpm"

// Klasse zum Speichern von Pfaden zu Worttransitionen
class Path {
public:
  list<edge> Path_Edges;  // Liste, die den Pfad zur Wortransition angibt
  int Steps;              // Schritte, die benoetigt werden, um die Worttransition zu ueberlaufen
  string Trans;           // eigentliche Worttransition

  friend __exportF ostream& operator<<(ostream& O, const Path& p) {};
  friend __exportF istream& operator>>(istream& I, Path& p) {};
};

// Klasse fuer die Undo-Funktion
class Tmp_Graph {
public:
  list<node> Graph_Nodes;
  list<edge> Graph_Edges;
  int undo_step;

  friend __exportF ostream& operator<<(ostream& O, const Tmp_Graph& TG) {};
  friend __exportF istream& operator>>(istream& I, Tmp_Graph& TG) {};
};

// definiere Epsilon als Sternchen
#define EPSILON "*"

// setze noch ein paar Farbdefinitionen
#define DEFAULT_NODE_COLOR grey2
#define DEFAULT_EDGE_COLOR black
#define FINAL_COLOR yellow
#define START_COLOR white
#define ANI_COLOR1 red
#define ANI_COLOR2 blue
#define ACCEPT_COLOR green
#define DONT_ACCEPT_COLOR red

// verwendete Funktionen
void redraw(void);
void stimuli(GraphWin& gw);
void start(GraphWin& gw);
void input(GraphWin& gw);
void final(GraphWin& gw);
void about(GraphWin& gw);
void help();
void settings();
void delay_panel();
void redrawFinals(void);
bool list_contains_string(string unknown_member, list<string> search_list);
bool is_prefix( string unknown_prefix, string search_string);
void dfs_eps( list<node> *Start_Nodes, list<node> *Draw_Nodes, list<edge> *Draw_Edges, node Node_To_Search_From, list<node> *Nodes_To_Draw, list<edge> *Edges_To_Draw, string actLetter, list<Path> *Trans_Paths, int Level );
string colorize_at(int pos);
void draw_nodes_and_edges( list<node> *Draw_Nodes, list<edge> *Draw_Edges );
bool test_states(GraphWin& gw);
void previous(GraphWin& gw);
void next_g(GraphWin& gw);
void redrawGraph(Tmp_Graph *act_Graph, color col);

GRAPH <string, string> EAgraph;   // gerichteter Graph, Kanten und Knoten mit Strings beschriftet
GraphWin gw(EAgraph,"EAsim : Simulation eines endlichen Automaten");  // ein Graph-Fenster dem unser Graph und unser Fenster uebergeben wird
node SNode;                       // Startzustand
string GraphInput = "";           // Eingabewort
list<node> FNodes;                // Finalzustaende
int delay=1;                      // Simulationsverzoegerung
int node_width=24;                // Groesse der Knoten
int edge_width=1;                 // Breite der Kanten
int act_Step=0;                   // Simulationschritt
int Step_Num=0;                   // wieviele Undo-Steps gibts denn
list<Tmp_Graph> Undo_Graphs;      // Liste, die sich die Simulationsschritte merkt

bool test_states(GraphWin& gw) {
  list<node> act_graph_nodes;
  node loopNode;

  act_graph_nodes = EAgraph.all_nodes();

  // teste ob aktuell gewaehlter Startknoten im Graph ist
  if( SNode == nil ) return false;
  forall( loopNode, act_graph_nodes ) {
    if( SNode == loopNode ) {
      return true;
    }
  }
  return false;
}

string colorize_at(int position){
  string retstring = "";
  retstring += "\\black " + GraphInput.head(position-1) + "\\red " + GraphInput(position-1,position-1)
    + "\\black " + GraphInput.tail(GraphInput.length()-position);
  return retstring;
}

bool list_contains_string( string unknown, list<string> to_search ) {
  string helpString;
  forall( helpString, to_search ) {
    if( helpString == unknown ) return true;
  }
  return false;
}

bool is_prefix( string unknown, string search ) {
  if( unknown == search.head(1) ) { return true; }
  else { return false; }
}

void draw_nodes_and_edges( list<node> *Draw_Nodes, list<edge> *Draw_Edges ) {
  edge draw_Edge;

    // faerbe alle neu erreichten Zustaende und die Kanten ueber die ich gelaufen
    // bin
    forall( draw_Edge, *Draw_Edges ){
      gw.set_color( EAgraph.target(draw_Edge), ANI_COLOR1 );
      gw.set_color( draw_Edge, ANI_COLOR1 );
      
    }
    leda_wait(((float)delay)/2);
    
    // faerbe sie um, damit ich Animationsschritte besser sehe
    forall( draw_Edge, *Draw_Edges ){
      gw.set_color( EAgraph.target(draw_Edge), ANI_COLOR2 );
      gw.set_color( draw_Edge, ANI_COLOR2 );
    }
    leda_wait(0.5);
    (*Draw_Edges).clear();
    (*Draw_Nodes).clear();
}


void start(GraphWin& gw) {
  list<node> tNode;                 // temp. Zustaende
  tNode = gw.get_selected_nodes();
  if(tNode.length() > 1) {
    gw.message( string("\\bf\\blue only one\\black node has to be selected") );
    return;
  }
  if(tNode.length() < 1) {
    gw.message( string("\\bf\\red select 1\\black node") );
    return;
  }
  SNode = tNode.front();
  gw.set_color( SNode, START_COLOR );
  gw.message( string("\\bf start-node saved") );
  gw.deselect_all_nodes();
}

void input(GraphWin& gw) {
  GraphInput = gw.get_window().read_string("Input");
  gw.message(string("input was \\bf\\blue "+GraphInput));
}

void dfs_eps( list<node> *NodesToDo, list<node> *Draw_Nodes, list<edge> *Draw_Edges, node Node_To_Search_From, list<node> *Maybe_Nodes_To_Draw, list<edge> *Maybe_Edges_To_Draw, string actLetter, list<Path> *Trans_Paths, int Level) {
  node act_Node, loopNode;       // brauche Knoten fuer forall
  edge act_Edge, loopEdge;       // dasselbe fuer Kanten
  string act_String;

  forall_adj_edges( act_Edge, Node_To_Search_From ) {

      // wenn epsilon-Transition gefunden -> gehe zu diesen Knoten und suche SOFORT weiter
      if( (string)EAgraph.inf(act_Edge) == EPSILON ) {
	(*Maybe_Nodes_To_Draw).push( EAgraph.target(act_Edge) );
	(*Maybe_Edges_To_Draw).push( act_Edge );
	dfs_eps( NodesToDo, Draw_Nodes, Draw_Edges, EAgraph.target(act_Edge), Maybe_Nodes_To_Draw, Maybe_Edges_To_Draw, actLetter, Trans_Paths, Level );
	continue;
      }

      // wenn aktueller Buchstabe an Kante gefunden -> nimm Knoten in die NodesToDo-Liste auf
      // jetzt weiss ich, dass ich diesen Pfad zeichnen kann
      if( (string)EAgraph.inf(act_Edge) == actLetter ) {
	(*Maybe_Nodes_To_Draw).push( EAgraph.target(act_Edge) );	// Zielknoten zeichnen
	(*Maybe_Edges_To_Draw).push( act_Edge ); 	// mit actLetter beschriftete Kante zeichnen
	(*NodesToDo).push( EAgraph.target(act_Edge) ); // merke mir noch den TargetKnoten als neuen Startknoten
	forall( loopNode, *Maybe_Nodes_To_Draw ) { 
	  (*Draw_Nodes).push(loopNode);
	}
	forall( loopEdge, *Maybe_Edges_To_Draw ) {
	  (*Draw_Edges).push(loopEdge);
	}
	continue;
      }

      // suche nach Worttransitionen
      if( ((string)EAgraph.inf(act_Edge)).length() > 1 ) {
	Path new_Path;
	if( is_prefix( actLetter, (string)EAgraph.inf(act_Edge) ) ) { 
	  forall( loopEdge, *Maybe_Edges_To_Draw ) {
	    new_Path.Path_Edges.push(loopEdge);
	  }
	  new_Path.Path_Edges.push(act_Edge);
	  new_Path.Steps = Level +  ((string)EAgraph.inf(act_Edge)).length();
	  new_Path.Trans = (string)EAgraph.inf(act_Edge);	  
	  (*Trans_Paths).push(new_Path);
	}
      }

  }

  (*Maybe_Nodes_To_Draw).clear();
  (*Maybe_Edges_To_Draw).clear();
  
  return;
}

void stimuli(GraphWin& gw) {

  list<node> NodesToDo; // Liste aller Knoten in denen ich gerade bin
  list<node> tmpNodes;  // merke mir Zustaende i die ich in Zustand i-1 finde
  list<edge> tmpEdges;  // Kanten die gezeichnet werden sollen
  string actLetter;     // aktueller Buchstabe im Eingabestring     
  node       myNode;    // aktuell angefasster Knoten
  edge       myEdge;    //   (dasselbe)        Kante
  node       testNode;  // tmp. Knoten zum Vergleich
  node_list  compList;  // Liste zum Vergleichen
  list<node> Draw_Nodes;  // Knoten, die gezeichnet werden sollen
  list<edge> Draw_Edges;  // Kanten, die gezeichnet werden sollen
  list<Path> Trans_Paths; // Liste mit Pfaden zu einer Worttransition
  Path       myPath;   

  act_Step = 0;
  Undo_Graphs.clear();

  if( SNode == nil ) {
    gw.message( string("\\bf\\blue no start-node selected"));
    return;
  }

  if( test_states(gw) == false ) {
    gw.message( string("\\bf\\blue no start- or final-node saved"));
    return;
  }

  NodesToDo.push(SNode); // Haenge Startknoten in die Liste

  delay_panel();

  redraw();

  gw.zoom_graph();

  if( SNode == nil ) {
    gw.message( string("\\bf\blue no\\black start-node selected") );
    return;
  }

  if( GraphInput == "" ) {
    gw.message( string("\\bf\\blue no\\black input made") );
    return;
  }

  for(int i=0; i<GraphInput.length(); i++) {

    act_Step++;
    
    actLetter = (string)GraphInput[i];
    
    gw.message(string("\\blue simulate with input \\bf "+colorize_at(i+1)));

    // schmeisse Transitions-Pfade raus, die nicht mehr erreicht werden koennen
    list<Path> new_Trans_Paths;
    forall( myPath, Trans_Paths ) {
      if( actLetter == (string)myPath.Trans( i-(myPath.Steps-myPath.Trans.length()), i-(myPath.Steps-myPath.Trans.length()) ) )  {
	new_Trans_Paths.push( myPath );
      }
    }
    Trans_Paths.clear();
    Trans_Paths.swap( new_Trans_Paths );
    new_Trans_Paths.clear();

    // suche fuer alle Knoten aus NodesToDo nach eps.-Transitionen und actLetter und Worttrans.
    forall( myNode, NodesToDo ) {
      list<edge> Maybe_Drawn_Edges;
      list<node> Maybe_Drawn_Nodes;

      if( myNode == nil ) {
	gw.message( string("\\bf\\black error: Nullpointer Exception") );
	return;
      }

      // nehme Zustand aus Schlange und ueberpruefe, ob die ausgehenden Kanten mit 
      // dem entsprechenden Buchstaben versehen sind
      forall_adj_edges( myEdge, myNode ) {
	// wenn Buchstaben gleich sind, nimm Zustand und Kante in Listen auf
	if( (string)EAgraph.inf(myEdge) == actLetter ) {
	  tmpNodes.push( EAgraph.target(myEdge) );
	  tmpEdges.push( myEdge );
	  Draw_Nodes.push( EAgraph.target(myEdge) );
	  Draw_Edges.push( myEdge );
	  continue;
	}

	// suche nach Worttransitionen
	if( ((string)EAgraph.inf(myEdge)).length() > 1 ) {
	  Path new_Path;
	  // nehme nur Kanten in die Liste auf, die als Prfix den aktuellen Buchstaben haben
	  if( is_prefix( actLetter, (string)EAgraph.inf(myEdge) ) ) { 
	    new_Path.Path_Edges.push(myEdge);
	    new_Path.Steps = i +  ((string)EAgraph.inf(myEdge)).length();
	    new_Path.Trans = (string)EAgraph.inf(myEdge);	  
	    Trans_Paths.push(new_Path);
	  }
	  continue;
	}

	// wenn Epsilon-Transition gefunden wurde -> starte DFS_EPS und suche evtl. vorhandene Kanten
	// mit dem aktuellen Buchstaben oder mit Worttransitionen
	if( (string)EAgraph.inf(myEdge) == EPSILON ) {
	  Maybe_Drawn_Nodes.push( EAgraph.target(myEdge) );
	  Maybe_Drawn_Edges.push( myEdge );
	  // suche in die Tiefe nach eps.-Trans., wenn ich passende mit anschliessendem actLetter gefunden
	  // habe, male diese und gehe in entsprechenden Zustand ueber
	  dfs_eps( &tmpNodes, &Draw_Nodes, &Draw_Edges, EAgraph.target(myEdge), &Maybe_Drawn_Nodes, &Maybe_Drawn_Edges, actLetter, &Trans_Paths, i );
	}
      }
    }

    // schaue mir alle Knoten an, die in der Transitionsliste stehen
    // sind noch Kanten dabei sein, deren Steps-Wert der aktuellen Tiefe im Automaten ist, sollten
    // diese gezeichnet und danach weggeworfen werden
    forall( myPath, Trans_Paths ) {
      myEdge;
      if( myPath.Steps == i+1 ) {
	forall( myEdge, myPath.Path_Edges ) {
	  Draw_Edges.push( myEdge );
	  Draw_Nodes.push( EAgraph.target( myEdge ) );
	  // Knoten der nach der Transition kommt, muss in die tmpNodes-Liste (also zu denen, 
	  // die dann Startkonten sind)  aufgenommen werden
	  if( ((string)EAgraph.inf(myEdge)).length() > 1 ) {
	    tmpNodes.push( EAgraph.target(myEdge) );
	  }
	}
      }
      // schmeisse abgearbeitete weg und hebe nur aktuelle in new_Trans_Paths auf
      else {
	new_Trans_Paths.push( myPath );
      }
    }
    // noch Listen swapen, um sie zu loeschen
    Trans_Paths.clear();
    Trans_Paths.swap( new_Trans_Paths );
    new_Trans_Paths.clear();

    // vor dem Zeichnen, noch kurz den Graph zum spaeteren Undo gemerkt
    Tmp_Graph act_Graph;
    forall( myNode, Draw_Nodes ) {
      act_Graph.Graph_Nodes.push( myNode );
    }
    forall( myEdge, Draw_Edges ) {
      act_Graph.Graph_Edges.push( myEdge );
    }
    act_Graph.undo_step = act_Step;
    Undo_Graphs.push(act_Graph);

    // dann zeichnen
    draw_nodes_and_edges( &Draw_Nodes, &Draw_Edges );
    tmpEdges.clear();
    
    // von allen neu erreichten Knoten suche ich jetzt mit dem naechsten Buchstaben
    // weiter
    NodesToDo.clear();
    forall( myNode, tmpNodes ) {
      NodesToDo.push( tmpNodes.pop() );
    }

    // wenn ich beim letzten Buchstabe ankomme, schaue ich nach in welchen Zustaenden
    // ich gelandet bin, wenn einer dieser Zustaende ein Finalzustand ist, dann markiere ihn
    // entsprechend (gruen), ansonsten faerbe ihn anders (rot)
    if( i == GraphInput.length()-1 ) {
      forall( myNode, FNodes ) {
	compList.push(myNode);
      }
      forall( myNode, NodesToDo ) {
	if( compList.member(myNode) == true ) {
	  gw.set_color( myNode, ACCEPT_COLOR );
	}
	else {
	  gw.set_color( myNode, DONT_ACCEPT_COLOR );
	}
      }
    }
    else {
      tmpNodes.clear();
      redrawFinals();
    }

  }

  Step_Num = act_Step;

  gw.message( string("\\bf\\blue simulation finished") );
}

void redraw() {
  list<node> allNodes;
  list<edge> allEdges;
  node actNode;
  edge actEdge;

  allNodes = EAgraph.all_nodes();
  allEdges = EAgraph.all_edges();

  forall( actNode, allNodes ) {
    gw.set_color( actNode, DEFAULT_NODE_COLOR );
  }
  forall( actEdge, allEdges ) {
    gw.set_color( actEdge, DEFAULT_EDGE_COLOR );
  }
  redrawFinals();
}

// zeichnet Finalzustaende in entsprechender Farbe neu
void redrawFinals() {
  node tNode;
  forall(tNode, FNodes) {
    gw.set_color( tNode, FINAL_COLOR );
  }
}

void final(GraphWin& gw) {
  node tNode;
  gw.message( string("\\bf final-nodes saved"));
  FNodes = gw.get_selected_nodes();
  gw.deselect_all_nodes();
  redrawFinals();
}

void exit(GraphWin& gw) {
  exit(0);
}

void delay_panel() {
  panel P("choose delaytime for simulation");
  P.text_item( string("\\bf choose delaytime for simulation\\c8") );
  P.int_item( "Delay", delay, 1, 10, 1, "choose delay of simulation");
  P.button("ok");
  P.open(gw.get_window());
}

void settings() {
  panel P("EAsim settings");
  P.int_item( "node-width", node_width, 20, 30);
  P.int_item( "edge-width", edge_width, 1, 3, 1);
  P.button("continue");
  P.open(gw.get_window());
}

void help() {
  panel P("EAsim");
  P.text_item( "\\bf\\blue EAsim: a program to simulate an automaton\\c8 \
  \
  \\black\\rm
  At first you have to edit a graph of your own imagination. \
  Note that you can put a letter or a string on an edge by \
  doublepressing the mouse-button. Then you can set your prefered \
  characters in the \"data(string)\" field. \
  You have to select an start-node for simulation and press \
  the first button to save the settings. Then do the same for final-nodes,\
  but press the second button to save them. Then it's time to choose a
  word from the alphabet you used in the graph. Now you can run the \
  simulation with pressing the \"flag\"-button.
  ");

  P.button("continue");
  P.open(gw.get_window());
}

void about(GraphWin& gw)
{ 
  panel P("About EAsim");
  P.text_item( "\\bf\\red EAsim\\c8 \\blue written by S. Hunold\\c8
                \\blue hunold@informatik.uni-halle.de \\c8
                \\rm\\black 1999\\c8
                \
                \\rm\\black some ideas: T. Lange \\c4
                ");
  P.button("ok");
  P.open(gw.get_window());
}

void redrawGraph(Tmp_Graph *act_Graph, color col) {
  node draw_node;
  edge draw_edge;

  forall( draw_node, (*act_Graph).Graph_Nodes ) {
    gw.set_color( draw_node, col);
  }
  forall( draw_edge, (*act_Graph).Graph_Edges ) {
    gw.set_color( draw_edge, col);
  }

}

void previous(GraphWin& gw) {
  Tmp_Graph myGraph;

  act_Step--;

  if( act_Step < 1 ) act_Step = Step_Num;

  redraw();

  gw.message(string("\\blue current inputposition:\\bf "+colorize_at(act_Step)));

  forall( myGraph, Undo_Graphs ) {
    if( myGraph.undo_step < act_Step ) {
      redrawGraph(&myGraph, ANI_COLOR2);
    }
  }
  forall( myGraph, Undo_Graphs ) { 
    if( myGraph.undo_step == act_Step ) {
      redrawGraph(&myGraph, ANI_COLOR1);
    }
  }
}

void next_g(GraphWin& gw) {
  Tmp_Graph myGraph;

  act_Step++;

  if( act_Step > Step_Num ) act_Step = 1;

  redraw();

  gw.message(string("\\blue current inputposition:\\bf "+colorize_at(act_Step)));

  forall( myGraph, Undo_Graphs ) {
    if( myGraph.undo_step < act_Step ) {
      redrawGraph(&myGraph, ANI_COLOR2);
    }
  }
  forall( myGraph, Undo_Graphs ) {
    if( myGraph.undo_step == act_Step ) {
      redrawGraph(&myGraph, ANI_COLOR1);
    }
  }
}


int main() {

  gw.add_simple_call(start, "choose start-state", 0, gw.get_window().create_pixrect(start_xpm) );
  gw.add_simple_call(final, "choose final-states", 0, gw.get_window().create_pixrect(final_xpm) );
  gw.add_simple_call(input, "gimme a word", 0, gw.get_window().create_pixrect(input_xpm) );
  gw.add_simple_call(stimuli, "simulation", 0, gw.get_window().create_pixrect(flag_xpm) );
  gw.add_simple_call(previous, "previous step", 0, gw.get_window().create_pixrect(prev_xpm) );
  gw.add_simple_call(next_g, "next step", 0, gw.get_window().create_pixrect(next_xpm) );
  gw.add_simple_call(about, "about", 0, gw.get_window().create_pixrect(about_xpm) );
  gw.add_simple_call(exit, "exit", 0, gw.get_window().create_pixrect(exit_xpm) );

  gw.set_edge_label_type(data_label);  // bewirkt, dass bei Doppelklick auf eine Kante, das entsprechende Label aus data genommen wird, da wir data auch zum Auslesen verwenden

  gw.clear_graph ();
  gw.display(window::center,window::center);

  help();

  settings();

  gw.set_node_shape (circle_node);
  gw.set_node_width (node_width);
  gw.set_node_height (node_width);
  gw.set_edge_width (edge_width);

  gw.edit ();

  return 0;

} 

