#define LEDA_CHECKING_OFF

#include <LEDA/graph.h>
#include <LEDA/s1_graph.h>
#include <LEDA/dimacs_graph.h>
#include <LEDA/assert.h>

#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>

#include <LEDA/templates/max_flow_s6.t>


using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif



struct flow_node : public data_slots<3>
{ int excess;
  int dist;
};

struct flow_edge : public data_slots<2>
{ int cap;
  int flow;
};
  
  

int main(int argc, char** argv)
{
  typedef s_graph<opposite_graph,flow_node,flow_edge > static_graph;

  typedef max_flow<int,static_graph>  max_flow_alg;

  static_graph G;
  static_graph::node s=0;
  static_graph::node t=0;

  cout << sizeof(static_graph::node_t) << endl;
  cout << sizeof(static_graph::edge_t) << endl;

  edge_slot<int,0,static_graph>   cap(G);
  edge_slot<int,1,static_graph>  flow(G);


  cout << endl;
  cout << "reading DIMACS maxflow problem from std input ... ";
  cout << flush;

  read_dimacs_mf(cin,G,s,t,cap);
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;

  static_graph::node v;
  forall_nodes(v,G) v->excess = 0;



  cout << "MAX_FLOW    " << flush;

  max_flow_alg  mf;

  if (argc >= 2)  
     mf.set_heuristic(atof(argv[1]));

  mf.run(G,s,t,cap,flow);

  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec  f = %d", mf.cpu_time(), mf.flow()) << endl;

  mf.statistics(cout);

/*
  mf.check();
*/

//mf.print();

  return 0;
}




