#define LEDA_CHECKING_OFF

#include <LEDA/graph.h>
#include <LEDA/s_graph.h>
#include <LEDA/dimacs_graph.h>
#include <LEDA/assert.h>

#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>

#include <LEDA/templates/max_flow_s6.t>



using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif

int main(int argc, char** argv)
{
  const int ns = 3; //max_flow_alg::node_slots;
  const int es = 0; //max_flow_alg::edge_slots;

  typedef s_graph<opposite_graph_traits<ns,es+2> > static_graph;

  typedef max_flow<int,static_graph>  max_flow_alg;

  static_graph G;
  static_graph::node s=0;
  static_graph::node t=0;

  edge_slot<int,es,  static_graph>    cap;
  edge_slot<int,es+1,static_graph>  flow;

  cout << sizeof(static_graph::node_type) << endl;
  cout << sizeof(static_graph::edge_type) << endl;


  cout << endl;
  cout << "reading DIMACS maxflow problem from std input ... ";
  cout << flush;

  read_dimacs_mf(cin,G,s,t,cap);
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;


  cout << "MAX_FLOW    " << flush;

  max_flow_alg  mf;

  if (argc >= 2)  
     mf.set_heuristic(atof(argv[1]));

  mf.run(G,s,t,cap,flow);

  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec  f = %d", mf.cpu_time(), mf.flow()) << endl;

  mf.statistics(cout);
/*
  mf.check();
*/

  //mf.print();

  return 0;
}




