#include <LEDA/window.h>
#include <LEDA/file.h>

#if defined(__win32__)
const char* exe_suffix = ".exe";
#else
const char* exe_suffix = "";
#endif


static string leda_root;

static string demo_cmd[256];
static string src_file[256];



static string basename(string path, string& fname)
{ int len = path.length();
  int n = len - 1;
  while (path[n] != '/' && path[n] != '\\') n--;
  fname = path(n+1,len-1);
  return path(0,n-1);
}


static void show_text(int i)
{ //set_label("Starting File Viewer"); 
  string fname;
  string dname = basename(src_file[i],fname); 
  string cmd = leda_root + "/demo/xlman/fileview " + fname + " " 
                                                   + dname + " '*.c'";
  //cerr << cmd << endl;
  create_process(cmd);
  //set_label(def_label); 
 }


static void start_demo(int i) 
{ string cmd;
  string dname = basename(demo_cmd[i],cmd);
  //set_label("Running  " + cmd); 
  cerr << demo_cmd[i] << endl;
  create_process(demo_cmd[i]);
  leda_wait(3);
  //set_label(def_label); 
}

static void start_ascii(int i) 
{ string rcmd = leda_root + "/Manual/cmd/runascii";
  string dcmd = demo_cmd[i];
#if defined(__unix__)
  string cmd = "xterm -sb -sl 500 -T " + dcmd + " -e " + rcmd + " " + dcmd;
#else
  string cmd = rcmd + ".bat " + dcmd;
#endif
  cerr << dcmd << endl;
  //set_label("Running  " + dcmd);
  create_process(cmd);
  leda_wait(3);
  //set_label(def_label); 
}



void build_menu(string demo_dir, menu& demo_menu, menu& src_menu, 
                                                   int& i, int& j,
                                                   void (*start)(int))
{
  list<string> dir_list;

  if (is_directory(demo_dir))
     dir_list  = get_directories(demo_dir);
  dir_list.sort();

  string dir;
  forall(dir,dir_list)
  { if (dir[0] == '.' || dir == "xlman") continue;

    menu* pm = new menu;
    menu* sm = new menu;

    list<string> src_files = get_files(demo_dir+"/"+dir,"*.c");
    src_files.sort();
    int count = 0;
    string s;
    forall(s,src_files) 
    { string cmd = s.replace(".c","");
      if (!is_file(demo_dir+"/"+dir+"/"+ cmd + exe_suffix)) continue;
      sm->button(s,i,show_text);
      src_file[i++] = demo_dir + "/" + dir + "/" + s;
      pm->button(cmd,j,start);
      demo_cmd[j++] = demo_dir + "/" + dir + "/" + cmd;
      count++;
     }

    if (count > 0)
      { demo_menu.button(dir,*pm);
        src_menu.button(dir,*sm);
       }
    else
      { delete pm;
        delete sm;
       }
   }
}




#if defined(WINMAIN)

int main() {
  int    argc;
  char** argv;
  get_command_line(argc,argv);

#else

int main(int argc, char** argv) {

#endif

 if (!get_environment("LEDAROOT",leda_root)) {
   error_handler(1,"xlman: LEDAROOT (environment variable) not defined.");
   exit(1);
 }

 if (!is_directory(leda_root)) {
   string msg = "xlman: directory " + leda_root + " (LEDAROOT) does not exist.";
   error_handler(1,msg);
   exit(1);
 }


 string demo_dir = leda_root + "/demo";
 string demo_cout_dir = leda_root + "/demo_cout";


  int i=0;
  int j=0;

  menu demo_menu;
  menu src_menu;

  build_menu(demo_dir,demo_menu,src_menu,i,j,start_demo);

  demo_menu.separator();
  src_menu.separator();

  build_menu(demo_cout_dir,demo_menu,src_menu,i,j,start_ascii);

  panel main_panel;
  main_panel.text_item("LEDA Version 4.0");
  main_panel.button("demos",  4, demo_menu);
  main_panel.button("sources",5, src_menu);
  main_panel.open();

  return 0;
}
