#include<LEDA/window.h>
#include<LEDA/file_panel.h>

static int* matrix = 0;
static int width = 0;
static int height = 0;
static string fname = "none";
static string dname = ".";

static bool show_grid = true;
static int anim_steps = 16;

static window W(500,500,"LEDA XPM-Editor");

void draw_matrix(window& W, int* mat, int w, int h)
{ 
  W.start_buffering();
  W.clear();
  int pix_d = W.real_to_pix(1);
  double d = 0.5;
  for(int x = 0; x < w; x++)
    for(int y = 0; y < h; y++)
    { int c = *(mat + (h-1-y)*w + x);
      if (pix_d < 2)
        W.draw_pixel(x,y,color(c));
      else
        W.draw_box(x-d,y-d,x+d,y+d, color(c));
     }

  if (show_grid && pix_d > 3)
  { for(int x = 0; x <= w; x++) W.draw_segment(x-0.5,-0.5,x-0.5,h-0.5,black);
    for(int y = 0; y <= h; y++) W.draw_segment(-0.5,y-0.5,w-0.5,y-0.5,black);
   }

  W.stop_buffering();
  W.flush_buffer();
}


void redraw(window* wp, double x0, double y0, double x1, double y1)
{ window& W = *wp;
  if (x0 == W.xmin() && y0 == W.ymin() && x1 == W.xmax() && y1 == W.ymax())
     draw_matrix(W,matrix,width,height);
  else
     W.flush_buffer(x0,y0,x1,y1);
}



void read_file(string fname)
{ char* pmap = W.create_pixrect(fname);
  width = W.get_width(pmap);
  height = W.get_height(pmap);
  if (matrix) delete[] matrix;
  matrix = new int[width*height];
  W.pixrect_to_matrix(pmap,matrix);
  draw_matrix(W,matrix,width,height);
}

void write_file(string fname) 
{ error_handler(0,"write_file not implemented.");}


void file_handler(int what)
{ file_panel FP(W,fname, dname);
  switch (what) {
  case 0: exit(0);
          break;
  case 1: FP.set_load_handler(read_file);
          break;
  case 2: FP.set_save_handler(write_file);
          break;
  }
  FP.set_load_handler(read_file);
  FP.set_pattern("XPM Files (*.xpm)","*.xpm");
  FP.open();
}



void zoom_handler(int what)
{ 
  switch (what) {
  case 1: W.zoom(2.0,anim_steps);
          break;
  case 2: W.zoom(0.5,anim_steps);
          break;
  case 3: W.zoom_area(0,0,width,height,anim_steps);
          break;
  }
}


void option_handler(int)
{ panel P;
  P.text_item("\\bf\\blue Options");
  P.text_item("");
  P.bool_item("show grid", show_grid);
  P.int_item("animation",anim_steps,0,16);
  P.open(W);
}



int main(int argc, char** argv)
{
  menu file_menu;
  file_menu.button("load",1,file_handler);
  file_menu.button("save",2,file_handler);
  file_menu.button("exit",0,file_handler);

  menu zoom_menu;
  zoom_menu.button("zoom up", 1, zoom_handler);
  zoom_menu.button("zoom down", 2, zoom_handler);
  zoom_menu.button("fill window", 3, zoom_handler);

  W.button("&File", file_menu);
  W.button("&Zoom", zoom_menu);
  W.button("&Options", option_handler);
  W.make_menu_bar();


  W.init(-10,120,-10);
  W.set_redraw(redraw);
  W.set_clear_on_resize(0);

  W.display();

  if (argc > 1) 
  { fname = argv[1];
    read_file(fname);
   }


  for(;;)
  {
    point p;
    int but = W.read_mouse(p);
 
    if (but != MOUSE_BUTTON(1)) continue;
 
    bool dblclick,drag;
    W.read_mouse(p,300,300,dblclick,drag);
  
    if (dblclick)
    { point a,b;
      W.read_zoom_rect(p,a,b);
      W.zoom_area(a,b);
      continue;
     }
  
     if (drag)
     { W.scroll_window(p);
       continue;
      }
   }

  return 0;
}

  
  
