#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/dimacs_graph.h>

#include "mcf_cap_scaling.t"


using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cin;
using std::cout;
using std::flush;
using std::endl;
#endif


int main(int argc, char** argv)
{

  //graph G(6,5);

  graph G(6,3);

  node_slot<int> supply(G);

  edge_slot<int> lcap(G);
  edge_slot<int> ucap(G);
  edge_slot<int> cost(G);
  edge_slot<int> flow(G);

  typedef edge_slot<int> cap_array;

  typedef node_slot<int>  pot_array;
  typedef node_slot<int>  excess_array;
  typedef node_slot<int>  dist_array;
  typedef node_slot<edge> pred_array;
  typedef node_slot<int>  mark_array;



  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);


/*
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;
*/

  mcf_cap_scaling<int> mcf;


  bool f = mcf.run(G,lcap,ucap,cost,supply,flow,4) ;

  edge e;
  double c = 0;
  forall_edges(e,G) c += cost[e]*flow[e];

  cout << string("time\t %.2f", mcf.cpu_time()) << endl;

  if (f)
    cout << string("result\t %.2f", c) << endl;
  else
    cout << string("result\t INFEASIBLE", c) << endl;
  return 0;
}

