#include <LEDA/graph/graph.h>
#include <LEDA/graph/dimacs.h>
#include <LEDA/system/stream.h>

using namespace leda;

using std::cout;
using std::cerr;
using std::cin;
using std::endl;
using std::ifstream;



int main(int argc, char** argv) 
{
  double f_cap = 1.0;
  double f_cost = 1.0;

  if (argc < 3) 
  { cerr << "usage:  mcf_scale {file} f_cap f_cost" << endl;
    return 1;
  }

  if (argc > 3) 
  { if (argc == 3)
      { f_cap  = string(argv[1]).atof(); 
        f_cost = string(argv[2]).atof(); 
       }
    else
      { f_cap  = string(argv[2]).atof(); 
        f_cost = string(argv[3]).atof(); 
       }
   }

  graph G;
  edge_array<int> lcap;
  edge_array<int> ucap;
  edge_array<int> cost;
  node_array<int> supply;

  if (argc > 3)
  { ifstream in(argv[1]);
    Read_Dimacs_MCF(in,G,supply,lcap,ucap,cost);
   }
  else
    Read_Dimacs_MCF(cin,G,supply,lcap,ucap,cost);

  node v;
  forall_nodes(v,G) supply[v] = int(f_cap * supply[v]);

  edge e;
  forall_edges(e,G) 
  { ucap[e] = int(f_cap * ucap[e]);
    lcap[e] = int(f_cap * lcap[e]);
    cost[e] = int(f_cost * cost[e]);
   }

  Write_Dimacs_MCF(cout,G,supply,lcap,ucap,cost);
    
 return 0;
}
