#define LEDA_CHECKING_OFF


#include <LEDA/graph/graph.h>
#include <LEDA/graph/dimacs_graph.h>

#include "incl/mcf_cost_scaling7.h"

using namespace leda;

using std::cin;
using std::cout;
using std::cerr;
using std::flush;
using std::endl;

int main(int argc, char** argv)
{

  typedef mcf_cost_scaling<int,double,double>::node_data nd_type;
  typedef mcf_cost_scaling<int,double,double>::edge_data ed_type;

  typedef node_array<nd_type> node_data_array;
  typedef edge_array<ed_type> edge_data_array;

  graph G;
  node_array<int>  supply(G);
  edge_array<int>  lcap(G);

  edge_array<int> ucap(G);
  edge_array<int> cost(G);

  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);

  edge_array<int> flow(G);

  double n = G.number_of_nodes();
  double m = G.number_of_edges();

  cout << endl;
  cout << string("%6.0f nodes",n) << endl;
  cout << string("%6.0f edges",m) << endl;
  cout << endl;

  mcf_cost_scaling<int,double,double, graph, node_data_array, 
                                             edge_data_array> mcf;

  bool do_check = false;
  bool dimacs_statistics = false;

  int i = 1;
  while (i < argc)
  { string s = argv[i];
    if (s[0] != '-') break;
    if (s == "-l") mcf.set_write_log(true);
    if (s == "-c") do_check = true;
    if (s == "-s") dimacs_statistics = true;
    i++;
   }

  if (i < argc) { 
    int x = string(argv[i++]).atoi(); 
    if (x != 0) mcf.set_scale_factor(x); 
   }

  if (i < argc) { 
    double x = string(argv[i++]).atof(); 
    if (x != 0) mcf.set_af_factor(x);
   }

  if (i < argc) { 
    double x = string(argv[i++]).atof(); 
    if (x != 0) mcf.set_af_update(x);
   }

  if (i < argc) { 
    double x = string(argv[i++]).atof(); 
    if (x != 0) mcf.set_eps_factor(x);
   }


  if (dimacs_statistics)
  { D.print_statistics(cout);
    cout << endl;
   }
  
  bool feasible = mcf.run(G,lcap,ucap,cost,supply,flow) ;

  double c = mcf.total_cost(G,cost,flow);
  if (feasible)
  { cout << string("time: %6.2f cost: %14.0f  a = %d  af = %.0f (%.0f)  rf = %d  rs = %d",
                                            mcf.cpu_time(), 
                                            c,
                                            mcf.scale_factor(),
                                            mcf.af(), 
                                            mcf.af_start(), 
                                            mcf.refines(),
                                            mcf.restarts()) <<endl;
     mcf.statistics(cout);


     if (do_check)
     { float T = used_time();
       string msg;
       bool b = mcf.check(G,lcap,ucap,cost,supply,flow,msg);
       cout << string("checking time: %.2f sec",used_time(T)) << endl;
       if (!b) 
       { cerr << "mcf::check failed: " + msg << endl;
         return 2;
        }
      }
     return 0;
   }

  cout << endl;
  cout << "Infeasible MCF-Problem." << endl;
  cout << endl;

  return 1;
}

