/*******************************************************************************
+
+               phase 1 parallel mit jeweils einer fifo_set für jeden Thread
+ 
*******************************************************************************/


#include <LEDA/graph/graph_alg.h>
#include <LEDA/graph/edge_array.h>
#include <LEDA/graph/graph.h>
#include <LEDA/core/queue.h>
#include <LEDA/core/string.h>
#include <LEDA/system/assert.h>
#include <LEDA/graph/dimacs_graph.h>
#include <LEDA/graph/max_flow.h>
#include <LEDA/graph/templates/max_flow.h> 
#include <LEDA/system/multi_thread.h>
#include <LEDA/system/thread.h>
#include <LEDA/system/timer.h>
#include <LEDA/core/b_queue.h>
#include <LEDA/core/array.h>
#include <LEDA/core/list.h>
#include <iostream>
#include <fstream>


using namespace leda;
using std::cout;
using std::cerr;
using std::cin;
using std::endl;


class fifo_set{

  list<node> L;

public:

  fifo_set(){}

  node del() { if (!L.empty()) return L.pop(); else return nil; }
 
  void insert(node v, int)  { L.append(v); }
  void insert0(node v, int) { L.append(v); }                            

  void split0(int& id, array<fifo_set>& F, node_array<int>& dist)
  { node v;
    forall(v,L) F[id].insert(v, dist[v]);
    L.clear();
   }

  void split(int& thr_num, array<fifo_set>& F, node_array<int>& dist)
  { int g = 0;
    node v;
    forall(v,L)
    { int number = g % thr_num;
      F[number].insert(v, dist[v]);
      g++;
     }
    L.clear();
  } 


  int size() { return L.size(); }

  bool empty() { return L.empty(); }

  void clear() { L.clear(); }

  ~fifo_set(){}
};



struct thread_data
{
  graph* G;
  node s;
  node t;

  edge_array<int>* cap;
  edge_array<int>* flow;
  node_array<int>* dist;
  node_array<int>* excess;
  //array<fifo_set>* liste;
  fifo_set* liste;

  int* num_pushes; 
  int* num_edge_inspections; 
  int* num_relabels; 
  int* num_global_relabels; 

  float h; 
  int limit_heur;

  b_queue<node>* Q;
  node_array<leda_mutex*>* lock;

  int id; 
  bool local_relabel;
  bool knappe_locks;
};


inline bool get_lock(node v, node_array<leda_mutex*>& lock)
{ return lock[v]->trylock() == 0; } 


bool get_locks(node v, node w, node_array<leda_mutex*>& lock)
{
  bool b(0);
  if (lock[v]->trylock()==0)
  { if (lock[w]->trylock()==0)      
      b = 1;
    else lock[v]->unlock();
        }
        return b;
} 

void compute_dist_t(const graph& G, node t, 
                    const edge_array<int>& flow, 
                    const edge_array<int>& cap, 
                    const node_array<int>& excess, 
                    node_array<int>& dist,
                    fifo_set& U, 
                    b_queue<node>& Q)
{ 
  int n = G.number_of_nodes();
  Q.append(t);

  dist.init(G,n);

  dist[t] = 0;  

  while ( !Q.empty() )
  { node v = Q.pop();
    int  d = dist[v] + 1;
    if ( d >= n) break;
    edge e;

    forall_out_edges(e,v)
    { if ( flow[e] == 0 ) continue;
      node u = target(e); 
      int& du = dist[u];
      if ( du >= n ) 
      { du = d;
        Q.append(u); 
        if ( excess[u] > 0 ) U.insert(u,d); 
       }
     }

    forall_in_edges(e,v)
    { if ( cap[e] == flow[e] ) continue;
      node u = source(e); 
      int& du = dist[u];
      if ( du >= n )  
      { du = d;  
        Q.append(u); 
        if (excess[u] > 0) U.insert(u,d); 
      }
    }
  }
}



void maxflow(const graph& G, node s, node t, 
             const edge_array<int>& cap, 
             edge_array<int>& flow, 
             node_array<int>& dist, 
             node_array<int>& excess,
             //array<fifo_set>& U, 
             fifo_set& U, 
             int& num_pushes, 
             int& num_edge_inspections,
             int& num_relabels, 
             int& num_global_relabels, 
             float h, 
             int limit_heur, 
             b_queue<node>& Q, 
             node_array<leda_mutex*>& lock, 
             int id, bool knappe_locks, bool local_relabel)
{   
  int n = G.number_of_nodes();  

  int local_num_pushes = 0;
  int local_num_relabels = 0;
  int local_num_inspections = 0;

  for(;;) 
  {
    //node v = U[id].del();
    node v = U.del();

    if (v == nil) break;

    if (dist[v] >= n) continue;

    if (v == t) continue;

    if (excess[v] <= 0) continue;

    if(knappe_locks==0)
    { 
      if (get_lock(v,lock)==0) 
      { //U[id].insert(v, dist[v]);  // wenn Knoten v gelockt, 
        U.insert(v, dist[v]);  
                                   // fuege v in die Liste ein 
                                   // und betrachte ihn später
        continue;
       }

     int ev   = excess[v];
     int dv   = dist[v];   
     int dmin = MAXINT;

     edge e; 

     if ( ev > 0 )
     { 
       forall_out_edges(e,v)
       { //Fetch_And_Increment(&num_edge_inspections);
         local_num_inspections++;
         int& fe = flow[e];
         int  rc = cap[e] - fe;
         if (rc == 0) continue;

         node w = target(e);

         if(get_lock(w,lock)==0) continue;

         int dw = dist[w];
         if ( dw == dv - 1 ) 
         { //Fetch_And_Increment(&num_pushes);
           local_num_pushes++;
           int& ew = excess[w];
           //if (ew == 0) U[id].insert0(w,dw); 
           if (ew == 0) U.insert0(w,dw); 
           if (ev <= rc) 
           { ew += ev; 
             fe += ev;
             ev = 0; 
             lock[w]->unlock(); 
             break;
            }
            else
            { ew += rc; fe += rc;
              ev -= rc;
              }
           }
          else 
            if (dw < dmin) dmin = dw;

          lock[w]->unlock();
        }
     }

     if ( ev > 0 )
     { 
       forall_in_edges(e,v)
       { //Fetch_And_Increment(&num_edge_inspections);
         local_num_inspections++;
         int& fe = flow[e];
         if (fe == 0) continue;
         if (get_lock(source(e),lock) == 0) continue;

         node w = source(e);
         int dw = dist[w];
         if ( dw == dv - 1 ) 
         { Fetch_And_Increment(&num_pushes);
           local_num_pushes++;
           int& ew = excess[w];
           //if (ew == 0) U[id].insert0(w,dw); 
           if (ew == 0) U.insert0(w,dw); 
           if (ev <= fe) 
            { fe -= ev; ew += ev;
              ev = 0; 
              lock[w]->unlock();
              break;
             }
            else
             { ew += fe; ev -= fe;
               fe = 0;
              }
           }
           else 
             if (dw < dmin) dmin = dw;

           lock[w]->unlock();
         }
     }
        
     excess[v] = ev;
        
     if (ev > 0 && local_relabel) 
     { 
       //if (num_edge_inspections <= limit_heur)
       if (local_num_inspections <= limit_heur)
       { dmin++; 
         //Fetch_And_Increment(&num_relabels);
         local_num_relabels++;
         if (dmin < n)
         { dist[v] = dmin;
            //U[id].insert(v,dmin);
            U.insert(v,dmin);
          }
       }
       else 
        { lock[v]->unlock();
          break;
         }
      }
     lock[v]->unlock();
   }
   else
   {
    int dmin = MAXINT;
    edge e; 

    if (excess[v] > 0)
    { 
      forall_out_edges(e,v)
      { local_num_inspections++;
        //Fetch_And_Increment(&num_edge_inspections);
        if (cap[e] - flow[e] == 0) continue;
        if(get_locks(v, target(e), lock)==0) continue;
        node w = target(e);
        int dv = dist[v];
        int dw = dist[w];
        if ( dw == dv - 1 ) 
        { local_num_pushes++;
          //Fetch_And_Increment(&num_pushes);
          int& fe = flow[e];
          int& ev = excess[v];
          int  rc = cap[e] - flow[e];
          int& ew = excess[w];
          //if (ew == 0) U[id].insert0(w,dw); 
          if (ew == 0) U.insert0(w,dw); 

          if (ev <= rc) 
          { ew += ev;
            fe += ev;
            ev = 0; 
            lock[v]->unlock();
            lock[w]->unlock(); 
            break;
           }
           else
           { ew += rc; fe += rc;
             ev -= rc;
            }
         }
         else 
           if ( dw < dmin ) dmin = dw;
         lock[v]->unlock();
         lock[w]->unlock();
        }
    }

    if ( excess[v] > 0 )
    { 
      forall_in_edges(e,v)
      { local_num_inspections++;
        //Fetch_And_Increment(&num_edge_inspections);
        if (flow[e] == 0) continue;
        if(get_locks(v, source(e), lock)==0) continue;
        node w = source(e);
        int dv = dist[v];
        int dw = dist[w];
        if ( dw == dv - 1 ) 
        { local_num_pushes++;
          //Fetch_And_Increment(&num_pushes);
          int& fe = flow[e];
          int& ev = excess[v];
          int& ew = excess[w];
          //if (ew == 0) U[id].insert0(w,dw); 
          if (ew == 0) U.insert0(w,dw); 
          if (ev <= fe) 
          { fe -= ev; 
            ew += ev;
            ev = 0; 
            lock[v]->unlock();
            lock[w]->unlock();
            break;
           }
          else
            { ew += fe; 
              ev -= fe;
              fe = 0;
             }
         }
         else 
           if ( dw < dmin ) dmin = dw;

         lock[v]->unlock();
         lock[w]->unlock();
        }
     }
        
        
     if (excess[v] > 0 && local_relabel) 
     { 
       //if (num_edge_inspections <= limit_heur)
       if (local_num_inspections<= limit_heur)
       { dmin++; 
         local_num_relabels++;
         //Fetch_And_Increment(&num_relabels);
         if ( dmin < n)
         { lock[v]->lock();
           dist[v] = dmin;
           lock[v]->unlock();
           //U[id].insert(v,dmin);
           U.insert(v,dmin);
          }
        }
        else 
        { break; }
      }

    } //knappe_locks = 1

  } // for(;;)     

  Fetch_And_Add(&num_relabels,local_num_relabels);
  Fetch_And_Add(&num_pushes,local_num_pushes);
  Fetch_And_Add(&num_edge_inspections,local_num_inspections);
}


void* maxflow_thread(void *threadarg)
{ 
  thread_data* my_data = (thread_data *) threadarg;       
  graph& G = *(my_data->G);       

  node s = my_data->s;        
  node t = my_data->t;                

  edge_array<int>& cap = *(my_data->cap);
  edge_array<int>& flow = *(my_data->flow);
  node_array<int>& dist = *(my_data->dist);
  node_array<int>& excess = *(my_data->excess);

  //array<fifo_set>& U = *(my_data->liste);         
  fifo_set& U = *(my_data->liste);         

  int& num_pushes = *(my_data->num_pushes);
  int& num_edge_inspections = *(my_data->num_edge_inspections);
  int& num_relabels = *(my_data->num_relabels);
  int& num_global_relabels = *(my_data->num_global_relabels);

  float h = my_data->h;
  int limit_heur = my_data->limit_heur;

  b_queue<node>& Q = *(my_data->Q);
  node_array<leda_mutex*>& lock = *(my_data->lock);

  int id = my_data->id;

  bool local_relabel = my_data->local_relabel;
  bool knappe_locks = my_data->knappe_locks;

  maxflow(G,s,t,cap,flow,dist,excess,U,
          num_pushes,num_edge_inspections,num_relabels,num_global_relabels, 
          h, limit_heur, Q, lock, id, knappe_locks, local_relabel);

  return 0;
}


int main(int argc, char** argv)
{
  if (argc < 3)
  { cerr << "usage: mfc thrnum h" << endl;
    return 1;
   }

  int   thr_num = atoi(argv[1]);
  float h = atof(argv[2]);

  graph G;
  node s;
  node t;
  edge_array<int> cap(G);
  edge_array<int> flow(G);
  int num_pushes, num_edge_inspections, num_relabels, num_global_relabels;

  //int thr_num = 4;

  // Graph einlesen
  read_dimacs_mf(cin,G,s,t,cap);
  if (s == t) LEDA_EXCEPTION(1,"MAXFLOW: source == sink");

/**************** Bitte hier waehlen ! ***************************************/

  bool local_relabel(1); // mit local relabel -> local_relabel = 1; 
  bool knappe_locks(0);  // für knappere locks -> knappe_locks = 1;

  //float h = 2;           // Heuristikparameter; 
                         // falls global relabel nur erwünscht,
                         // wenn Listen der Threads leer sind, 
                         // dann setze h = MAXINT;

/*****************************************************************************/

  //for(thr_num=1; thr_num <= 4; thr_num *= 2)
  {
  float t3 = used_time(); 
  float t4 = real_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges(); 
  fifo_set U; 

  flow.init(G,0);
  node_array<int>  excess(G,0);

  if (G.outdeg(s) == 0) return 0;
 
  node_array<leda_mutex*> lock(G);
  node v;
  forall_nodes(v,G) lock[v] = new leda_mutex;

  // saturate all edges leaving s

  edge e;
  forall_out_edges(e,s) 
  { int c = cap[e];
    if (c == 0) continue;
    node v = target(e);
    flow[e] = c;
    excess[s] -= c;
    excess[v] += c;
   } 

 
  b_queue<node> Q(n);
  list<node> S;


  int limit_heur  = (int) (h*m)/thr_num;
  
  node_array<int> dist(G);

  compute_dist_t(G,t,flow,cap,excess,dist,U,Q);

  num_relabels = num_pushes = num_edge_inspections = 0;
  num_global_relabels  = 0;


  while(1)
  {

    if(U.size() <= thr_num)
    {
      int id = 0;                       
/*
      array<fifo_set> F(thr_num);
      U.split0(id, F, dist);
*/
      maxflow(G, s, t, cap, flow, dist, excess, U, num_pushes, 
              num_edge_inspections, num_relabels, num_global_relabels, h, 
              limit_heur, Q, lock, id, knappe_locks, local_relabel);
      U.clear();
     }
     else
     {
       array<fifo_set> F(thr_num);
       U.split(thr_num, F, dist);

       // Threads
       // thread_data thread_data_array[thr_num];

       array<thread_data> thread_data_array(thr_num);
       thread_t* thr = new thread_t[thr_num];


       // Threadargumente
       for(int i=0; i<thr_num; i++) {
        thread_data_array[i].G = &G;
        thread_data_array[i].s = s;
        thread_data_array[i].t = t;

        thread_data_array[i].cap = &cap;
        thread_data_array[i].flow = &flow;
        thread_data_array[i].dist = &dist;
        thread_data_array[i].excess = &excess;
        //thread_data_array[i].liste = &F;
        thread_data_array[i].liste = &F[i];

        thread_data_array[i].num_pushes = &num_pushes;
        thread_data_array[i].num_edge_inspections = &num_edge_inspections;
        thread_data_array[i].num_relabels = &num_relabels;
        thread_data_array[i].num_global_relabels = &num_global_relabels;

        thread_data_array[i].h = h;
        thread_data_array[i].limit_heur = limit_heur;

        thread_data_array[i].Q = &Q;
        thread_data_array[i].lock = &lock;
        thread_data_array[i].id = i;
        thread_data_array[i].local_relabel = local_relabel;
        thread_data_array[i].knappe_locks = knappe_locks;
       }


      for(int i=0; i<thr_num; i++) 
          thr[i] = thread_create(maxflow_thread,(void*)&thread_data_array[i]);

      for(int i=0; i<thr_num; i++) thread_join(thr[i]);
    }


    num_global_relabels++;
    assert(U.empty());
    compute_dist_t(G,t,flow,cap,excess,dist,U,Q);

    if ( U.empty() ) break;
  }


   float time_complete = real_time(t4);

   cout << endl;
   cout << string("threads: %2d h = %.1f ",thr_num,h) << endl;
   cout << string("flow: %6d ",excess[t]);
   cout << string ("time: %5.2f  cpu: %5.2f sec", time_complete, used_time(t3));
   cout << endl;
   
   int ops = num_relabels + num_pushes + num_edge_inspections;
   
   cout << string("pushes: %7d ",  num_pushes);
   cout << string("relabels: %4d ", num_relabels);
   cout << string("inspects: %7d ", num_edge_inspections);
   cout << string("updates: %2d ",  num_global_relabels);
   cout << string("ops/sec: %6d ",  int(ops/time_complete)) << endl;
   
   
   forall_nodes(v,G) delete lock[v]; 
  }

  return 0;
}
