

using namespace leda;

using std::cout;
using std::endl;


template <class graph_t>
class  STRONG_COMPONENTS_KOSARAJU {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int   count1;
int   count2;
float cputime;



template <class compnum_array>
void dfs_comp(const graph_t& G, node v, compnum_array& compnum, b_stack<node>& S)
{ compnum[v] = -1;
  edge e;
  forall_out_edges(e,v) 
  { node w = G.target(e);
    if (compnum[w] == 0) dfs_comp(G,w,compnum,S);
   }
  S.push(v);
} 


template <class compnum_array>
void dfs(const graph_t& G, node v, node* A, node** first_in,
                                            compnum_array& compnum, int k)
{ compnum[v] = k; 
  int i = G.index(v);
  node* stop = first_in[i+1];
  for(node* p = first_in[i]; p< stop; p++)
  { node w = *p;
    if (compnum[w] == -1) dfs(G,w,A,first_in,compnum,k);
   }
 }


public:


template <class compnum_array>
void stage1(const graph_t& G, compnum_array& compnum)
{ 
  float t = used_time();

  int n = G.number_of_nodes();
  b_stack<node> S(n);

  node v;
  forall_nodes(v,G) compnum[v] = 0;

  forall_nodes(v,G)
    if (compnum[v] == 0) dfs_comp(G,v,compnum,S);

  cputime = used_time(t);
}


template <class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{ 
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<node> S(n);

  node* A = new node[m];
  node** first_in = new node*[n+1];


  node v;
  forall_nodes(v,G) 
  { compnum[v] = 0;
    first_in[G.index(v)] = A;
   }

  forall_nodes(v,G) 
  { edge e;
    forall_out_edges(e,v)
    { node w = G.target(e);
      first_in[G.index(w)]++;
     }
   }

  int d = 0;
  forall_nodes(v,G)  
  { int i = G.index(v);
    first_in[i] += d;
    d = (first_in[i]-A);
  }

  first_in[n] = A+m;

  forall_nodes(v,G) 
  { edge e;
    forall_out_edges(e,v)
    { node w = G.target(e);
      node* p = first_in[G.index(w)];
      *--p = v;
      first_in[G.index(w)] = p;
    }
   }


  forall_nodes(v,G)
    if (compnum[v] == 0) dfs_comp(G,v,compnum,S);

  int count = 0;

  while (!S.empty())
  { node u = S.pop();
    if (compnum[u] == -1) dfs(G,u,A,first_in,compnum,count++);
   }

 cputime = used_time(t);

 return count;
   
}


template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};



