#include "my_boost.h"
#include "cm_rec.h"
#include "cm_nonrec.h"
#include "tarjan_rec.h"
#include "tarjan_nonrec.h"

#include "dfsnum.h"

#include <iomanip>

using namespace std;


int main(int argc, char* argv[])
{
  typedef boost::adjacency_list<boost::vecS,boost::vecS,boost::directedS> Graph;
  typedef graph_traits <Graph> traits;

  typedef traits::vertex_iterator   node;
  typedef traits::out_edge_iterator edge;

  typedef boost_graph<Graph> graph;

  graph G;

  int n; cout << "n =  "; cin >> n;
  int m; cout << "m =  "; cin >> m;

  //acyclic_dense_graph(G, n);
  random_graph(G,n,m);

/*
  vector<int> compNum0(n);
  vector<int> compNum(n);
*/
  node_array<int,graph> compNum0(G);
  node_array<int,graph> compNum(G);


  cout <<  fixed << setw(5) << setprecision(2) << right;


  float T = used_time();

  int scc0 = boost::strong_components(G, 
            make_iterator_property_map(compNum0.begin(),get(vertex_index, G)));

  cout << endl;
  cout << "boost:   scc0 = " << scc0 << "   time = " << used_time(T) << endl; 
  cout << endl;


  cout << endl;
  cout << "      Cheriyan/Mehlhorn                 Tarjan                  DFS";
  cout << endl;
  cout << " orig  rec1  rec2  non1  non2    orig  rec1  rec2  non1  non2";
  cout << endl;
  cout << endl;



{
  STRONG_COMPONENTS_CM0<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_CM1<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_CM2<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_CM_NR1<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}


{
  STRONG_COMPONENTS_CM_NR2<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

  cout << "  ";

{
  STRONG_COMPONENTS_TARJAN0<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_TARJAN1<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_TARJAN2<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_TARJAN_NR1<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

{
  STRONG_COMPONENTS_TARJAN_NR2<graph> SCC;
  int scc = SCC(G,compNum);
  cout << setw(5) << SCC.cpu_time() << " " << flush;
  assert(scc == scc0);
  node v;
  forall_nodes(v,G) assert(compNum[v] == compNum0[v]);
}

  cout << " " << flush;

{
  DFS_NUM<graph> dfs;
  int s = dfs(G,compNum);
  cout << setw(5) << dfs.cpu_time() << " " << flush;
}


  cout << endl;


  return 0;
}

 
