
//------------------------------------------------------------------------------


template <class graph_t>
class  STRONG_COMPONENTS_CM0 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int   dfs_count;
int   scc_count;

float cputime;


template<class compnum_array>
void dfs(const graph_t& G, node v, compnum_array& compnum,
                                   node_array<int,graph_t>& dfsnum,
                                   b_stack<node>& unfinished,
                                   node_array<int,graph_t>& in_unfinished,
                                   b_stack<node>& roots)
{
  node w;

  dfsnum[v] = ++dfs_count;
  in_unfinished[v] = true;
  unfinished.push(v);
  roots.push(v);

  edge e;
  forall_out_edges(e,v)
    { w = G.target(e);
      if (dfsnum[w]==-1) 
       dfs(G,w,compnum,dfsnum,unfinished,in_unfinished,roots);
      else 
       if (in_unfinished[w])
        while (dfsnum[roots.top()]>dfsnum[w])  roots.pop();
     }

  if (v==roots.top())
  { node u;
    do { u=unfinished.pop();
         in_unfinished[u] = false;
         compnum[u] = scc_count;
       } while (u != v);
     scc_count++;
     roots.pop(); 
    }
}


public:

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<node> roots(n);
  b_stack<node> unfinished(n);
  node_array<int,graph_t> in_unfinished(G,false);
  node_array<int,graph_t> dfsnum(G,-1);

  dfs_count = 0; 
  scc_count = 0;

  node v;
  forall_nodes(v,G) 
     if (dfsnum[v] == -1) 
        dfs(G,v,compnum,dfsnum,unfinished,in_unfinished,roots);

  cputime = used_time(t);
  return scc_count;
}

template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};



//------------------------------------------------------------------------------


template <class graph_t>
class  STRONG_COMPONENTS_CM1 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int dfs_count;
int scc_count;

float cputime;



template<class compnum_array>
void dfs(const graph_t& G, node v, compnum_array& compnum,
                                   b_stack<node>& unfinished,
                                   b_stack<int>& roots)
{ 
  int dfsnum = --dfs_count;

  compnum[v] = dfsnum;
  unfinished.push(v);
  roots.push(dfsnum);

  edge e;
  forall_out_edges(e,v)
  { node w =G.target(e);
    int d = compnum[w];
    if (d == -1) 
      dfs(G,w,compnum,unfinished,roots);
    else 
      if (d < -1)
         while (roots.top() < d)  roots.pop();
   }

  if (roots.top() == dfsnum) 
  { node u;
    do { u = unfinished.pop(); // u is a node of the scc with root v 
         compnum[u] = scc_count;
        } while (v != u);
    roots.pop(); 
    scc_count++; 
   }
}


public:

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<int>  roots(n);
  b_stack<node> unfinished(n);

  dfs_count = -1; 
  scc_count = 0;

  node v;
  forall_nodes(v,G) compnum[v] = -1; 

  forall_nodes(v,G) 
    if (compnum[v] == -1) 
       dfs(G,v,compnum,unfinished,roots);

  cputime = used_time(t);
  return scc_count;
}

template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};


//------------------------------------------------------------------------------


template <class graph_t>
class  STRONG_COMPONENTS_CM2 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int dfs_count;
int scc_count;

float cputime;



template<class compnum_array>
void dfs(const graph_t& G, node v, compnum_array& compnum,
                                   b_stack<node>& edgeStack,
                                   b_stack<node>& unfinished,
                                   b_stack<int>&  roots)
{ 
  int dfsnum = --dfs_count;

  compnum[v] = dfsnum;
  roots.push(dfsnum);
  unfinished.push(v);

  int sz = edgeStack.size();  // remember current size of edgeStack 

  edge e;
  forall_rev_out_edges(e,v) edgeStack.push(G.target(e));
  
  while (edgeStack.size() > sz)
  { node w = edgeStack.pop();
    int  d = compnum[w];

    if (d >= 0) continue;

    if (d == -1) 
      dfs(G,w,compnum,edgeStack,unfinished,roots);
    else 
      while (roots.top() < d) roots.pop();
   }


  if (roots.top() == dfsnum) 
  { node u;
    do { u = unfinished.pop(); // u is a node of the scc with root v 
         compnum[u] = scc_count;
        } while (v != u);
    roots.pop(); 
    scc_count++; 
   }
}


public:

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<node> edgeStack(m+1);
  b_stack<int>  roots(n);
  b_stack<node> unfinished(n);

  dfs_count = -1; 
  scc_count = 0;

  node v;
  forall_nodes(v,G) compnum[v] = -1; 

  forall_nodes(v,G) 
    if (compnum[v] == -1) 
       dfs(G,v,compnum,edgeStack,unfinished,roots);

  cputime = used_time(t);
  return scc_count;
}

template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};



//------------------------------------------------------------------------------

// prefetching

template <class graph_t>
class  STRONG_COMPONENTS_CM3 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int dfs_count;
int scc_count;

float cputime;



template<class compnum_array>
void dfs(const graph_t& G, node v, compnum_array& compnum,
                                   node*  edgeTop,
                                   b_stack<node>& unfinished,
                                   b_stack<int>&  roots)
{ 
  int dfsnum = --dfs_count;

  compnum[v] = dfsnum;
  roots.push(dfsnum);
  unfinished.push(v);

  node* p = edgeTop;  // remember current size of edgeStack 

  edge e;
  forall_rev_out_edges(e,v) *++edgeTop = G.target(e);
  
  while (edgeTop > p)
  { node w = *edgeTop--;

    node prefv = edgeTop[-4];
__builtin_prefetch(&compnum[prefv]);

    int  d = compnum[w];

    if (d >= 0) continue;

    if (d == -1) 
      dfs(G,w,compnum,edgeTop,unfinished,roots);
    else 
      while (roots.top() < d) roots.pop();
   }


  if (roots.top() == dfsnum) 
  { node u;
    do { u = unfinished.pop(); // u is a node of the scc with root v 
         compnum[u] = scc_count;
        } while (v != u);
    roots.pop(); 
    scc_count++; 
   }
}


public:

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  node* edgeStack = new node[n+m];
  node* edgeTop = edgeStack-1;

  b_stack<int>  roots(n);
  b_stack<node> unfinished(n);

  dfs_count = -1; 
  scc_count = 0;

  node v;
  forall_nodes(v,G) compnum[v] = -1; 

  forall_nodes(v,G) 
    if (compnum[v] == -1) 
       dfs(G,v,compnum,edgeTop,unfinished,roots);

  delete[] edgeStack;
  cputime = used_time(t);
  return scc_count;
}

template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};


//------------------------------------------------------------------------------


template <class graph_t>
class  STRONG_COMPONENTS_CM_NR1 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

typedef two_tuple<node,edge> stack_el;

float cputime;

public:


template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{ 
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<node> nStack(n);
  b_stack<edge> eStack(n);

  b_stack<int>      roots(n);
  b_stack<node>     unfinished(n);

  int count     = -1;   // dfs numbers:  -2, -3, ... 
  int scc_count =  0;   // scc numbers:   0,  1, ...

  node v;
  forall_nodes(v,G) compnum[v] = -1;  // mark unvisited

  node u;
  forall_nodes(u,G)
  { 
    if (compnum[u] != -1) continue;

    unfinished.push(u);
    roots.push(--count);
    compnum[u] = count;

    node v = u;
    edge e = G.first_out_edge(u);


    for(;;)
    { 
#if defined(BOOST)
      if (e == G.stop_out_edge(v))     // all edges scanned: finish v
#else
      if (e == v->stop_out_edge())
#endif
      { if (compnum[v] == roots.top()) 
        { node u;
          do { u = unfinished.pop(); // u is a node of the scc with root v 
               compnum[u] = scc_count;
             } while (v != u);
          roots.pop(); 
          scc_count++;
         }
   
        if (nStack.empty())
           break;
        else
         { v = nStack.pop();
           e = eStack.pop();
           continue;
          }
       }

      node w = G.target(e);
      int d = compnum[w];

      if (d == -1)  // tree edge (v,w)
        { compnum[w] = --count;
          roots.push(count);
          unfinished.push(w);
          nStack.push(v);
          eStack.push(G.next_out_edge(e));
          v = w;
          e = G.first_out_edge(w);
         }
       else
        { if (d < -1) while (roots.top() < d)  roots.pop();
          e = G.next_out_edge(e);
         }
    } 
  }

  cputime = used_time(t);
  return scc_count;
}



template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};


//------------------------------------------------------------------------------

template <class graph_t>
class  STRONG_COMPONENTS_CM_NR1a {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

typedef two_tuple<node,edge> stack_el;

float cputime;

public:


template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{ 
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<stack_el> S(n+1);
  b_stack<int>      roots(n);
  b_stack<node>     unfinished(n);

  int count     = -1;   // dfs numbers:  -2, -3, ... 
  int scc_count =  0;   // scc numbers:   0,  1, ...

  node v;
  forall_nodes(v,G) compnum[v] = -1;  // mark unvisited

  node u;
  forall_nodes(u,G)
  { 
    if (compnum[u] != -1) continue;

    unfinished.push(u);
    roots.push(--count);
    compnum[u] = count;

    node v = u;
    edge e = G.first_out_edge(u);


    for(;;)
    { 
#if defined(BOOST)
      if (e == G.stop_out_edge(v))     // all edges scanned: finish v
#else
      if (e == v->stop_out_edge())
#endif
      { if (compnum[v] == roots.top()) 
        { node u;
          do { u = unfinished.pop(); // u is a node of the scc with root v 
               compnum[u] = scc_count;
             } while (v != u);
          roots.pop(); 
          scc_count++;
         }
   
        if (S.empty())
           break;
        else
         { stack_el p = S.pop();
           v = p.first();
           e = p.second();
           continue;
          }
       }

      node w = G.target(e);
      int d = compnum[w];

      if (d == -1)  // tree edge (v,w)
        { compnum[w] = --count;
          roots.push(count);
          unfinished.push(w);
          S.push(stack_el(v,G.next_out_edge(e)));
          v = w;
          e = G.first_out_edge(w);
         }
       else
        { if (d < -1) while (roots.top() < d)  roots.pop();
          e = G.next_out_edge(e);
         }
    } 
  }

  cputime = used_time(t);
  return scc_count;
}



template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};


//------------------------------------------------------------------------------


template<class graph_t>
class  STRONG_COMPONENTS_CM_NR2 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;


float cputime;

public:

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<int>  dfsnumStack(n);
  b_stack<node> edgeStack(2*n+m);

  b_stack<node> unfinished(n);
  b_stack<int>  roots(n);

  int count1 = -1;  // dfsnumbers  -2, -3, ...
  int count2 =  0;  // scc numbers  0, 1, 2, ... 

  node v;
  forall_nodes(v,G) compnum[v] = -1; // unvisited

  forall_nodes(v,G)
  {
    if (compnum[v] != -1) continue;

    int dfsnum = --count1;
    compnum[v] = dfsnum;
    roots.push(dfsnum);
    unfinished.push(v);

    edgeStack.push(G.stop_node());
    edge e;
    forall_rev_out_edges(e,v) edgeStack.push(G.target(e));

    for(;;)
    { 
      node w = edgeStack.pop();

      if (w == G.stop_node())
      {
        if (roots.top() == dfsnum)
        { int d = 0;
          while (d != dfsnum)
          { node u = unfinished.pop();
            d = compnum[u];
            compnum[u] = count2;
           }
          roots.pop();
          count2++;
        }

        if (edgeStack.empty()) break;

        dfsnum = dfsnumStack.pop();
        continue;
      }

      int d = compnum[w];

      if (d == -1)
      { dfsnumStack.push(dfsnum);
        dfsnum = --count1;
        compnum[w] = dfsnum;
        roots.push(dfsnum);
        unfinished.push(w);
        edgeStack.push(G.stop_node());
        edge e;
        forall_rev_out_edges(e,w) edgeStack.push(G.target(e));
        continue;
       }

      if (d < -1) while (roots.top() < d) roots.pop();
    }
  }

  cputime = used_time(t);

  return count2;
}


template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};




//------------------------------------------------------------------------------

#define    FLAG(x) node((unsigned long)x | 0x80000000)
#define  UNFLAG(x) node((unsigned long)x & 0x7fffffff)
#define FLAGGED(x) ((unsigned long)x & 0x80000000)



template<class graph_t>
class  STRONG_COMPONENTS_CM_NR3 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;


float cputime;

public:


template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<int>  dfsnumStack(n);
  b_stack<node> edgeStack(n+m);

  b_stack<node> unfinished(n);
  b_stack<int>  roots(n);

  int count1 = -1;  // dfsnumbers  -2, -3, ...
  int count2 =  0;  // scc numbers  0, 1, 2, ... 

  node v;
  forall_nodes(v,G) compnum[v] = -1; // unvisited

  forall_nodes(v,G)
  {
    if (compnum[v] != -1) continue;

    int dfsnum = --count1;
    compnum[v] = dfsnum;

    roots.push(dfsnum);
    unfinished.push(v);
    edgeStack.push(FLAG(v));

    edge e;
    forall_rev_out_edges(e,v) edgeStack.push(G.target(e));

    for (;;)
    { node w = edgeStack.pop();

      if (FLAGGED(w))
      {
        if (roots.top() == dfsnum)
        { w = UNFLAG(w);
          node u;
          do { u = unfinished.pop();
               compnum[u] = count2;
             } while (w != u);
          roots.pop();
          count2++;
        }

        if (edgeStack.empty()) break;

        dfsnum = dfsnumStack.pop();
        continue;
      }


      int d = compnum[w];

      if (d == -1)
      { dfsnumStack.push(dfsnum);
        dfsnum = --count1;          
        compnum[w] = dfsnum;
        roots.push(dfsnum);
        unfinished.push(w);
        edgeStack.push(FLAG(w));
        edge e;
        forall_rev_out_edges(e,w) edgeStack.push(G.target(e));
        continue;
       }

      if (d < -1) while (roots.top() < d) roots.pop();
    }
  }

  cputime = used_time(t);

  return count2;
}


template <class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

float cpu_time() const { return cputime; }

};



