#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/core/b_stack.h>
#include <LEDA/core/array.h>
#include <LEDA/core/tuple.h>
#include <assert.h>

#if !defined(forall_rev_out_edges)
#define forall_rev_out_edges(e,v) \
for(void* loop_stop = ((e = (v+1)->first_out_edge()-1), v->first_out_edge()-1);\
    e != loop_stop; e = e-1)
#endif



using namespace leda;

#include "cm_rec.h"
#include "cm_nonrec.h"
#include "tarjan_rec.h"
#include "tarjan_nonrec.h"
#include "kosaraju.h"
#include "dfsnum.h"



using std::cout;
using std::flush;
using std::endl;



void acyclic_dense_graph(graph& G, int n)
{ array<node> V(n);
  for(int i=0; i<n; i++) V[i] = G.new_node();

  for(int i=0; i<n; i++) 
     for(int j=i+1; j<n; j++) 
        G.new_edge(V[i],V[j]);
}
  

void SCC_generator1(graph& G, int n, int m)
{ 
  array<node> V(n);

  for (int i = 0; i < n; i++) V[i] = G.new_node();

  int k = m/n;

  for (int i = 1; i < n; i++) G.new_edge(V[0],V[i]);

  for (int i = 1; i <= k+1; i++) G.new_edge(V[i],V[0]);

  for (int i = k+2; i < n; i++) 
    for (int j = 0; j < k; j++) 
       G.new_edge(V[i],V[j + 1]);
}

void SCC_generator2(graph& G, int n, int m)
{ 
  array<node> V(n);

  for (int i = 0; i < n; i++) V[i] = G.new_node();

  int k = m/n;

  for (int i = 1; i < n; i++) G.new_edge(V[0],V[i]);

  for (int i = 1; i <= k+1; i++) G.new_edge(V[i],V[0]);

  for (int i = k+2; i < n; i++) 
    for (int j = 0; j < k; j++) 
       G.new_edge(V[i],V[k - j]);
}



int main() 
{

  //int d = read_int("d = ");
  //int d = 10;


int m = 1 << 23;   // 8 Mio

cout << endl;
cout << "m = " << m << endl;
cout << endl;

  cout << "   n     GCM0  GCMrc GCMnr GTa0  GTarc GTanr GDFS  GKos" << endl;

for (double f = m;  f*(f-1) >= m;  f/=2) 
{
    int n = int(f);

/*
for(int n = 100000; n < 2000000; n += 100000)
{ 
  int m = d*n;
*/

  graph G;
  random_graph(G,n,m);

  node_array<int> compnum0(G);   // for comparing results
  STRONG_COMPONENTS_CM0<graph> scc0;
  int scc_num = scc0(G,compnum0);

  node_array<int> compnum(G);


  cout << string("%7d ",n) << flush;

{
  STRONG_COMPONENTS_CM0<graph> scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}

{
  STRONG_COMPONENTS_CM2<graph>     scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}

{
  STRONG_COMPONENTS_CM_NR2<graph>  scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}


{
  STRONG_COMPONENTS_TARJAN0<graph>    scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}

{
  STRONG_COMPONENTS_TARJAN2<graph>    scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}


{
  STRONG_COMPONENTS_TARJAN_NR2<graph> scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  node v;
  forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  assert(s == scc_num);
}


{
  DFS_NUM<graph> dfs;
  int s = dfs(G,compnum);
  cout << string("%5.2f ", dfs.cpu_time()) << flush;
}

{
  STRONG_COMPONENTS_KOSARAJU<graph> scc;
  int s = scc(G,compnum);
  cout << string("%5.2f ", scc.cpu_time()) << flush;
  assert(s == scc_num);
}

  cout << endl;
}

  return 0;
}


