#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/static_graph.h>
#include <LEDA/graph/node_slot.h>
#include <LEDA/core/b_stack.h>
#include <LEDA/core/tuple.h>
#include <assert.h>


#if !defined(forall_rev_out_edges)
#define forall_rev_out_edges(e,v) \
for(void* loop_stop = ((e = (v+1)->first_out_edge()-1), v->first_out_edge()-1);\
    e != loop_stop; e = e-1)
#endif


using namespace leda;

#include "cm_rec.h"
#include "cm_nonrec.h"
#include "tarjan_rec.h"
#include "tarjan_nonrec.h"



using std::cout;
using std::flush;
using std::endl;

#include <time.h>

// elapsed CPU time see also /usr/include/sys/time.h
inline double cpuTime()
{ //struct timespec tp;

  return clock() * 1e-6;
//  clock_gettime(CLOCK_VIRTUAL, &tp);
//  return tp.tv_sec + tp.tv_nsec * 1e-9;
}



template <class st_graph>
void construct_static_graph(st_graph& Gs, const graph& G) 
{ 
  // constructs a static copy of G

  typedef typename st_graph::node st_node;
  typedef typename st_graph::edge st_edge;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  Gs.start_construction(n,m);
  node_array<st_node> N(G);

  node v;
  forall_nodes(v,G) N[v] = Gs.new_node();

  forall_nodes(v,G)
  { st_node x = N[v];
    for (edge e = G.first_out_edge(v); e; e = G.next_out_edge(e))
    { st_node y = N[G.target(e)]; 
      Gs.new_edge(x,y);
     }
   }
  Gs.finish_construction();
}


#define try(r,f) double time=cpuTime();\
for (int i=0;  i < r; i++) f;\
time = cpuTime() - time;\
cout << string(" %5.2f",time) << flush;


int main(int argc, char** argv) 
{
  int m, repeat=1;

  if (argc < 2) 
  { repeat = 1;
    m = read_int("m = ");
   }
  else
  { 
    m = atoi(argv[1]);
    repeat = atoi(argv[2]);
  }

  cout << "#  m      n     rep CMorig CMrec1 CMrec2 CMnr1 CMnr2 Torig Trec1 Trec2 Tnr1 Tnr2" << endl;

  for (double f = m;  f*(f-1) >= m;  f/=2) {

    int n = int(f);

    cout << string("%8d %8d %2d", m, n, repeat) << flush;

    graph GG;
    random_graph(GG,n,m);


    typedef static_graph<directed_graph,data_slots<1>,data_slots<0> > st_graph;
    typedef st_graph::node node;
    typedef st_graph::edge edge;

    st_graph G;
    construct_static_graph(G,GG);


    node_array<int,st_graph> compnum0(G);   // for comparing results
    STRONG_COMPONENTS_CM0<st_graph> scc0;
    scc0(G,compnum0);

    node_slot<int,st_graph,0> compnum(G);

  {
    STRONG_COMPONENTS_CM0<st_graph> scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_CM1<st_graph>     scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }

  {
    STRONG_COMPONENTS_CM2<st_graph>     scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_CM_NR1<st_graph>  scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_CM_NR2<st_graph>  scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_TARJAN0<st_graph>    scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }

  {
    STRONG_COMPONENTS_TARJAN1<st_graph>    scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_TARJAN2<st_graph>    scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }


  {
    STRONG_COMPONENTS_TARJAN_NR1<st_graph> scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
  }

  {
    STRONG_COMPONENTS_TARJAN_NR2<st_graph> scc;
    try(repeat, scc(G,compnum));
    node v;
    forall_nodes(v,G) assert(compnum0[v] == compnum[v]);
   }


    cout << endl;
  }
  cout << endl;

  return 0;
}


