#define LEDA_CHECKING_OFF

#include <LEDA/system/basic.h>
#include <LEDA/core/array.h>
#include <LEDA/core/basic_alg.h>
#include <LEDA/core/quicksort.h>

#include <LEDA/core/b_stack.h>

#include <assert.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::endl;
using std::flush;
#endif



template<class E, class SMALLER>
void quicksort_non_rec0(E* A, int n, const SMALLER& smaller)
{ 
  if (n <= 1) return;

  b_stack<int> P(n);

  P.push(n);

  int l = 0;
  int r;

  while (!P.empty())
  { 
    r = P.top();

    if (r <= l+1)
    { P.pop();
      l = r+1; 
      continue;
     }

    E  s = A[l];
    int i = l;
    int j = r;

    for(;;)
    { while (smaller(A[++i],s));
      while (smaller(s,A[--j]));
      if (i >= j) break;
      swap(A[i],A[j]);
     }
    swap(A[l],A[j]);
    P.push(j);
  }
}


#define CONTINUE { l = r+2; P.pop(); continue; }


template<class E, class SMALLER>
void quicksort_non_rec1(E* first, E* last, const SMALLER& smaller)
{ 
  int n = last-first+1;

  if (n <= 1) return;

  b_stack<E*> P(1000);

  P.push(last+1);

  E* l = first;

  while (!P.empty())
  { 
    E* r = P.top()-1;

    if (r <= l) CONTINUE;

    if (smaller(*r,*l)) swap(*l,*r);

    if (r == l+1) CONTINUE;

    E* k = l + (r-l)/2;

    if (smaller(*k,*l))
       swap(*l,*k);
    else
       if (smaller(*r,*k)) swap(*k,*r);

    if (r == l+2) CONTINUE;



    E s = *k;
    E* i = l+1;
    *k = *i;
    *i = s;

    for(;;)
    { while (smaller(*++i,s));
      while (smaller(s,*--r));
      if (i >= r) break;
      swap(*i,*r);
     }

    *(l+1) = *r;
    *r = s;
    P.push(r);
  }
}


#define CONTINUE2 { while (l < r+2) *out++ = *l++; P.pop(); continue; }

template<class E, class SMALLER>
void quicksort_non_rec2(E* first, E* last, const SMALLER& smaller)
{ 
  int n = last-first+1;

  if (n <= 1) return;

  b_stack<E*> P(1000);

  P.push(last+1);
  
  E* out=first;

  E* l = first;

  while (!P.empty())
  { 
    E* r = P.top()-1;

    if (r <= l) CONTINUE2;

    if (smaller(*r,*l)) swap(*l,*r);

    if (r == l+1) CONTINUE2;

    E* k = l + (r-l)/2;

    if (smaller(*k,*l))
       swap(*l,*k);
    else
       if (smaller(*r,*k)) swap(*k,*r);

    if (r == l+2) CONTINUE2;


    E s = *k;
    E* i = l+1;
    *k = *i;
    *i = s;

    for(;;)
    { while (smaller(*++i,s));
      while (smaller(s,*--r));
      if (i >= r) break;
      swap(*i,*r);
     }

    *(l+1) = *r;
    *r = s;
    P.push(r);
  }
}


#define CONTINUE3 { l = r+2; P.pop(); continue; }

template<class E, class SMALLER>
void quicksort_non_rec3(E* first, E* last, const SMALLER& smaller)
{ 
  int n = last-first+1;

  if (n <= 1) return;

  b_stack<E*> P(1000);

  P.push(last+1);
  
  E* l = first;

  while (!P.empty())
  { 
    E* r = P.top()-1;

    if (r <= l) CONTINUE3;

    if (smaller(*r,*l)) swap(*l,*r);

    if (r == l+1) CONTINUE3;

    E* k = l + (r-l)/2;

    if (smaller(*k,*l))
       swap(*l,*k);
    else
       if (smaller(*r,*k)) swap(*k,*r);

    if (r == l+2) CONTINUE3;


    E* i = l;
    E s = *k;

    for(;;)
    { while (smaller(*++i,s));
      while (smaller(s,*--r));
      if (i >= r) break;
      swap(*i,*r);
      if (k == r) k = i;
      else if (k == i) k = r;
     }

    if (k > r) r = i; 
    *k = *r;
    *r = s;
    P.push(r);
  }
}



int main () 
{
  int N = read_int("N = "); 


  array<int> A(N);
  array<int> B(N);
  array<int> C(N);

  for (int i = 0; i < N; i++) A[i] = B[i] = C[i] = rand_int(1,1000000);

  leda_smaller<int> smaller;

  float T = used_time();

  cout << "quicksort        " << flush;
  quicksort(A.first_item(),A.last_item(),smaller);
  cout << string("%5.3f sec",used_time(T)) << endl;

  cout << "quicksort_non_r1 " << flush;
  quicksort_non_rec1(B.first_item(),B.last_item(),smaller);
  cout << string("%5.3f sec",used_time(T)) << endl;

  cout << "quicksort_non_r3 " << flush;
  quicksort_non_rec3(C.first_item(),C.last_item(),smaller);
  cout << string("%5.3f sec",used_time(T)) << endl;

  for (int i = 0; i < N; i++) 
  { assert(A[i] == B[i]);
    assert(B[i] == C[i]);
   }
 
  return 0;

}


